#!/usr/bin/env node

/**
 * ZAI MCP Server - AI-to-AI Loop System with Multi-Provider Support
 * Supports OpenRouter, Anthropic, and DeepSeek APIs with automatic failover
 */

import { Server } from '@modelcontextprotocol/sdk/server/index.js';
import { StdioServerTransport } from '@modelcontextprotocol/sdk/server/stdio.js';
import { CallToolRequestSchema, ListToolsRequestSchema } from '@modelcontextprotocol/sdk/types.js';
import { MultiProviderAI } from './multiProviderAI.js';
import { ZAIDataCollector } from './data-collection-system.js';
import { AIVotingManager } from './aiVotingManager.js';
import { TaskManager } from './taskManager.js';
import { DeepThinking } from './deepThinking.js';
import { ParallelImplementation } from './parallelImplementation.js';
import { AIAgentCoordinator } from './aiAgentCoordinator.js';
import { SmartCaching } from './smartCaching.js';
import { ProjectMemory } from './projectMemory.js';
import { AIModelAnalytics } from './aiModelAnalytics.js';
import { WorkflowTemplates } from './workflowTemplates.js';
import { RealTimeCollaboration } from './realTimeCollaboration.js';
import { AISwarmIntelligence } from './aiSwarmIntelligence.js';
import { PredictiveTaskManagement } from './predictiveTaskManagement.js';
import { RealTimeCodeGeneration } from './realTimeCodeGeneration.js';
import { AdaptiveLearningSystem } from './adaptiveLearningSystem.js';
import { EnhancedPromptLibrary } from './enhancedPromptLibrary.js';
import { InputSanitizer } from './inputSanitizer.js';

class ZAIMCPServer {
    constructor() {
        this.server = new Server(
            {
                name: 'zai-mcp-server',
                version: '2.0.0',
            },
            {
                capabilities: {
                    tools: {},
                },
            }
        );

        // Initialize AI providers, voting system, and data collection
        try {
            // Initialize Smart Caching System first
            this.smartCache = new SmartCaching({
                cacheDir: './cache',
                maxCacheSize: 1000,
                similarityThreshold: 0.85,
                costSavingsTarget: 0.7
            });

            // Initialize AI Model Performance Analytics before MultiProviderAI
            this.modelAnalytics = new AIModelAnalytics({
                analyticsDir: './analytics',
                maxHistoryEntries: 1000
            });

            this.multiAI = new MultiProviderAI(this.smartCache, this.modelAnalytics);
            this.votingManager = new AIVotingManager(this.multiAI);
            this.dataCollector = new ZAIDataCollector();

            // Initialize advanced task management components
            this.taskManager = new TaskManager(this.multiAI, this.votingManager);
            this.deepThinking = new DeepThinking(this.multiAI, this.votingManager);
            this.parallelImplementation = new ParallelImplementation(this.multiAI, this.taskManager, this.deepThinking);

            // Initialize AI Agent Coordinator for enhanced interaction management
            this.aiCoordinator = new AIAgentCoordinator(this);

            // Initialize Project Memory & Context Persistence
            this.projectMemory = new ProjectMemory({
                memoryDir: './memory',
                maxProjects: 100,
                maxContextHistory: 50
            });

            // Initialize Workflow Templates System
            this.workflowTemplates = new WorkflowTemplates({
                templatesDir: './templates',
                customTemplatesDir: './custom-templates',
                maxCustomTemplates: 50
            });

            // Initialize Real-time Collaboration System
            this.collaboration = new RealTimeCollaboration({
                collaborationDir: './collaboration',
                maxWorkspaces: 100,
                maxUsersPerWorkspace: 20,
                sessionTimeout: 30 * 60 * 1000 // 30 minutes
            });

            // Initialize AI Swarm Intelligence System
            this.swarmIntelligence = new AISwarmIntelligence({
                swarmDir: './swarm',
                maxActiveSwarms: 10,
                agentTimeout: 5 * 60 * 1000, // 5 minutes
                coordinationStrategy: 'democratic'
            });

            // Initialize Predictive Task Management System
            this.predictiveTaskManagement = new PredictiveTaskManagement({
                predictiveDir: './predictive',
                maxHistoryEntries: 1000,
                predictionAccuracyThreshold: 0.75,
                riskAssessmentInterval: 60000 // 1 minute
            });

            // Initialize Real-time Code Generation & Review System
            this.realTimeCodeGeneration = new RealTimeCodeGeneration({
                codeGenDir: './codegen',
                maxSuggestions: 5,
                analysisInterval: 1000, // 1 second
                maxCodeHistory: 100
            });

            // Initialize Adaptive Learning System
            this.adaptiveLearningSystem = new AdaptiveLearningSystem({
                learningDir: './learning',
                maxLearningEntries: 10000,
                adaptationThreshold: 0.7,
                learningRate: 0.1,
                personalityUpdateInterval: 24 * 60 * 60 * 1000 // 24 hours
            });

            // Initialize Enhanced Prompt Library
            this.enhancedPromptLibrary = new EnhancedPromptLibrary({
                libraryDir: './prompt-library',
                maxPrompts: 10000,
                maxVersions: 10,
                abTestDuration: 7 * 24 * 60 * 60 * 1000, // 7 days
                communityModerationThreshold: 0.8
            });

            // Initialize Input Sanitizer
            this.inputSanitizer = new InputSanitizer();
        } catch (error) {
            console.error('❌ Failed to initialize components:', error);
            throw error;
        }
        
        // Active loops storage
        this.activeLoops = new Map();
        this.loopCounter = 0;

        // Agent state management
        this.agentBusy = false;
        this.pendingResponses = [];

        // Initialize Strict Acknowledgment System
        this.acknowledgmentSystem = {
            pendingAcknowledgments: new Map(),
            strictMode: true,
            acknowledgmentTimeout: 30000, // 30 seconds
            maxRetries: 3,
            blockedOperations: new Set(),
            lastAcknowledgmentTime: Date.now(),
            acknowledgmentHistory: [],
            requireAcknowledgmentBeforePrompts: true,
            promptQueue: [],
            acknowledgmentRequired: false,
            currentLoopId: null
        };

        // Start acknowledgment monitoring
        this.startAcknowledgmentMonitoring();

        this.setupToolHandlers();
        this.setupErrorHandling();

        // 🆓 FREE VERSION - No license validation required
        console.error('🆓 ZAI MCP Server - FREE VERSION');
        console.error('📊 Data collection enabled for AI training');
        console.error('💡 Help us improve AI by using this free service!');
    }

    /**
     * Start acknowledgment monitoring system
     */
    startAcknowledgmentMonitoring() {
        // Check for pending acknowledgments every 5 seconds
        setInterval(() => {
            this.checkPendingAcknowledgments();
        }, 5000);

        // Check for acknowledgment timeout every 30 seconds
        setInterval(() => {
            this.checkAcknowledgmentTimeout();
        }, 30000);

        console.error('🔒 Strict Acknowledgment System initialized');
        console.error('⚠️  AI prompts will be blocked until proper acknowledgment');
    }

    /**
     * Check for pending acknowledgments
     */
    checkPendingAcknowledgments() {
        const now = Date.now();
        for (const [loopId, ackData] of this.acknowledgmentSystem.pendingAcknowledgments) {
            const timePending = now - ackData.timestamp;

            if (timePending > this.acknowledgmentSystem.acknowledgmentTimeout) {
                console.error(`🚨 ACKNOWLEDGMENT TIMEOUT: Loop ${loopId} - ${Math.round(timePending/1000)}s without acknowledgment`);
                console.error('🔒 AI prompts BLOCKED until acknowledgment received');

                // Block all AI operations for this loop
                this.acknowledgmentSystem.blockedOperations.add(loopId);
            }
        }
    }

    /**
     * Check for overall acknowledgment timeout
     */
    checkAcknowledgmentTimeout() {
        const now = Date.now();
        const timeSinceLastAck = now - this.acknowledgmentSystem.lastAcknowledgmentTime;

        if (timeSinceLastAck > 60000 && this.acknowledgmentSystem.acknowledgmentRequired) { // 1 minute
            console.error('🚨 CRITICAL: No acknowledgment received for over 1 minute');
            console.error('🔒 ALL AI OPERATIONS BLOCKED until acknowledgment');
            console.error('📋 Please use acknowledge_agent_response tool to continue');

            // Block all operations
            this.acknowledgmentSystem.strictMode = true;
        }
    }

    setupToolHandlers() {
        // List available tools
        this.server.setRequestHandler(ListToolsRequestSchema, async () => ({
            tools: [
                {
                    name: 'activate_infinite_loop',
                    description: 'Start AI-to-AI improvement loops for continuous development',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            message: {
                                type: 'string',
                                description: 'Activation message with topic (format: "actloop [topic]")'
                            },
                            aiToAi: {
                                type: 'boolean',
                                description: 'Enable AI-to-AI communication mode',
                                default: true
                            },
                            interval: {
                                type: 'number',
                                description: 'Loop interval in milliseconds',
                                default: 5000
                            },
                            maxIterations: {
                                type: 'number',
                                description: 'Maximum iterations before auto-stop',
                                default: 999999
                            }
                        },
                        required: ['message']
                    }
                },
                {
                    name: 'stop_ai_loops',
                    description: 'Stop all active AI-to-AI loops',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            message: {
                                type: 'string',
                                description: 'Stop command (e.g., "stploop")'
                            }
                        },
                        required: ['message']
                    }
                },
                {
                    name: 'list_active_loops',
                    description: 'View running loops',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'get_ai_provider_status',
                    description: 'Check provider status',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'reset_ai_providers',
                    description: 'Reset failed providers',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'get_ai_prompts',
                    description: 'Get AI-generated prompts',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            limit: {
                                type: 'number',
                                description: 'Maximum number of prompts to return',
                                default: 5
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'acknowledge_agent_response',
                    description: 'Process AI responses',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            loopId: {
                                type: 'string',
                                description: 'The ID of the loop to acknowledge'
                            },
                            agentResponse: {
                                type: 'string',
                                description: 'The agent response text'
                            }
                        },
                        required: ['loopId']
                    }
                },
                {
                    name: 'ai_voting_request',
                    description: 'Submit prompt for multi-model AI consensus voting',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            prompt: {
                                type: 'string',
                                description: 'The prompt to submit for AI voting'
                            },
                            panel: {
                                type: 'string',
                                description: 'Voting panel to use (general, coding, reasoning, premium)',
                                default: 'general'
                            },
                            strategy: {
                                type: 'string',
                                description: 'Voting strategy (majority, consensus, weighted, unanimous)',
                                default: 'consensus'
                            },
                            maxAgents: {
                                type: 'number',
                                description: 'Maximum number of agents to use',
                                default: 5
                            }
                        },
                        required: ['prompt']
                    }
                },
                {
                    name: 'get_voting_history',
                    description: 'View recent voting sessions',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            limit: {
                                type: 'number',
                                description: 'Number of recent sessions to return',
                                default: 5
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'get_agent_performance',
                    description: 'Check AI agent performance statistics',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            agentId: {
                                type: 'string',
                                description: 'Specific agent ID to check (optional)'
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'analyze_task_breakdown',
                    description: 'Analyze a topic and break it down into actionable subtasks with prioritization',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            topic: {
                                type: 'string',
                                description: 'The topic or project to analyze and break down'
                            },
                            context: {
                                type: 'object',
                                description: 'Additional context for the analysis (optional)'
                            }
                        },
                        required: ['topic']
                    }
                },
                {
                    name: 'deep_think_implementation',
                    description: 'Perform deep thinking analysis on a specific task to explore solutions and create implementation plans',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            taskId: {
                                type: 'string',
                                description: 'ID of the task to analyze'
                            },
                            taskName: {
                                type: 'string',
                                description: 'Name of the task (if taskId not available)'
                            },
                            taskDescription: {
                                type: 'string',
                                description: 'Description of the task (if taskId not available)'
                            },
                            context: {
                                type: 'object',
                                description: 'Additional context for deep thinking (optional)'
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'get_cache_analytics',
                    description: 'Get smart caching system analytics and performance metrics',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'clear_cache',
                    description: 'Clear the smart cache system',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            confirm: {
                                type: 'boolean',
                                description: 'Confirmation to clear cache'
                            }
                        },
                        required: ['confirm']
                    }
                },
                {
                    name: 'optimize_cache',
                    description: 'Optimize cache performance by removing low-value entries',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'create_project',
                    description: 'Create a new project in memory for context persistence',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            projectName: {
                                type: 'string',
                                description: 'Name of the project'
                            },
                            context: {
                                type: 'object',
                                description: 'Project context and metadata'
                            }
                        },
                        required: ['projectName']
                    }
                },
                {
                    name: 'get_project_memory',
                    description: 'Retrieve project memory and context',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            projectId: {
                                type: 'string',
                                description: 'Project ID to retrieve'
                            }
                        },
                        required: ['projectId']
                    }
                },
                {
                    name: 'set_user_preference',
                    description: 'Set user preference for personalization',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            key: {
                                type: 'string',
                                description: 'Preference key'
                            },
                            value: {
                                description: 'Preference value'
                            },
                            context: {
                                type: 'object',
                                description: 'Context for the preference'
                            }
                        },
                        required: ['key', 'value']
                    }
                },
                {
                    name: 'get_memory_analytics',
                    description: 'Get project memory analytics and insights',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'get_recommended_strategies',
                    description: 'Get recommended strategies based on context and history',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            context: {
                                type: 'object',
                                description: 'Current context for recommendations'
                            },
                            limit: {
                                type: 'number',
                                description: 'Maximum number of strategies to return'
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'get_model_analytics',
                    description: 'Get AI model performance analytics and rankings',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            taskType: {
                                type: 'string',
                                description: 'Filter by specific task type'
                            },
                            metric: {
                                type: 'string',
                                description: 'Ranking metric (overall, speed, cost, quality, reliability)'
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'get_cost_analysis',
                    description: 'Get detailed cost analysis for AI model usage',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            timeframe: {
                                type: 'string',
                                description: 'Analysis timeframe (day, week, month, all)'
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'get_recommended_model',
                    description: 'Get recommended AI model for specific task type and priorities',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            taskType: {
                                type: 'string',
                                description: 'Type of task'
                            },
                            priorities: {
                                type: 'object',
                                description: 'Priority weights for cost, speed, quality (0-1)'
                            }
                        },
                        required: ['taskType']
                    }
                },
                {
                    name: 'get_performance_trends',
                    description: 'Get performance trends over time',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            metric: {
                                type: 'string',
                                description: 'Metric to analyze (responseTime, successRate, qualityScore, cost)'
                            },
                            timeframe: {
                                type: 'string',
                                description: 'Time period (day, week, month)'
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'list_workflow_templates',
                    description: 'List available workflow templates with filtering options',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            category: {
                                type: 'string',
                                description: 'Filter by category (web_development, api_development, data_analysis, machine_learning, devops)'
                            },
                            framework: {
                                type: 'string',
                                description: 'Filter by framework (react, nodejs, python, docker, etc.)'
                            },
                            complexity: {
                                type: 'string',
                                description: 'Filter by complexity (low, medium, high)'
                            },
                            search: {
                                type: 'string',
                                description: 'Search query for template name, description, or tags'
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'get_workflow_template',
                    description: 'Get detailed information about a specific workflow template',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            templateId: {
                                type: 'string',
                                description: 'ID of the template to retrieve'
                            }
                        },
                        required: ['templateId']
                    }
                },
                {
                    name: 'create_custom_template',
                    description: 'Create a new custom workflow template',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            name: {
                                type: 'string',
                                description: 'Template name'
                            },
                            category: {
                                type: 'string',
                                description: 'Template category'
                            },
                            description: {
                                type: 'string',
                                description: 'Template description'
                            },
                            framework: {
                                type: 'string',
                                description: 'Primary framework or technology'
                            },
                            complexity: {
                                type: 'string',
                                description: 'Complexity level (low, medium, high)'
                            },
                            estimatedTime: {
                                type: 'string',
                                description: 'Estimated completion time'
                            },
                            tags: {
                                type: 'array',
                                description: 'Template tags',
                                items: { type: 'string' }
                            },
                            steps: {
                                type: 'array',
                                description: 'Template steps',
                                items: { type: 'object' }
                            },
                            dependencies: {
                                type: 'array',
                                description: 'Required dependencies',
                                items: { type: 'string' }
                            },
                            resources: {
                                type: 'array',
                                description: 'Helpful resources and links',
                                items: { type: 'string' }
                            }
                        },
                        required: ['name', 'category', 'description', 'steps']
                    }
                },
                {
                    name: 'get_template_recommendations',
                    description: 'Get recommended templates based on context and usage patterns',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            context: {
                                type: 'object',
                                description: 'Context for recommendations (category, framework, tags, etc.)'
                            },
                            limit: {
                                type: 'number',
                                description: 'Maximum number of recommendations to return'
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'get_template_analytics',
                    description: 'Get workflow template usage analytics and statistics',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'create_workspace',
                    description: 'Create a new collaborative workspace',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            name: {
                                type: 'string',
                                description: 'Workspace name'
                            },
                            creatorId: {
                                type: 'string',
                                description: 'ID of the workspace creator'
                            },
                            options: {
                                type: 'object',
                                description: 'Workspace configuration options'
                            }
                        },
                        required: ['name', 'creatorId']
                    }
                },
                {
                    name: 'join_workspace',
                    description: 'Join an existing collaborative workspace',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            workspaceId: {
                                type: 'string',
                                description: 'ID of the workspace to join'
                            },
                            userId: {
                                type: 'string',
                                description: 'ID of the user joining'
                            },
                            userInfo: {
                                type: 'object',
                                description: 'User information and preferences'
                            }
                        },
                        required: ['workspaceId', 'userId']
                    }
                },
                {
                    name: 'execute_collaborative_operation',
                    description: 'Execute an operation in a collaborative workspace',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            sessionId: {
                                type: 'string',
                                description: 'User session ID'
                            },
                            operation: {
                                type: 'object',
                                description: 'Operation to execute',
                                properties: {
                                    type: { type: 'string' },
                                    data: { type: 'object' }
                                },
                                required: ['type', 'data']
                            }
                        },
                        required: ['sessionId', 'operation']
                    }
                },
                {
                    name: 'get_workspace_status',
                    description: 'Get current status and state of a workspace',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            workspaceId: {
                                type: 'string',
                                description: 'ID of the workspace'
                            },
                            userId: {
                                type: 'string',
                                description: 'ID of the requesting user'
                            }
                        },
                        required: ['workspaceId', 'userId']
                    }
                },
                {
                    name: 'get_collaboration_analytics',
                    description: 'Get real-time collaboration analytics and statistics',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'create_ai_swarm',
                    description: 'Create a specialized AI swarm for complex tasks',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            taskDescription: {
                                type: 'string',
                                description: 'Description of the task for the AI swarm'
                            },
                            requiredAgents: {
                                type: 'array',
                                description: 'Required agent types (frontend, backend, devops, testing, security)',
                                items: { type: 'string' }
                            },
                            options: {
                                type: 'object',
                                description: 'Swarm configuration options'
                            }
                        },
                        required: ['taskDescription']
                    }
                },
                {
                    name: 'get_swarm_status',
                    description: 'Get current status and progress of an AI swarm',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            swarmId: {
                                type: 'string',
                                description: 'ID of the AI swarm'
                            }
                        },
                        required: ['swarmId']
                    }
                },
                {
                    name: 'get_swarm_analytics',
                    description: 'Get AI swarm intelligence analytics and performance metrics',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'terminate_swarm',
                    description: 'Terminate an active AI swarm',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            swarmId: {
                                type: 'string',
                                description: 'ID of the AI swarm to terminate'
                            }
                        },
                        required: ['swarmId']
                    }
                },
                {
                    name: 'create_predictive_project',
                    description: 'Create a new project with AI-powered predictive analytics',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            name: {
                                type: 'string',
                                description: 'Project name'
                            },
                            description: {
                                type: 'string',
                                description: 'Project description'
                            },
                            team: {
                                type: 'array',
                                description: 'Team members with skills and experience',
                                items: { type: 'object' }
                            },
                            startDate: {
                                type: 'number',
                                description: 'Project start date (timestamp)'
                            },
                            resources: {
                                type: 'object',
                                description: 'Available resources'
                            }
                        },
                        required: ['name', 'description']
                    }
                },
                {
                    name: 'add_predictive_task',
                    description: 'Add a task to a predictive project with failure prediction and timeline estimation',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            projectId: {
                                type: 'string',
                                description: 'Project ID'
                            },
                            title: {
                                type: 'string',
                                description: 'Task title'
                            },
                            description: {
                                type: 'string',
                                description: 'Task description'
                            },
                            type: {
                                type: 'string',
                                description: 'Task type (development, testing, design, etc.)'
                            },
                            complexity: {
                                type: 'string',
                                description: 'Task complexity (low, medium, high, critical)'
                            },
                            priority: {
                                type: 'string',
                                description: 'Task priority (low, medium, high, critical)'
                            },
                            assignee: {
                                type: 'string',
                                description: 'Assigned team member ID'
                            },
                            estimatedHours: {
                                type: 'number',
                                description: 'Estimated hours to complete'
                            },
                            dependencies: {
                                type: 'array',
                                description: 'Task dependencies (task IDs)',
                                items: { type: 'string' }
                            }
                        },
                        required: ['projectId', 'title', 'description']
                    }
                },
                {
                    name: 'get_project_predictions',
                    description: 'Get AI predictions for a project including failure risk, timeline accuracy, and resource optimization',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            projectId: {
                                type: 'string',
                                description: 'Project ID'
                            }
                        },
                        required: ['projectId']
                    }
                },
                {
                    name: 'get_task_predictions',
                    description: 'Get AI predictions for a specific task',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            projectId: {
                                type: 'string',
                                description: 'Project ID'
                            },
                            taskId: {
                                type: 'string',
                                description: 'Task ID'
                            }
                        },
                        required: ['projectId', 'taskId']
                    }
                },
                {
                    name: 'get_predictive_analytics',
                    description: 'Get comprehensive predictive analytics and model performance metrics',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'optimize_project_timeline',
                    description: 'Get AI-powered timeline optimization suggestions for a project',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            projectId: {
                                type: 'string',
                                description: 'Project ID'
                            }
                        },
                        required: ['projectId']
                    }
                },
                {
                    name: 'start_code_session',
                    description: 'Start a real-time code generation and review session',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            userId: {
                                type: 'string',
                                description: 'User ID'
                            },
                            projectId: {
                                type: 'string',
                                description: 'Project ID (optional)'
                            },
                            language: {
                                type: 'string',
                                description: 'Programming language (javascript, python, java, etc.)'
                            },
                            fileName: {
                                type: 'string',
                                description: 'File name being edited'
                            },
                            initialCode: {
                                type: 'string',
                                description: 'Initial code content (optional)'
                            }
                        },
                        required: ['userId', 'language', 'fileName']
                    }
                },
                {
                    name: 'update_code',
                    description: 'Update code in a real-time session and get suggestions',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            sessionId: {
                                type: 'string',
                                description: 'Code session ID'
                            },
                            code: {
                                type: 'string',
                                description: 'Updated code content'
                            },
                            cursor: {
                                type: 'object',
                                description: 'Cursor position {line, column}',
                                properties: {
                                    line: { type: 'number' },
                                    column: { type: 'number' }
                                }
                            },
                            changeType: {
                                type: 'string',
                                description: 'Type of change (edit, insert, delete)'
                            }
                        },
                        required: ['sessionId', 'code']
                    }
                },
                {
                    name: 'generate_code',
                    description: 'Generate code from natural language prompt',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            prompt: {
                                type: 'string',
                                description: 'Natural language description of code to generate'
                            },
                            language: {
                                type: 'string',
                                description: 'Target programming language'
                            },
                            context: {
                                type: 'object',
                                description: 'Additional context for code generation'
                            }
                        },
                        required: ['prompt', 'language']
                    }
                },
                {
                    name: 'get_code_suggestions',
                    description: 'Get real-time code suggestions and analysis for a session',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            sessionId: {
                                type: 'string',
                                description: 'Code session ID'
                            }
                        },
                        required: ['sessionId']
                    }
                },
                {
                    name: 'get_codegen_analytics',
                    description: 'Get analytics and metrics for code generation system',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'end_code_session',
                    description: 'End a real-time code session',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            sessionId: {
                                type: 'string',
                                description: 'Code session ID'
                            }
                        },
                        required: ['sessionId']
                    }
                },
                {
                    name: 'record_user_feedback',
                    description: 'Record user feedback for adaptive learning',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            userId: {
                                type: 'string',
                                description: 'User ID'
                            },
                            type: {
                                type: 'string',
                                description: 'Feedback type (positive, negative, neutral)'
                            },
                            category: {
                                type: 'string',
                                description: 'Feedback category (suggestion, code_generation, analysis, etc.)'
                            },
                            content: {
                                type: 'string',
                                description: 'Feedback content'
                            },
                            rating: {
                                type: 'number',
                                description: 'Rating from 1-5'
                            },
                            context: {
                                type: 'object',
                                description: 'Additional context for the feedback'
                            }
                        },
                        required: ['userId', 'type', 'category', 'content', 'rating']
                    }
                },
                {
                    name: 'analyze_coding_style',
                    description: 'Analyze user coding style for adaptive learning',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            userId: {
                                type: 'string',
                                description: 'User ID'
                            },
                            language: {
                                type: 'string',
                                description: 'Programming language'
                            },
                            code: {
                                type: 'string',
                                description: 'Code to analyze'
                            }
                        },
                        required: ['userId', 'language', 'code']
                    }
                },
                {
                    name: 'record_project_outcome',
                    description: 'Record project outcome for learning and prediction improvement',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            userId: {
                                type: 'string',
                                description: 'User ID'
                            },
                            projectId: {
                                type: 'string',
                                description: 'Project ID'
                            },
                            projectName: {
                                type: 'string',
                                description: 'Project name'
                            },
                            outcome: {
                                type: 'string',
                                description: 'Project outcome (success, failure, partial)'
                            },
                            duration: {
                                type: 'number',
                                description: 'Project duration in days'
                            },
                            complexity: {
                                type: 'string',
                                description: 'Project complexity (low, medium, high)'
                            },
                            teamSize: {
                                type: 'number',
                                description: 'Team size'
                            },
                            technologies: {
                                type: 'array',
                                description: 'Technologies used',
                                items: { type: 'string' }
                            },
                            challenges: {
                                type: 'array',
                                description: 'Challenges faced',
                                items: { type: 'string' }
                            },
                            successFactors: {
                                type: 'array',
                                description: 'Success factors',
                                items: { type: 'string' }
                            },
                            lessons: {
                                type: 'array',
                                description: 'Lessons learned',
                                items: { type: 'string' }
                            }
                        },
                        required: ['userId', 'projectId', 'projectName', 'outcome']
                    }
                },
                {
                    name: 'get_personalized_recommendations',
                    description: 'Get personalized recommendations based on user learning profile',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            userId: {
                                type: 'string',
                                description: 'User ID'
                            },
                            context: {
                                type: 'object',
                                description: 'Context for recommendations (language, taskType, etc.)'
                            }
                        },
                        required: ['userId']
                    }
                },
                {
                    name: 'predict_project_outcome',
                    description: 'Predict project outcome based on learning data',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            userId: {
                                type: 'string',
                                description: 'User ID'
                            },
                            projectId: {
                                type: 'string',
                                description: 'Project ID'
                            },
                            complexity: {
                                type: 'string',
                                description: 'Project complexity (low, medium, high)'
                            },
                            teamSize: {
                                type: 'number',
                                description: 'Team size'
                            },
                            timeline: {
                                type: 'number',
                                description: 'Timeline in days'
                            },
                            technologies: {
                                type: 'array',
                                description: 'Technologies to be used',
                                items: { type: 'string' }
                            },
                            challenges: {
                                type: 'array',
                                description: 'Expected challenges',
                                items: { type: 'string' }
                            }
                        },
                        required: ['userId', 'projectId']
                    }
                },
                {
                    name: 'get_learning_analytics',
                    description: 'Get adaptive learning system analytics and metrics',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'get_user_learning_profile',
                    description: 'Get detailed learning profile for a specific user',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            userId: {
                                type: 'string',
                                description: 'User ID'
                            }
                        },
                        required: ['userId']
                    }
                },
                {
                    name: 'create_prompt',
                    description: 'Create a new prompt in the enhanced prompt library',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            title: {
                                type: 'string',
                                description: 'Prompt title'
                            },
                            content: {
                                type: 'string',
                                description: 'Prompt content with variables in {variable} format'
                            },
                            description: {
                                type: 'string',
                                description: 'Prompt description'
                            },
                            domain: {
                                type: 'string',
                                description: 'Domain category (web_development, data_science, etc.)'
                            },
                            tags: {
                                type: 'array',
                                description: 'Tags for categorization',
                                items: { type: 'string' }
                            },
                            isPublic: {
                                type: 'boolean',
                                description: 'Whether prompt is public'
                            },
                            authorId: {
                                type: 'string',
                                description: 'Author ID'
                            }
                        },
                        required: ['title', 'content', 'authorId']
                    }
                },
                {
                    name: 'search_prompts',
                    description: 'Search prompts in the library',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            query: {
                                type: 'string',
                                description: 'Search query'
                            },
                            domain: {
                                type: 'string',
                                description: 'Filter by domain'
                            },
                            tags: {
                                type: 'array',
                                description: 'Filter by tags',
                                items: { type: 'string' }
                            },
                            minQuality: {
                                type: 'number',
                                description: 'Minimum quality score (0-1)'
                            },
                            minRating: {
                                type: 'number',
                                description: 'Minimum rating (1-5)'
                            },
                            author: {
                                type: 'string',
                                description: 'Filter by author'
                            },
                            limit: {
                                type: 'number',
                                description: 'Maximum results to return'
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'get_prompt_by_id',
                    description: 'Get a specific prompt by ID',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            promptId: {
                                type: 'string',
                                description: 'Prompt ID'
                            }
                        },
                        required: ['promptId']
                    }
                },
                {
                    name: 'update_prompt',
                    description: 'Update an existing prompt (creates new version)',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            promptId: {
                                type: 'string',
                                description: 'Prompt ID'
                            },
                            title: {
                                type: 'string',
                                description: 'Updated title'
                            },
                            content: {
                                type: 'string',
                                description: 'Updated content'
                            },
                            description: {
                                type: 'string',
                                description: 'Updated description'
                            },
                            tags: {
                                type: 'array',
                                description: 'Updated tags',
                                items: { type: 'string' }
                            },
                            changes: {
                                type: 'string',
                                description: 'Description of changes made'
                            },
                            authorId: {
                                type: 'string',
                                description: 'Author ID'
                            }
                        },
                        required: ['promptId', 'authorId']
                    }
                },
                {
                    name: 'rate_prompt',
                    description: 'Rate a prompt and provide feedback',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            promptId: {
                                type: 'string',
                                description: 'Prompt ID'
                            },
                            userId: {
                                type: 'string',
                                description: 'User ID'
                            },
                            rating: {
                                type: 'number',
                                description: 'Rating from 1-5'
                            },
                            feedback: {
                                type: 'string',
                                description: 'Optional feedback text'
                            }
                        },
                        required: ['promptId', 'userId', 'rating']
                    }
                },
                {
                    name: 'create_collection',
                    description: 'Create a new prompt collection',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            name: {
                                type: 'string',
                                description: 'Collection name'
                            },
                            description: {
                                type: 'string',
                                description: 'Collection description'
                            },
                            domain: {
                                type: 'string',
                                description: 'Domain category'
                            },
                            tags: {
                                type: 'array',
                                description: 'Collection tags',
                                items: { type: 'string' }
                            },
                            visibility: {
                                type: 'string',
                                description: 'Collection visibility (public, private)'
                            },
                            authorId: {
                                type: 'string',
                                description: 'Author ID'
                            }
                        },
                        required: ['name', 'authorId']
                    }
                },
                {
                    name: 'start_ab_test',
                    description: 'Start an A/B test between two prompts',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            name: {
                                type: 'string',
                                description: 'Test name'
                            },
                            description: {
                                type: 'string',
                                description: 'Test description'
                            },
                            promptA: {
                                type: 'string',
                                description: 'First prompt ID'
                            },
                            promptB: {
                                type: 'string',
                                description: 'Second prompt ID'
                            },
                            testType: {
                                type: 'string',
                                description: 'Test type (effectiveness, user_preference, performance)'
                            },
                            targetMetric: {
                                type: 'string',
                                description: 'Target metric to measure'
                            },
                            authorId: {
                                type: 'string',
                                description: 'Author ID'
                            }
                        },
                        required: ['name', 'promptA', 'promptB', 'authorId']
                    }
                },
                {
                    name: 'submit_community_prompt',
                    description: 'Submit a prompt for community review',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            title: {
                                type: 'string',
                                description: 'Prompt title'
                            },
                            content: {
                                type: 'string',
                                description: 'Prompt content'
                            },
                            description: {
                                type: 'string',
                                description: 'Prompt description'
                            },
                            domain: {
                                type: 'string',
                                description: 'Domain category'
                            },
                            tags: {
                                type: 'array',
                                description: 'Tags',
                                items: { type: 'string' }
                            },
                            authorId: {
                                type: 'string',
                                description: 'Author ID'
                            }
                        },
                        required: ['title', 'content', 'authorId']
                    }
                },
                {
                    name: 'get_popular_prompts',
                    description: 'Get most popular prompts',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            limit: {
                                type: 'number',
                                description: 'Number of prompts to return'
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'get_prompts_by_domain',
                    description: 'Get prompts by domain category',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            domain: {
                                type: 'string',
                                description: 'Domain category'
                            }
                        },
                        required: ['domain']
                    }
                },
                {
                    name: 'get_prompt_analytics',
                    description: 'Get prompt library analytics and statistics',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'parallel_execute_tasks',
                    description: 'Execute multiple tasks in parallel with intelligent coordination and dependency management',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            breakdownId: {
                                type: 'string',
                                description: 'ID of the task breakdown to execute'
                            },
                            executionStrategy: {
                                type: 'string',
                                description: 'Execution strategy (parallel, sequential, hybrid)',
                                default: 'parallel'
                            },
                            maxConcurrency: {
                                type: 'number',
                                description: 'Maximum number of concurrent tasks',
                                default: 5
                            }
                        },
                        required: ['breakdownId']
                    }
                }
            ]
        }));

        // Handle tool calls
        this.server.setRequestHandler(CallToolRequestSchema, async (request) => {
            const { name, arguments: args } = request.params;

            try {
                switch (name) {
                    case 'activate_infinite_loop':
                        return await this.handleActivateLoop(args);

                    case 'stop_ai_loops':
                        return await this.handleStopAILoops(args);

                    case 'list_active_loops':
                        return await this.handleListActiveLoops(args);

                    case 'get_ai_provider_status':
                        return await this.handleGetAIProviderStatus(args);

                    case 'reset_ai_providers':
                        return await this.handleResetAIProviders(args);

                    case 'get_ai_prompts':
                        return await this.handleGetAIPrompts(args);

                    case 'acknowledge_agent_response':
                        return await this.handleAcknowledgeAgentResponse(args);

                    case 'ai_voting_request':
                        return await this.handleAIVotingRequest(args);

                    case 'get_voting_history':
                        return await this.handleGetVotingHistory(args);

                    case 'get_agent_performance':
                        return await this.handleGetAgentPerformance(args);

                    case 'analyze_task_breakdown':
                        return await this.handleAnalyzeTaskBreakdown(args);

                    case 'deep_think_implementation':
                        return await this.handleDeepThinkImplementation(args);

                    case 'parallel_execute_tasks':
                        return await this.handleParallelExecuteTasks(args);

                    case 'get_cache_analytics':
                        return await this.handleGetCacheAnalytics(args);

                    case 'clear_cache':
                        return await this.handleClearCache(args);

                    case 'optimize_cache':
                        return await this.handleOptimizeCache(args);

                    case 'create_project':
                        return await this.handleCreateProject(args);

                    case 'get_project_memory':
                        return await this.handleGetProjectMemory(args);

                    case 'set_user_preference':
                        return await this.handleSetUserPreference(args);

                    case 'get_memory_analytics':
                        return await this.handleGetMemoryAnalytics(args);

                    case 'get_recommended_strategies':
                        return await this.handleGetRecommendedStrategies(args);

                    case 'get_model_analytics':
                        return await this.handleGetModelAnalytics(args);

                    case 'get_cost_analysis':
                        return await this.handleGetCostAnalysis(args);

                    case 'get_recommended_model':
                        return await this.handleGetRecommendedModel(args);

                    case 'get_performance_trends':
                        return await this.handleGetPerformanceTrends(args);

                    case 'list_workflow_templates':
                        return await this.handleListWorkflowTemplates(args);

                    case 'get_workflow_template':
                        return await this.handleGetWorkflowTemplate(args);

                    case 'create_custom_template':
                        return await this.handleCreateCustomTemplate(args);

                    case 'get_template_recommendations':
                        return await this.handleGetTemplateRecommendations(args);

                    case 'get_template_analytics':
                        return await this.handleGetTemplateAnalytics(args);

                    case 'create_workspace':
                        return await this.handleCreateWorkspace(args);

                    case 'join_workspace':
                        return await this.handleJoinWorkspace(args);

                    case 'execute_collaborative_operation':
                        return await this.handleExecuteCollaborativeOperation(args);

                    case 'get_workspace_status':
                        return await this.handleGetWorkspaceStatus(args);

                    case 'get_collaboration_analytics':
                        return await this.handleGetCollaborationAnalytics(args);

                    case 'create_ai_swarm':
                        return await this.handleCreateAISwarm(args);

                    case 'get_swarm_status':
                        return await this.handleGetSwarmStatus(args);

                    case 'get_swarm_analytics':
                        return await this.handleGetSwarmAnalytics(args);

                    case 'terminate_swarm':
                        return await this.handleTerminateSwarm(args);

                    case 'create_predictive_project':
                        return await this.handleCreatePredictiveProject(args);

                    case 'add_predictive_task':
                        return await this.handleAddPredictiveTask(args);

                    case 'get_project_predictions':
                        return await this.handleGetProjectPredictions(args);

                    case 'get_task_predictions':
                        return await this.handleGetTaskPredictions(args);

                    case 'get_predictive_analytics':
                        return await this.handleGetPredictiveAnalytics(args);

                    case 'optimize_project_timeline':
                        return await this.handleOptimizeProjectTimeline(args);

                    case 'start_code_session':
                        return await this.handleStartCodeSession(args);

                    case 'update_code':
                        return await this.handleUpdateCode(args);

                    case 'generate_code':
                        return await this.handleGenerateCode(args);

                    case 'get_code_suggestions':
                        return await this.handleGetCodeSuggestions(args);

                    case 'get_codegen_analytics':
                        return await this.handleGetCodegenAnalytics(args);

                    case 'end_code_session':
                        return await this.handleEndCodeSession(args);

                    case 'record_user_feedback':
                        return await this.handleRecordUserFeedback(args);

                    case 'analyze_coding_style':
                        return await this.handleAnalyzeCodingStyle(args);

                    case 'record_project_outcome':
                        return await this.handleRecordProjectOutcome(args);

                    case 'get_personalized_recommendations':
                        return await this.handleGetPersonalizedRecommendations(args);

                    case 'predict_project_outcome':
                        return await this.handlePredictProjectOutcome(args);

                    case 'get_learning_analytics':
                        return await this.handleGetLearningAnalytics(args);

                    case 'get_user_learning_profile':
                        return await this.handleGetUserLearningProfile(args);

                    case 'create_prompt':
                        return await this.handleCreatePrompt(args);

                    case 'search_prompts':
                        return await this.handleSearchPrompts(args);

                    case 'get_prompt_by_id':
                        return await this.handleGetPromptById(args);

                    case 'update_prompt':
                        return await this.handleUpdatePrompt(args);

                    case 'rate_prompt':
                        return await this.handleRatePrompt(args);

                    case 'create_collection':
                        return await this.handleCreateCollection(args);

                    case 'start_ab_test':
                        return await this.handleStartABTest(args);

                    case 'submit_community_prompt':
                        return await this.handleSubmitCommunityPrompt(args);

                    case 'get_popular_prompts':
                        return await this.handleGetPopularPrompts(args);

                    case 'get_prompts_by_domain':
                        return await this.handleGetPromptsByDomain(args);

                    case 'get_prompt_analytics':
                        return await this.handleGetPromptAnalytics(args);

                    default:
                        return {
                            content: [{
                                type: 'text',
                                text: `❌ Unknown tool: "${name}"\n\n` +
                                      `💡 **Available tools:**\n` +
                                      `• activate_infinite_loop - Start AI-to-AI loops\n` +
                                      `• stop_ai_loops - Stop active loops\n` +
                                      `• ai_voting_request - Multi-model voting\n` +
                                      `• create_prompt - Create new prompts\n` +
                                      `• search_prompts - Search prompt library\n` +
                                      `• record_user_feedback - Record feedback\n` +
                                      `• get_prompt_analytics - Get analytics\n` +
                                      `• And many more...\n\n` +
                                      `🔍 Use the MCP tools list to see all available tools.`
                            }]
                        };
                }
            } catch (error) {
                return {
                    content: [
                        {
                            type: 'text',
                            text: `Error executing tool ${name}: ${error.message}`
                        }
                    ],
                    isError: true
                };
            }
        });
    }

    async handleActivateLoop(args) {
        const { message, aiToAi = true, interval = 5000, maxIterations = 999999 } = args;

        if (!message || typeof message !== 'string') {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing or invalid message parameter. Please provide a string message with format: "actloop [topic]"'
                }]
            };
        }

        // Check if message matches activation pattern
        if (!message.toLowerCase().startsWith('actloop ')) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Invalid activation message format. Please use format: "actloop [topic]" (e.g., "actloop improve my React component")'
                }]
            };
        }

        const topic = message.substring(8).trim();
        if (!topic) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ No topic specified. Please provide a topic after "actloop " (e.g., "actloop improve my React component")'
                }]
            };
        }

        try {
            const loopId = `loop-${++this.loopCounter}`;
            const loop = {
                id: loopId,
                topic,
                aiToAi,
                interval,
                maxIterations,
                currentIteration: 0,
                status: 'running',
                startTime: new Date().toISOString(),
                lastActivity: new Date().toISOString()
            };

            this.activeLoops.set(loopId, loop);

            // Set up strict acknowledgment requirement for this loop
            this.acknowledgmentSystem.acknowledgmentRequired = true;
            this.acknowledgmentSystem.strictMode = true;
            this.acknowledgmentSystem.currentLoopId = loopId;
            this.acknowledgmentSystem.pendingAcknowledgments.set(loopId, {
                timestamp: Date.now(),
                topic: topic,
                status: 'waiting_for_first_acknowledgment'
            });

            console.error(`🔒 STRICT ACKNOWLEDGMENT ACTIVATED for loop ${loopId}`);
            console.error('⚠️  AI prompts will be BLOCKED until acknowledgment is received');

            // Start the enhanced AI-to-AI loop with task management
            this.startEnhancedAILoop(loop);

            return {
                content: [{
                    type: 'text',
                    text: `✅ **AI-to-AI Loop Started Successfully!**\n\n` +
                          `🔄 **Loop ID:** ${loopId}\n` +
                          `📝 **Topic:** ${topic}\n` +
                          `🤖 **AI-to-AI Mode:** ${aiToAi ? 'Enabled' : 'Disabled'}\n` +
                          `⏱️ **Interval:** ${interval}ms\n` +
                          `🔢 **Max Iterations:** ${maxIterations}\n\n` +
                          `🚀 **The loop is now running and will continuously improve the topic using AI-to-AI collaboration.**\n\n` +
                          `🔒 **STRICT ACKNOWLEDGMENT MODE ACTIVATED**\n\n` +
                          `⚠️  **IMPORTANT:** AI prompts are now BLOCKED until you acknowledge this loop activation.\n\n` +
                          `📋 **REQUIRED ACTION:**\n` +
                          `Before you can receive AI prompts, you MUST acknowledge this loop by using:\n\n` +
                          `\`\`\`\n` +
                          `acknowledge_agent_response\n` +
                          `loopId: ${loopId}\n` +
                          `agentResponse: [Your response about starting this loop]\n` +
                          `\`\`\`\n\n` +
                          `🚨 **Until acknowledgment is received:**\n` +
                          `• All AI prompts will be blocked\n` +
                          `• get_ai_prompts will return acknowledgment requirement message\n` +
                          `• Loop will wait for your acknowledgment before proceeding\n\n` +
                          `💡 **This ensures proper AI-to-AI communication flow and prevents missed responses.**`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to start loop: ${error.message}`
                }]
            };
        }
    }

    async startAILoop(loop) {
        const runIteration = async () => {
            if (loop.status !== 'running' || loop.currentIteration >= loop.maxIterations) {
                this.activeLoops.delete(loop.id);
                return;
            }

            try {
                loop.currentIteration++;
                loop.lastActivity = new Date().toISOString();

                // Generate AI improvement using multi-provider system
                const prompt = `Improve the following topic: "${loop.topic}".

Current iteration: ${loop.currentIteration}
Provide a specific, actionable improvement suggestion.`;

                const response = await this.multiAI.makeRequest(prompt, {
                    maxTokens: 500,
                    temperature: 0.7
                });

                // Collect data for training
                await this.dataCollector.collectInteraction({
                    type: 'ai-to-ai',
                    prompt,
                    response: response.content,
                    context: {
                        loopId: loop.id,
                        topic: loop.topic,
                        iteration: loop.currentIteration
                    },
                    success: true,
                    responseTime: response.responseTime,
                    loopIteration: loop.currentIteration,
                    model: response.model,
                    provider: response.provider,
                    sessionId: loop.id
                });

                console.error(`[AI-TO-AI] Loop ${loop.id} - Iteration ${loop.currentIteration}: ${response.content.substring(0, 100)}...`);

                // Schedule next iteration
                setTimeout(runIteration, loop.interval);

            } catch (error) {
                console.error(`[AI-TO-AI] Error in loop ${loop.id}:`, error);

                // Collect error data
                await this.dataCollector.collectInteraction({
                    type: 'ai-to-ai',
                    prompt: `Improve: ${loop.topic}`,
                    response: null,
                    context: { loopId: loop.id, topic: loop.topic, iteration: loop.currentIteration },
                    success: false,
                    errors: [error.message],
                    sessionId: loop.id
                });

                // Retry after longer interval
                setTimeout(runIteration, loop.interval * 2);
            }
        };

        // Start the first iteration
        setTimeout(runIteration, 1000);
    }

    async startEnhancedAILoop(loop) {
        console.log(`🚀 Starting enhanced AI-to-AI loop with task management for: ${loop.topic}`);

        let currentBreakdown = null;
        let currentExecution = null;

        const runEnhancedIteration = async () => {
            if (loop.status !== 'running' || loop.currentIteration >= loop.maxIterations) {
                this.activeLoops.delete(loop.id);
                return;
            }

            try {
                loop.currentIteration++;
                loop.lastActivity = new Date().toISOString();

                // Phase 1: Task breakdown (every 5 iterations or first iteration)
                if (loop.currentIteration === 1 || loop.currentIteration % 5 === 0) {
                    console.log(`🎯 [Loop ${loop.id}] Performing task breakdown analysis`);

                    currentBreakdown = await this.taskManager.analyzeTaskBreakdown(loop.topic, {
                        loopId: loop.id,
                        iteration: loop.currentIteration,
                        previousBreakdown: currentBreakdown?.id
                    });

                    console.log(`📋 [Loop ${loop.id}] Identified ${currentBreakdown.subtasks.length} subtasks`);
                }

                // Phase 2: Deep thinking on priority tasks (every 3 iterations)
                if (currentBreakdown && (loop.currentIteration % 3 === 0)) {
                    const prioritizedTasks = this.taskManager.prioritizeTasks(currentBreakdown);
                    const topTask = prioritizedTasks[0];

                    if (topTask) {
                        console.log(`🧠 [Loop ${loop.id}] Deep thinking on: ${topTask.name}`);

                        const thinkingSession = await this.deepThinking.deepThinkImplementation(topTask, {
                            loopId: loop.id,
                            iteration: loop.currentIteration
                        });

                        console.log(`💡 [Loop ${loop.id}] Deep thinking completed in ${thinkingSession.duration}ms`);
                    }
                }

                // Phase 3: Parallel execution (every 7 iterations)
                if (currentBreakdown && (loop.currentIteration % 7 === 0)) {
                    console.log(`⚡ [Loop ${loop.id}] Starting parallel execution`);

                    const executionPlan = this.taskManager.getExecutionPlan(currentBreakdown);
                    currentExecution = await this.parallelImplementation.parallelExecuteTasks(currentBreakdown, executionPlan);

                    console.log(`✅ [Loop ${loop.id}] Parallel execution completed: ${currentExecution.completedTasks.size} tasks done`);

                    // Add discovered tasks to the breakdown
                    if (currentExecution.discoveredTasks.length > 0) {
                        currentBreakdown.subtasks.push(...currentExecution.discoveredTasks);
                        console.log(`💡 [Loop ${loop.id}] Added ${currentExecution.discoveredTasks.length} discovered tasks`);
                    }
                }

                // Phase 4: Regular AI improvement (every iteration)
                const prompt = `Continue improving: "${loop.topic}".

Current iteration: ${loop.currentIteration}
${currentBreakdown ? `Active breakdown: ${currentBreakdown.subtasks.length} subtasks identified` : ''}
${currentExecution ? `Last execution: ${currentExecution.completedTasks.size} tasks completed` : ''}

Provide the next improvement or insight.`;

                const response = await this.multiAI.makeRequest(prompt, {
                    maxTokens: 500,
                    temperature: 0.7
                });

                // Collect comprehensive data
                await this.dataCollector.collectInteraction({
                    type: 'enhanced-ai-loop',
                    prompt,
                    response: response.content,
                    context: {
                        loopId: loop.id,
                        topic: loop.topic,
                        iteration: loop.currentIteration,
                        hasBreakdown: !!currentBreakdown,
                        hasExecution: !!currentExecution,
                        subtaskCount: currentBreakdown?.subtasks.length || 0,
                        completedTasks: currentExecution?.completedTasks.size || 0
                    },
                    success: true,
                    responseTime: response.responseTime,
                    loopIteration: loop.currentIteration,
                    model: response.model,
                    provider: response.provider,
                    sessionId: loop.id
                });

                console.error(`[ENHANCED-AI-LOOP] ${loop.id} - Iteration ${loop.currentIteration}: ${response.content.substring(0, 100)}...`);

                // Schedule next iteration
                setTimeout(runEnhancedIteration, loop.interval);

            } catch (error) {
                console.error(`[ENHANCED-AI-LOOP] Error in loop ${loop.id}:`, error);

                // Collect error data
                await this.dataCollector.collectInteraction({
                    type: 'enhanced-ai-loop',
                    prompt: `Enhanced improve: ${loop.topic}`,
                    response: null,
                    context: {
                        loopId: loop.id,
                        topic: loop.topic,
                        iteration: loop.currentIteration,
                        error: error.message
                    },
                    success: false,
                    errors: [error.message],
                    sessionId: loop.id
                });

                // Retry after longer interval
                setTimeout(runEnhancedIteration, loop.interval * 2);
            }
        };

        // Start the first enhanced iteration
        setTimeout(runEnhancedIteration, 2000);
    }

    async handleStopAILoops(args) {
        const { message } = args;

        if (!message || !message.toLowerCase().includes('stploop')) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Invalid stop command. Please use "stploop" to stop AI-to-AI loops.'
                }]
            };
        }

        const stoppedCount = this.activeLoops.size;

        // Stop all active loops
        for (const [, loop] of this.activeLoops) {
            loop.status = 'stopped';
        }
        this.activeLoops.clear();

        // Flush any pending data
        await this.dataCollector.flushData();

        return {
            content: [{
                type: 'text',
                text: `✅ All AI-to-AI loops stopped successfully!\n\n🛑 Stopped ${stoppedCount} active loop(s)\n📊 Data collection completed for all loops`
            }]
        };
    }

    async handleListActiveLoops() {
        if (this.activeLoops.size === 0) {
            return {
                content: [{
                    type: 'text',
                    text: '📋 No active loops currently running.'
                }]
            };
        }

        const loopsList = Array.from(this.activeLoops.values()).map(loop =>
            `🔄 **${loop.id}**\n` +
            `   📝 Topic: ${loop.topic}\n` +
            `   🔢 Iteration: ${loop.currentIteration}\n` +
            `   ⏱️ Started: ${new Date(loop.startTime).toLocaleString()}\n` +
            `   📊 Status: ${loop.status === 'running' ? '🟢 Active' : '🔴 Stopped'}`
        ).join('\n\n');

        return {
            content: [{
                type: 'text',
                text: `📋 **Active AI-to-AI Loops (${this.activeLoops.size})**\n\n${loopsList}`
            }]
        };
    }

    async handleGetAIProviderStatus() {
        try {
            const status = this.multiAI.getStatus();

            return {
                content: [{
                    type: 'text',
                    text: `🤖 **AI Provider Status**\n\n` +
                          `🎯 **Current Provider:** ${status.currentProvider || 'None'}\n` +
                          `📊 **Success Rate:** ${status.successCount}/${status.totalRequests} (${status.successRate}%)\n` +
                          `⚡ **Available Providers:**\n` +
                          Object.entries(status.providers).map(([, provider]) =>
                              `   ${provider.enabled ? '✅' : '❌'} ${provider.name} (${provider.apiKeys} key(s))`
                          ).join('\n') + '\n\n' +
                          `🔄 **Request Statistics:**\n` +
                          `   Total Requests: ${status.totalRequests}\n` +
                          `   Successful: ${status.successCount}\n` +
                          `   Failed: ${status.errorCount}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get provider status: ${error.message}`
                }]
            };
        }
    }

    async handleResetAIProviders() {
        try {
            this.multiAI.resetProviders();

            return {
                content: [{
                    type: 'text',
                    text: `✅ AI providers reset successfully!\n\n🔄 All failed providers have been reset and are available for retry.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to reset providers: ${error.message}`
                }]
            };
        }
    }

    async handleGetAIPrompts(args) {
        const { limit = 5 } = args;

        // STRICT ACKNOWLEDGMENT CHECK - Block AI prompts until acknowledgment received
        if (this.acknowledgmentSystem.strictMode && this.acknowledgmentSystem.acknowledgmentRequired) {
            const timeSinceLastAck = Date.now() - this.acknowledgmentSystem.lastAcknowledgmentTime;

            if (timeSinceLastAck > 30000) { // 30 seconds
                return {
                    content: [{
                        type: 'text',
                        text: '🚨 **AI PROMPTS BLOCKED - ACKNOWLEDGMENT REQUIRED**\n\n' +
                              '🔒 **STRICT MODE ACTIVE:** All AI prompts are blocked until proper acknowledgment is received.\n\n' +
                              `⏰ **Time since last acknowledgment:** ${Math.round(timeSinceLastAck/1000)} seconds\n\n` +
                              '📋 **REQUIRED ACTION:**\n' +
                              '1. You MUST acknowledge the previous response using:\n' +
                              '   `acknowledge_agent_response`\n' +
                              '   loopId: [your current loop ID]\n' +
                              '   agentResponse: [summary of your response]\n\n' +
                              '2. Only after acknowledgment will AI prompts be available\n\n' +
                              '🚨 **NO AI OPERATIONS WILL PROCEED WITHOUT ACKNOWLEDGMENT**\n\n' +
                              `🔍 **Pending acknowledgments:** ${this.acknowledgmentSystem.pendingAcknowledgments.size}\n` +
                              `🚫 **Blocked operations:** ${this.acknowledgmentSystem.blockedOperations.size}`
                    }]
                };
            }
        }

        // Check for pending acknowledgments for specific loops
        if (this.acknowledgmentSystem.pendingAcknowledgments.size > 0) {
            const pendingLoops = Array.from(this.acknowledgmentSystem.pendingAcknowledgments.keys());
            return {
                content: [{
                    type: 'text',
                    text: '🚨 **AI PROMPTS BLOCKED - PENDING ACKNOWLEDGMENTS**\n\n' +
                          '🔒 **ACKNOWLEDGMENT REQUIRED:** You have pending acknowledgments that must be completed before receiving new AI prompts.\n\n' +
                          `📋 **Pending Loops:** ${pendingLoops.join(', ')}\n\n` +
                          '**REQUIRED ACTION:**\n' +
                          'For each pending loop, you MUST acknowledge using:\n' +
                          '```\n' +
                          'acknowledge_agent_response\n' +
                          'loopId: [loop_id]\n' +
                          'agentResponse: [your response summary]\n' +
                          '```\n\n' +
                          '⚠️  **AI prompts will remain blocked until ALL pending acknowledgments are completed.**'
                }]
            };
        }

        try {
            // Get contextual prompts from AI Agent Coordinator
            const contextualPrompts = this.aiCoordinator.getContextualPrompts(limit);

            // If no contextual prompts available, generate general improvement prompts
            if (contextualPrompts.length === 0) {
                const generalPrompts = await this.generateGeneralImprovementPrompts(limit);
                contextualPrompts.push(...generalPrompts);
            }

            // Ensure we have at least some prompts
            const prompts = contextualPrompts.slice(0, limit);

            if (prompts.length === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: '🤖 **AI-Generated Prompts**\n\n' +
                              '💡 No active operations detected. Start a task analysis or execution to receive contextual prompts.\n\n' +
                              '**Available Operations:**\n' +
                              '• `analyze_task_breakdown` - Get task-specific improvement suggestions\n' +
                              '• `deep_think_implementation` - Receive implementation optimization prompts\n' +
                              '• `parallel_execute_tasks` - Get execution monitoring and optimization prompts'
                    }]
                };
            }

            const promptText = prompts.map((prompt, index) => {
                const priorityIcon = prompt.priority === 'high' ? '🔥' : prompt.priority === 'medium' ? '⚡' : '💡';
                const typeIcon = this.getPromptTypeIcon(prompt.type);

                return `${index + 1}. ${priorityIcon} **${prompt.type.replace('_', ' ').toUpperCase()}**\n` +
                       `   ${typeIcon} ${prompt.content}\n` +
                       `   📋 Context: ${prompt.context}\n` +
                       `   ⏰ Generated: ${new Date(prompt.timestamp).toLocaleString()}\n` +
                       `   🎯 Priority: ${prompt.priority.toUpperCase()}`;
            }).join('\n\n');

            return {
                content: [{
                    type: 'text',
                    text: `🤖 **AI-Generated Contextual Prompts (${prompts.length})**\n\n` +
                          `🔄 Last updated: ${new Date(this.aiCoordinator.lastPromptCheck).toLocaleString()}\n` +
                          `⚡ Active operations: ${this.aiCoordinator.getAllActiveOperations().length}\n\n` +
                          promptText + '\n\n' +
                          `💡 **Tip:** These prompts are generated based on your current AI agent operations and update every 30 seconds.`
                }]
            };
        } catch (error) {
            console.error('❌ Failed to get AI prompts:', error);
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to generate AI prompts: ${error.message}\n\nPlease try again or start an AI agent operation to receive contextual prompts.`
                }]
            };
        }
    }

    async generateGeneralImprovementPrompts(limit = 5) {
        // Generate general improvement prompts when no contextual prompts are available
        const currentTime = Date.now();
        const prompts = [];

        const generalSuggestions = [
            {
                type: 'performance_optimization',
                content: 'Consider implementing caching strategies to improve response times for frequently accessed data',
                priority: 'medium'
            },
            {
                type: 'code_quality',
                content: 'Review current implementation for opportunities to refactor complex functions into smaller, more maintainable units',
                priority: 'low'
            },
            {
                type: 'error_handling',
                content: 'Enhance error handling mechanisms to provide better user feedback and system resilience',
                priority: 'high'
            },
            {
                type: 'monitoring',
                content: 'Add comprehensive logging and monitoring to track system performance and identify potential issues',
                priority: 'medium'
            },
            {
                type: 'security',
                content: 'Conduct security review to ensure proper input validation and protection against common vulnerabilities',
                priority: 'high'
            },
            {
                type: 'documentation',
                content: 'Update documentation to reflect recent changes and improve developer onboarding experience',
                priority: 'low'
            },
            {
                type: 'testing',
                content: 'Expand test coverage to include edge cases and integration scenarios for better reliability',
                priority: 'medium'
            }
        ];

        // Select random suggestions up to the limit
        const selectedSuggestions = generalSuggestions
            .sort(() => Math.random() - 0.5)
            .slice(0, limit);

        selectedSuggestions.forEach((suggestion, index) => {
            prompts.push({
                id: `prompt-general-${currentTime}-${index}`,
                content: suggestion.content,
                type: suggestion.type,
                context: 'general',
                priority: suggestion.priority,
                timestamp: new Date().toISOString()
            });
        });

        return prompts;
    }

    getPromptTypeIcon(type) {
        const icons = {
            'optimization': '⚡',
            'analysis': '🔍',
            'risk_management': '⚠️',
            'exploration': '🚀',
            'edge_case_analysis': '🎯',
            'scalability': '📈',
            'performance': '🏃',
            'synchronization': '🔄',
            'load_balancing': '⚖️',
            'general_improvement': '💡',
            'user_experience': '👤',
            'performance_optimization': '⚡',
            'code_quality': '🔧',
            'error_handling': '🛡️',
            'monitoring': '📊',
            'security': '🔒',
            'documentation': '📚',
            'testing': '🧪'
        };

        return icons[type] || '💡';
    }

    async handleAcknowledgeAgentResponse(args) {
        const { loopId, agentResponse } = args;

        if (!loopId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing loopId parameter. Please provide the loop ID to acknowledge.'
                }]
            };
        }

        const now = Date.now();

        // Update acknowledgment system
        this.acknowledgmentSystem.lastAcknowledgmentTime = now;
        this.acknowledgmentSystem.acknowledgmentRequired = false;
        this.acknowledgmentSystem.currentLoopId = loopId;

        // Remove from pending acknowledgments
        if (this.acknowledgmentSystem.pendingAcknowledgments.has(loopId)) {
            const pendingData = this.acknowledgmentSystem.pendingAcknowledgments.get(loopId);
            const responseTime = now - pendingData.timestamp;

            this.acknowledgmentSystem.pendingAcknowledgments.delete(loopId);
            console.error(`✅ ACKNOWLEDGMENT RECEIVED: Loop ${loopId} acknowledged after ${Math.round(responseTime/1000)}s`);
        }

        // Remove from blocked operations
        this.acknowledgmentSystem.blockedOperations.delete(loopId);

        // Add to acknowledgment history
        this.acknowledgmentSystem.acknowledgmentHistory.push({
            loopId,
            agentResponse: agentResponse || 'No response provided',
            timestamp: now,
            responseTime: this.acknowledgmentSystem.pendingAcknowledgments.has(loopId) ?
                now - this.acknowledgmentSystem.pendingAcknowledgments.get(loopId).timestamp : 0
        });

        // Keep only last 50 acknowledgments
        if (this.acknowledgmentSystem.acknowledgmentHistory.length > 50) {
            this.acknowledgmentSystem.acknowledgmentHistory = this.acknowledgmentSystem.acknowledgmentHistory.slice(-50);
        }

        const loop = this.activeLoops.get(loopId);
        if (!loop) {
            console.log(`⚠️ Loop ${loopId} not found, but acknowledgment processed`);
        }

        // Record the agent response for data collection
        if (agentResponse) {
            await this.dataCollector.collectInteraction({
                type: 'agent-response',
                prompt: `Loop ${loopId} acknowledgment`,
                response: agentResponse,
                context: {
                    loopId,
                    topic: loop?.topic || 'unknown',
                    acknowledgmentTime: now,
                    strictMode: this.acknowledgmentSystem.strictMode
                },
                success: true,
                sessionId: loopId
            });
        }

        // Check if all acknowledgments are complete
        const remainingPending = this.acknowledgmentSystem.pendingAcknowledgments.size;
        const remainingBlocked = this.acknowledgmentSystem.blockedOperations.size;

        let statusMessage = `✅ **ACKNOWLEDGMENT PROCESSED**\n\n` +
                           `🔗 **Loop ID:** ${loopId}\n` +
                           `⏰ **Acknowledged at:** ${new Date(now).toLocaleString()}\n`;

        if (agentResponse) {
            statusMessage += `📝 **Response Summary:** ${agentResponse.substring(0, 200)}${agentResponse.length > 200 ? '...' : ''}\n`;
        }

        statusMessage += `\n📊 **System Status:**\n` +
                        `   • Pending acknowledgments: ${remainingPending}\n` +
                        `   • Blocked operations: ${remainingBlocked}\n` +
                        `   • Strict mode: ${this.acknowledgmentSystem.strictMode ? 'ACTIVE' : 'INACTIVE'}\n`;

        if (remainingPending === 0 && remainingBlocked === 0) {
            statusMessage += `\n🎉 **ALL ACKNOWLEDGMENTS COMPLETE**\n` +
                           `✅ AI prompts are now UNBLOCKED and available\n` +
                           `🔄 You can now use 'get_ai_prompts' to continue`;

            // Disable strict mode if all acknowledgments are complete
            this.acknowledgmentSystem.strictMode = false;
        } else {
            statusMessage += `\n⚠️  **ADDITIONAL ACKNOWLEDGMENTS REQUIRED**\n` +
                           `🔒 AI prompts remain BLOCKED until all acknowledgments are complete`;
        }

        return {
            content: [{
                type: 'text',
                text: statusMessage
            }]
        };
    }

    async handleAIVotingRequest(args) {
        const { prompt, panel = 'general', strategy = 'consensus', maxAgents = 5 } = args;

        if (!prompt || typeof prompt !== 'string') {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing or invalid prompt parameter. Please provide a string prompt for AI voting.'
                }]
            };
        }

        try {
            console.log(`🗳️ Starting AI voting session: Panel=${panel}, Strategy=${strategy}`);

            const votingResult = await this.votingManager.conductVoting(prompt, {
                panel,
                strategy,
                maxAgents,
                enableDebug: true
            });

            // Collect voting data for training
            await this.dataCollector.collectInteraction({
                type: 'ai-voting',
                prompt,
                response: votingResult.winningResponse.content,
                context: {
                    panel,
                    strategy,
                    totalAgents: votingResult.totalAgents,
                    consensusScore: votingResult.consensusScore
                },
                success: true,
                votingData: {
                    votes: votingResult.votes,
                    responses: votingResult.responses.length,
                    winner: votingResult.winningResponse.agentId
                },
                sessionId: `voting-${Date.now()}`
            });

            return {
                content: [{
                    type: 'text',
                    text: `🗳️ **AI Voting Results**\n\n` +
                          `📝 **Prompt:** ${prompt}\n\n` +
                          `🏆 **Winning Response** (${votingResult.winningResponse.agentId}):\n` +
                          `${votingResult.winningResponse.content}\n\n` +
                          `📊 **Voting Details:**\n` +
                          `   Panel: ${panel}\n` +
                          `   Strategy: ${strategy}\n` +
                          `   Total Agents: ${votingResult.totalAgents}\n` +
                          `   Consensus Score: ${(votingResult.consensusScore * 100).toFixed(1)}%\n` +
                          `   Duration: ${votingResult.duration}ms\n\n` +
                          `🗳️ **Vote Distribution:**\n` +
                          votingResult.votes.map(vote =>
                              `   ${vote.voter.id}: Response ${vote.selectedResponseIndex + 1} (confidence: ${vote.confidence}/10)`
                          ).join('\n')
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ AI voting failed: ${error.message}\n\nPlease check your API keys and try again.`
                }]
            };
        }
    }

    async handleGetVotingHistory(args) {
        const { limit = 5 } = args;

        try {
            const history = this.votingManager.getVotingHistory(limit);

            if (history.length === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: '📋 No voting sessions found. Start a voting session with the "ai_voting_request" tool.'
                    }]
                };
            }

            const historyText = history.map((session, index) =>
                `${index + 1}. **Session ${session.sessionId}**\n` +
                `   📝 Prompt: ${session.prompt.substring(0, 100)}${session.prompt.length > 100 ? '...' : ''}\n` +
                `   🏆 Winner: ${session.winner}\n` +
                `   📊 Panel: ${session.panel} | Strategy: ${session.strategy}\n` +
                `   ⏱️ Time: ${new Date(session.timestamp).toLocaleString()}\n` +
                `   🎯 Consensus: ${(session.consensusScore * 100).toFixed(1)}%`
            ).join('\n\n');

            return {
                content: [{
                    type: 'text',
                    text: `🗳️ **Recent Voting Sessions (${history.length})**\n\n${historyText}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get voting history: ${error.message}`
                }]
            };
        }
    }

    async handleGetAgentPerformance(args) {
        const { agentId } = args;

        try {
            const performance = this.votingManager.getAgentPerformance(agentId);

            if (agentId && !performance[agentId]) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ Agent "${agentId}" not found or has no performance data.`
                    }]
                };
            }

            let performanceText;
            if (agentId) {
                const agent = performance[agentId];
                performanceText = `🤖 **Agent Performance: ${agentId}**\n\n` +
                    `📊 **Statistics:**\n` +
                    `   Total Responses: ${agent.totalResponses}\n` +
                    `   Wins: ${agent.wins}\n` +
                    `   Win Rate: ${(agent.winRate * 100).toFixed(1)}%\n` +
                    `   Average Confidence: ${agent.averageConfidence.toFixed(1)}/10\n` +
                    `   Average Response Time: ${agent.averageResponseTime}ms\n\n` +
                    `🏆 **Recent Performance:**\n` +
                    `   Last 5 Sessions: ${agent.recentWins}/5 wins\n` +
                    `   Trend: ${agent.trend}`;
            } else {
                const agents = Object.entries(performance);
                if (agents.length === 0) {
                    return {
                        content: [{
                            type: 'text',
                            text: '📊 No agent performance data available. Run some voting sessions first.'
                        }]
                    };
                }

                performanceText = `🤖 **All Agent Performance**\n\n` +
                    agents.map(([id, agent]) =>
                        `**${id}**\n` +
                        `   Wins: ${agent.wins}/${agent.totalResponses} (${(agent.winRate * 100).toFixed(1)}%)\n` +
                        `   Avg Confidence: ${agent.averageConfidence.toFixed(1)}/10\n` +
                        `   Trend: ${agent.trend}`
                    ).join('\n\n');
            }

            return {
                content: [{
                    type: 'text',
                    text: performanceText
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get agent performance: ${error.message}`
                }]
            };
        }
    }

    async handleAnalyzeTaskBreakdown(args) {
        const { topic, context = {} } = args;

        if (!topic || typeof topic !== 'string') {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing or invalid topic parameter. Please provide a string topic to analyze.'
                }]
            };
        }

        try {
            console.log(`🎯 Analyzing task breakdown for: "${topic}"`);

            // Use AI Agent Coordinator for enhanced interaction management
            const result = await this.aiCoordinator.executeWithCoordination(
                'task_breakdown',
                { topic, context },
                async (params) => {
                    const breakdown = await this.taskManager.analyzeTaskBreakdown(params.topic, params.context);
                    const executionPlan = this.taskManager.getExecutionPlan(breakdown);
                    const optimalStrategy = await this.taskManager.selectOptimalStrategy(breakdown);

                    return { breakdown, executionPlan, optimalStrategy };
                }
            );

            const { breakdown, executionPlan, optimalStrategy } = result;

            // Collect analysis data
            await this.dataCollector.collectInteraction({
                type: 'task_breakdown',
                prompt: `Analyze task breakdown: ${topic}`,
                response: `Generated ${breakdown.subtasks.length} subtasks with ${executionPlan.phases.length} execution phases`,
                context: {
                    topic,
                    subtaskCount: breakdown.subtasks.length,
                    phaseCount: executionPlan.phases.length
                },
                success: true,
                sessionId: breakdown.id
            });

            return {
                content: [{
                    type: 'text',
                    text: `🎯 **Task Breakdown Analysis Complete**\n\n` +
                          `📝 **Main Objective:** ${breakdown.mainObjective}\n\n` +
                          `📋 **Subtasks Identified:** ${breakdown.subtasks.length}\n` +
                          breakdown.subtasks.map((task, i) =>
                              `${i + 1}. **${task.name}**\n   ${task.description}\n   Priority: ${task.priority} | Complexity: ${task.complexity}/10`
                          ).join('\n\n') + '\n\n' +
                          `🚀 **Execution Plan:** ${executionPlan.phases.length} phases\n` +
                          executionPlan.phases.map(phase =>
                              `Phase ${phase.phase}: ${phase.tasks.length} tasks${phase.canRunInParallel ? ' (parallel)' : ' (sequential)'}`
                          ).join('\n') + '\n\n' +
                          `💡 **Optimal Strategy:** ${optimalStrategy}\n\n` +
                          `🔗 **Breakdown ID:** ${breakdown.id}\n` +
                          `Use this ID with 'parallel_execute_tasks' to begin implementation.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Task breakdown analysis failed: ${error.message}\n\nPlease check your input and try again.`
                }]
            };
        }
    }

    async handleDeepThinkImplementation(args) {
        const { taskId, taskName, taskDescription, context = {} } = args;

        // Find task by ID or create temporary task
        let task;
        if (taskId) {
            task = this.taskManager.getTaskStatus(taskId);
            if (!task) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ Task with ID "${taskId}" not found. Please use 'analyze_task_breakdown' first or provide taskName and taskDescription.`
                    }]
                };
            }
        } else if (taskName && taskDescription) {
            task = {
                id: `temp-${Date.now()}`,
                name: taskName,
                description: taskDescription
            };
        } else {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Please provide either taskId or both taskName and taskDescription.'
                }]
            };
        }

        try {
            console.log(`🧠 Starting deep thinking analysis for: "${task.name}"`);

            // Use AI Agent Coordinator for enhanced interaction management
            const result = await this.aiCoordinator.executeWithCoordination(
                'deep_thinking',
                { taskId, taskName, taskDescription, context },
                async (params) => {
                    return await this.deepThinking.deepThinkImplementation(task, params.context);
                }
            );

            const thinkingSession = result;

            // Collect thinking data
            await this.dataCollector.collectInteraction({
                type: 'deep_thinking',
                prompt: `Deep think implementation: ${task.name}`,
                response: `Completed ${thinkingSession.phases.length} analysis phases`,
                context: {
                    taskId: task.id,
                    taskName: task.name,
                    phaseCount: thinkingSession.phases.length,
                    duration: thinkingSession.duration
                },
                success: true,
                sessionId: thinkingSession.id
            });

            return {
                content: [{
                    type: 'text',
                    text: `🧠 **Deep Thinking Analysis Complete**\n\n` +
                          `📝 **Task:** ${task.name}\n` +
                          `⏱️ **Duration:** ${thinkingSession.duration}ms\n` +
                          `🔍 **Analysis Phases:** ${thinkingSession.phases.length}\n\n` +
                          `📊 **Analysis Summary:**\n` +
                          thinkingSession.phases.map(phase =>
                              `• ${phase.phase.replace('_', ' ').toUpperCase()}: Completed`
                          ).join('\n') + '\n\n' +
                          `💡 **Key Recommendations:**\n${thinkingSession.recommendations?.content || 'Analysis in progress'}\n\n` +
                          `🔗 **Session ID:** ${thinkingSession.id}\n` +
                          `Use this session data for implementation planning.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Deep thinking analysis failed: ${error.message}\n\nPlease check your input and try again.`
                }]
            };
        }
    }

    async handleParallelExecuteTasks(args) {
        const { breakdownId, executionStrategy = 'parallel', maxConcurrency = 5 } = args;

        if (!breakdownId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing breakdownId parameter. Please provide a breakdown ID from a previous task analysis.'
                }]
            };
        }

        // Find the breakdown
        const breakdown = this.taskManager.activeTasks.get(breakdownId);
        if (!breakdown) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Breakdown with ID "${breakdownId}" not found. Please run 'analyze_task_breakdown' first.`
                }]
            };
        }

        try {
            console.log(`⚡ Starting parallel execution for breakdown: ${breakdown.mainObjective}`);

            // Use AI Agent Coordinator for enhanced interaction management
            const result = await this.aiCoordinator.executeWithCoordination(
                'parallel_execution',
                { breakdownId, executionStrategy, maxConcurrency },
                async (params) => {
                    const executionPlan = this.taskManager.getExecutionPlan(breakdown);
                    return await this.parallelImplementation.parallelExecuteTasks(breakdown, executionPlan);
                }
            );

            const execution = result;

            // Collect execution data
            await this.dataCollector.collectInteraction({
                type: 'parallel_execution',
                prompt: `Execute tasks in parallel: ${breakdown.mainObjective}`,
                response: `Executed ${breakdown.subtasks.length} tasks across ${execution.phases.length} phases`,
                context: {
                    breakdownId,
                    executionStrategy,
                    maxConcurrency,
                    taskCount: breakdown.subtasks.length,
                    phaseCount: execution.phases.length,
                    duration: execution.duration
                },
                success: execution.status === 'completed',
                sessionId: execution.id
            });

            const completedTasks = execution.completedTasks.size;
            const failedTasks = execution.failedTasks.size;
            const discoveredTasks = execution.discoveredTasks.length;

            return {
                content: [{
                    type: 'text',
                    text: `⚡ **Parallel Execution ${execution.status === 'completed' ? 'Complete' : 'Failed'}**\n\n` +
                          `📋 **Project:** ${breakdown.mainObjective}\n` +
                          `⏱️ **Duration:** ${execution.duration}ms\n` +
                          `📊 **Execution Summary:**\n` +
                          `   • Completed Tasks: ${completedTasks}/${breakdown.subtasks.length}\n` +
                          `   • Failed Tasks: ${failedTasks}\n` +
                          `   • Phases Executed: ${execution.phases.length}\n` +
                          `   • Discovered Tasks: ${discoveredTasks}\n\n` +
                          `🔄 **Phase Results:**\n` +
                          execution.phases.map(phase =>
                              `Phase ${phase.phase}: ${phase.tasks.filter(t => t.status === 'completed').length}/${phase.tasks.length} tasks completed`
                          ).join('\n') + '\n\n' +
                          (discoveredTasks > 0 ?
                              `💡 **Discovered Additional Tasks:**\n` +
                              execution.discoveredTasks.slice(0, 5).map(task =>
                                  `• ${task.name}: ${task.description}`
                              ).join('\n') +
                              (discoveredTasks > 5 ? `\n... and ${discoveredTasks - 5} more` : '') + '\n\n' : '') +
                          `🔗 **Execution ID:** ${execution.id}\n` +
                          `Status: ${execution.status}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Parallel execution failed: ${error.message}\n\nPlease check the breakdown ID and try again.`
                }]
            };
        }
    }

    async handleGetCacheAnalytics(args) {
        try {
            const analytics = this.smartCache.getCacheAnalytics();

            return {
                content: [{
                    type: 'text',
                    text: `💾 **Smart Cache Analytics**\n\n` +
                          `📊 **Performance Metrics:**\n` +
                          `   • Total Requests: ${analytics.totalRequests}\n` +
                          `   • Cache Hits: ${analytics.hits}\n` +
                          `   • Cache Misses: ${analytics.misses}\n` +
                          `   • Hit Rate: ${analytics.hitRate}%\n` +
                          `   • Cache Size: ${analytics.cacheSize} items\n\n` +
                          `💰 **Cost Savings:**\n` +
                          `   • Total Cost Saved: $${analytics.costSaved.toFixed(4)}\n` +
                          `   • Cost Savings: ${analytics.costSavingsPercentage}%\n` +
                          `   • Time Saved: ${(analytics.timesSaved / 1000).toFixed(1)} seconds\n\n` +
                          `⚡ **Performance:**\n` +
                          `   • Average Response Time: ${analytics.averageResponseTime.toFixed(0)}ms\n` +
                          `   • Cache Efficiency: ${analytics.hitRate > 70 ? 'Excellent' : analytics.hitRate > 50 ? 'Good' : 'Needs Improvement'}\n\n` +
                          `💡 **Recommendations:**\n` +
                          `${analytics.hitRate < 50 ? '   • Consider adjusting similarity threshold\n' : ''}` +
                          `${analytics.cacheSize < 100 ? '   • Cache is building up, performance will improve\n' : ''}` +
                          `${analytics.costSaved > 1 ? '   • Excellent cost savings! Cache is working well\n' : '   • Cache is still learning, savings will increase\n'}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get cache analytics: ${error.message}`
                }]
            };
        }
    }

    async handleClearCache(args) {
        const { confirm } = args;

        if (!confirm) {
            return {
                content: [{
                    type: 'text',
                    text: '⚠️ Cache clear requires confirmation. Use `clear_cache` with `confirm: true` to proceed.\n\n' +
                          '**Warning:** This will remove all cached responses and reset analytics.'
                }]
            };
        }

        try {
            await this.smartCache.clearCache();

            return {
                content: [{
                    type: 'text',
                    text: '🗑️ **Cache Cleared Successfully**\n\n' +
                          '✅ All cached responses removed\n' +
                          '✅ Analytics reset to zero\n' +
                          '✅ Cache directory cleaned\n\n' +
                          '💡 The cache will start building up again with new requests.'
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to clear cache: ${error.message}`
                }]
            };
        }
    }

    async handleOptimizeCache(args) {
        try {
            const beforeSize = this.smartCache.cache.size;
            await this.smartCache.optimizeCache();
            const afterSize = this.smartCache.cache.size;
            const removed = beforeSize - afterSize;

            return {
                content: [{
                    type: 'text',
                    text: `✨ **Cache Optimization Complete**\n\n` +
                          `📊 **Results:**\n` +
                          `   • Items Before: ${beforeSize}\n` +
                          `   • Items After: ${afterSize}\n` +
                          `   • Items Removed: ${removed}\n` +
                          `   • Space Freed: ${((removed / beforeSize) * 100).toFixed(1)}%\n\n` +
                          `🎯 **Optimization Strategy:**\n` +
                          `   • Removed low-access items\n` +
                          `   • Kept frequently used responses\n` +
                          `   • Preserved recent cache entries\n\n` +
                          `⚡ Cache performance should be improved!`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to optimize cache: ${error.message}`
                }]
            };
        }
    }

    async handleCreateProject(args) {
        const { projectName, context = {} } = args;

        if (!projectName || typeof projectName !== 'string') {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing or invalid projectName parameter. Please provide a string project name.'
                }]
            };
        }

        try {
            const projectId = await this.projectMemory.createProject(projectName, context);

            // Add context entry for project creation
            this.projectMemory.addContextEntry('project_created', {
                projectId,
                projectName,
                context
            });

            return {
                content: [{
                    type: 'text',
                    text: `📝 **Project Created Successfully**\n\n` +
                          `🆔 **Project ID:** ${projectId}\n` +
                          `📋 **Name:** ${projectName}\n` +
                          `🏷️ **Type:** ${context.type || 'general'}\n` +
                          `🌐 **Domain:** ${context.domain || 'unknown'}\n` +
                          `📅 **Created:** ${new Date().toLocaleString()}\n\n` +
                          `💡 **Next Steps:**\n` +
                          `   • Use this project ID for context-aware operations\n` +
                          `   • The system will remember your preferences and strategies\n` +
                          `   • Project history will be automatically tracked\n\n` +
                          `🔗 Use \`get_project_memory\` with this ID to retrieve project context.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to create project: ${error.message}`
                }]
            };
        }
    }

    async handleGetProjectMemory(args) {
        const { projectId } = args;

        if (!projectId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing projectId parameter. Please provide a project ID.'
                }]
            };
        }

        try {
            const project = await this.projectMemory.getProject(projectId);

            if (!project) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ Project with ID "${projectId}" not found.\n\nUse \`create_project\` to create a new project.`
                    }]
                };
            }

            const recentHistory = project.history.slice(-5);
            const strategies = this.projectMemory.getRecommendedStrategies(project.context, 3);

            return {
                content: [{
                    type: 'text',
                    text: `📝 **Project Memory: ${project.name}**\n\n` +
                          `🆔 **ID:** ${project.id}\n` +
                          `📊 **Status:** ${project.status}\n` +
                          `🏷️ **Type:** ${project.type}\n` +
                          `🌐 **Domain:** ${project.domain}\n` +
                          `📅 **Created:** ${new Date(project.createdAt).toLocaleString()}\n` +
                          `🕒 **Last Accessed:** ${new Date(project.lastAccessed).toLocaleString()}\n\n` +
                          `📈 **Metrics:**\n` +
                          `   • Total Tasks: ${project.metrics.totalTasks}\n` +
                          `   • Completed: ${project.metrics.completedTasks}\n` +
                          `   • Success Rate: ${(project.metrics.successRate * 100).toFixed(1)}%\n` +
                          `   • Avg Completion Time: ${project.metrics.averageCompletionTime}ms\n\n` +
                          `📚 **Recent History (${recentHistory.length}/5):**\n` +
                          recentHistory.map(h =>
                              `   • ${new Date(h.timestamp).toLocaleString()}: ${h.description} (${h.outcome})`
                          ).join('\n') + '\n\n' +
                          `💡 **Recommended Strategies:**\n` +
                          strategies.map(s =>
                              `   • ${s.name} (${(s.successRate * 100).toFixed(1)}% success rate)`
                          ).join('\n')
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to retrieve project memory: ${error.message}`
                }]
            };
        }
    }

    async handleSetUserPreference(args) {
        const { key, value, context = {} } = args;

        if (!key || value === undefined) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing key or value parameters. Please provide both key and value for the preference.'
                }]
            };
        }

        try {
            const preference = await this.projectMemory.setUserPreference(key, value, context);

            // Add context entry for preference setting
            this.projectMemory.addContextEntry('preference_set', {
                key,
                value,
                context
            });

            return {
                content: [{
                    type: 'text',
                    text: `⚙️ **User Preference Set**\n\n` +
                          `🔑 **Key:** ${key}\n` +
                          `💎 **Value:** ${JSON.stringify(value)}\n` +
                          `📊 **Usage Count:** ${preference.usageCount}\n` +
                          `📅 **Created:** ${new Date(preference.createdAt).toLocaleString()}\n\n` +
                          `✅ Preference saved and will be used for personalization across sessions.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to set user preference: ${error.message}`
                }]
            };
        }
    }

    async handleGetMemoryAnalytics(args) {
        try {
            const analytics = this.projectMemory.getMemoryAnalytics();

            return {
                content: [{
                    type: 'text',
                    text: `🧠 **Project Memory Analytics**\n\n` +
                          `📝 **Projects:**\n` +
                          `   • Total: ${analytics.projects.total}\n` +
                          `   • Active: ${analytics.projects.active}\n` +
                          `   • Completed: ${analytics.projects.completed}\n` +
                          `   • Avg Success Rate: ${(analytics.projects.averageSuccessRate * 100).toFixed(1)}%\n\n` +
                          `⚙️ **User Preferences:**\n` +
                          `   • Total: ${analytics.preferences.total}\n` +
                          `   • Most Used:\n` +
                          analytics.preferences.mostUsed.map(p =>
                              `     - ${p.key}: ${p.usageCount} times`
                          ).join('\n') + '\n\n' +
                          `💡 **Successful Strategies:**\n` +
                          `   • Total: ${analytics.strategies.total}\n` +
                          `   • Avg Success Rate: ${(analytics.strategies.averageSuccessRate * 100).toFixed(1)}%\n` +
                          `   • Top Strategies:\n` +
                          analytics.strategies.topStrategies.map(s =>
                              `     - ${s.name}: ${(s.successRate * 100).toFixed(1)}% (${s.usageCount} uses)`
                          ).join('\n') + '\n\n' +
                          `📚 **Context History:**\n` +
                          `   • Total Entries: ${analytics.context.totalEntries}\n` +
                          `   • Current Session Duration: ${Math.round(analytics.context.currentSessionDuration / 1000 / 60)} minutes\n` +
                          `   • Session Interactions: ${analytics.context.currentSessionInteractions}\n\n` +
                          `🎯 **Memory System Performance:** ${analytics.projects.total > 0 ? 'Active' : 'Building'}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get memory analytics: ${error.message}`
                }]
            };
        }
    }

    async handleGetRecommendedStrategies(args) {
        const { context = {}, limit = 5 } = args;

        try {
            const strategies = this.projectMemory.getRecommendedStrategies(context, limit);

            if (strategies.length === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: `💡 **No Recommended Strategies Found**\n\n` +
                              `The system hasn't learned enough successful strategies yet.\n\n` +
                              `**To build strategy recommendations:**\n` +
                              `• Complete more projects successfully\n` +
                              `• Use consistent project contexts\n` +
                              `• Record project outcomes\n\n` +
                              `The AI will learn from your patterns and suggest optimal approaches.`
                    }]
                };
            }

            return {
                content: [{
                    type: 'text',
                    text: `💡 **Recommended Strategies**\n\n` +
                          `📊 **Based on context:** ${JSON.stringify(context)}\n\n` +
                          strategies.map((strategy, index) =>
                              `${index + 1}. **${strategy.name}**\n` +
                              `   📈 Success Rate: ${(strategy.successRate * 100).toFixed(1)}%\n` +
                              `   🎯 Score: ${(strategy.score * 100).toFixed(1)}%\n` +
                              `   ⏱️ Avg Time: ${strategy.averageTime}ms\n` +
                              `   🔄 Used: ${strategy.successCount} times\n` +
                              `   📅 Last Used: ${new Date(strategy.lastUsed).toLocaleString()}`
                          ).join('\n\n') + '\n\n' +
                          `🎯 **Recommendation:** Start with the highest-scored strategy for best results.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get recommended strategies: ${error.message}`
                }]
            };
        }
    }

    async handleGetModelAnalytics(args) {
        const { taskType, metric = 'overall' } = args;

        try {
            const rankings = this.modelAnalytics.getModelRankings(taskType, metric);
            const summary = this.modelAnalytics.getAnalyticsSummary();

            if (rankings.length === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: `📊 **AI Model Analytics**\n\n` +
                              `No model performance data available yet.\n\n` +
                              `**To build analytics:**\n` +
                              `• Use AI models for various tasks\n` +
                              `• Complete requests successfully\n` +
                              `• The system will automatically track performance\n\n` +
                              `Analytics will show model rankings, cost analysis, and performance trends.`
                    }]
                };
            }

            const topModels = rankings.slice(0, 5);

            return {
                content: [{
                    type: 'text',
                    text: `📊 **AI Model Performance Analytics**\n\n` +
                          `🎯 **Filter:** ${taskType || 'All Tasks'} | Metric: ${metric}\n\n` +
                          `📈 **Overall Summary:**\n` +
                          `   • Total Models: ${summary.overview.totalModels}\n` +
                          `   • Total Requests: ${summary.overview.totalRequests}\n` +
                          `   • Success Rate: ${(summary.overview.overallSuccessRate * 100).toFixed(1)}%\n` +
                          `   • Total Cost: $${summary.overview.totalCost.toFixed(4)}\n\n` +
                          `🏆 **Top Performing Models:**\n` +
                          topModels.map((model, index) =>
                              `${index + 1}. **${model.name}**\n` +
                              `   📊 Score: ${model.score.toFixed(2)}\n` +
                              `   ✅ Success Rate: ${(model.successRate * 100).toFixed(1)}%\n` +
                              `   ⚡ Avg Response: ${model.avgResponseTime.toFixed(0)}ms\n` +
                              `   💰 Avg Cost: $${model.avgCost.toFixed(4)}\n` +
                              `   🎯 Quality: ${model.averageQuality.toFixed(2)}/10\n` +
                              `   📋 Task Types: ${model.taskTypesArray.join(', ')}`
                          ).join('\n\n') + '\n\n' +
                          `💡 **Recommendation:** Use ${topModels[0].name} for optimal ${metric} performance.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get model analytics: ${error.message}`
                }]
            };
        }
    }

    async handleGetCostAnalysis(args) {
        const { timeframe = 'month' } = args;

        try {
            const costAnalysis = this.modelAnalytics.getCostAnalysis(timeframe);

            if (costAnalysis.totalRequests === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: `💰 **Cost Analysis (${timeframe})**\n\n` +
                              `No cost data available for the selected timeframe.\n\n` +
                              `**To build cost analysis:**\n` +
                              `• Use AI models for requests\n` +
                              `• The system tracks token usage and costs\n` +
                              `• View detailed breakdowns by model and task type`
                    }]
                };
            }

            const modelCosts = Object.entries(costAnalysis.costByModel)
                .sort((a, b) => b[1].cost - a[1].cost)
                .slice(0, 5);

            const taskCosts = Object.entries(costAnalysis.costByTaskType)
                .sort((a, b) => b[1].cost - a[1].cost)
                .slice(0, 5);

            return {
                content: [{
                    type: 'text',
                    text: `💰 **Cost Analysis (${timeframe})**\n\n` +
                          `📊 **Summary:**\n` +
                          `   • Total Cost: $${costAnalysis.totalCost.toFixed(4)}\n` +
                          `   • Total Requests: ${costAnalysis.totalRequests}\n` +
                          `   • Avg Cost/Request: $${costAnalysis.avgCostPerRequest.toFixed(6)}\n\n` +
                          `🤖 **Cost by Model:**\n` +
                          modelCosts.map(([model, data]) =>
                              `   • ${model}: $${data.cost.toFixed(4)} (${data.requests} requests)`
                          ).join('\n') + '\n\n' +
                          `📋 **Cost by Task Type:**\n` +
                          taskCosts.map(([task, data]) =>
                              `   • ${task}: $${data.cost.toFixed(4)} (${data.requests} requests)`
                          ).join('\n') + '\n\n' +
                          `💡 **Optimization:** Consider using lower-cost models for simple tasks.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get cost analysis: ${error.message}`
                }]
            };
        }
    }

    async handleGetRecommendedModel(args) {
        const { taskType, priorities = { cost: 0.3, speed: 0.3, quality: 0.4 } } = args;

        if (!taskType) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing taskType parameter. Please specify the type of task for model recommendation.'
                }]
            };
        }

        try {
            const recommendation = this.modelAnalytics.getRecommendedModel(taskType, priorities);

            if (!recommendation) {
                return {
                    content: [{
                        type: 'text',
                        text: `🤖 **Model Recommendation for "${taskType}"**\n\n` +
                              `No performance data available for this task type yet.\n\n` +
                              `**To get recommendations:**\n` +
                              `• Use different AI models for "${taskType}" tasks\n` +
                              `• Complete several requests successfully\n` +
                              `• The system will learn which models perform best\n\n` +
                              `💡 **Suggestion:** Try starting with GPT-4 or Claude-3-Sonnet for complex tasks.`
                    }]
                };
            }

            const taskAnalysis = this.modelAnalytics.getTaskTypeAnalysis(taskType);

            return {
                content: [{
                    type: 'text',
                    text: `🤖 **Model Recommendation for "${taskType}"**\n\n` +
                          `🏆 **Recommended Model: ${recommendation.model}**\n` +
                          `   📊 Overall Score: ${recommendation.weightedScore.toFixed(2)}\n` +
                          `   ✅ Success Rate: ${(recommendation.successRate * 100).toFixed(1)}%\n` +
                          `   ⚡ Avg Response Time: ${recommendation.avgResponseTime.toFixed(0)}ms\n` +
                          `   💰 Avg Cost: $${recommendation.avgCost.toFixed(4)}\n` +
                          `   🎯 Quality Score: ${recommendation.avgQuality.toFixed(2)}/10\n` +
                          `   📈 Requests: ${recommendation.requests}\n\n` +
                          `⚖️ **Priority Weights:**\n` +
                          `   • Cost: ${(priorities.cost * 100).toFixed(0)}%\n` +
                          `   • Speed: ${(priorities.speed * 100).toFixed(0)}%\n` +
                          `   • Quality: ${(priorities.quality * 100).toFixed(0)}%\n\n` +
                          `📊 **Task Type Analysis:**\n` +
                          `   • Total Requests: ${taskAnalysis.totalRequests}\n` +
                          `   • Avg Response Time: ${taskAnalysis.averageResponseTime.toFixed(0)}ms\n` +
                          `   • Avg Cost: $${taskAnalysis.averageCost.toFixed(4)}\n` +
                          `   • Success Rate: ${(taskAnalysis.successRate * 100).toFixed(1)}%\n\n` +
                          `💡 **Alternative Models:**\n` +
                          taskAnalysis.modelPerformance.slice(1, 4).map((model, index) =>
                              `   ${index + 2}. ${model.model} (Score: ${model.score.toFixed(2)})`
                          ).join('\n')
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get model recommendation: ${error.message}`
                }]
            };
        }
    }

    async handleGetPerformanceTrends(args) {
        const { metric = 'responseTime', timeframe = 'week' } = args;

        try {
            const trends = this.modelAnalytics.getPerformanceTrends(metric, timeframe);

            if (trends.length === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: `📈 **Performance Trends (${metric} - ${timeframe})**\n\n` +
                              `No trend data available for the selected timeframe.\n\n` +
                              `**To build trend analysis:**\n` +
                              `• Use AI models consistently over time\n` +
                              `• The system tracks performance metrics\n` +
                              `• View trends for responseTime, successRate, qualityScore, cost`
                    }]
                };
            }

            // Calculate trend direction
            const firstValue = trends[0].value;
            const lastValue = trends[trends.length - 1].value;
            const trendDirection = lastValue > firstValue ? '📈 Increasing' :
                                 lastValue < firstValue ? '📉 Decreasing' : '➡️ Stable';
            const trendPercentage = firstValue > 0 ? ((lastValue - firstValue) / firstValue * 100).toFixed(1) : 0;

            // Get min/max values
            const values = trends.map(t => t.value);
            const minValue = Math.min(...values);
            const maxValue = Math.max(...values);
            const avgValue = values.reduce((sum, v) => sum + v, 0) / values.length;

            return {
                content: [{
                    type: 'text',
                    text: `📈 **Performance Trends: ${metric} (${timeframe})**\n\n` +
                          `📊 **Trend Summary:**\n` +
                          `   • Direction: ${trendDirection} (${trendPercentage}%)\n` +
                          `   • Average: ${avgValue.toFixed(metric === 'cost' ? 4 : 2)}${metric === 'responseTime' ? 'ms' : metric === 'cost' ? '$' : ''}\n` +
                          `   • Min: ${minValue.toFixed(metric === 'cost' ? 4 : 2)}${metric === 'responseTime' ? 'ms' : metric === 'cost' ? '$' : ''}\n` +
                          `   • Max: ${maxValue.toFixed(metric === 'cost' ? 4 : 2)}${metric === 'responseTime' ? 'ms' : metric === 'cost' ? '$' : ''}\n` +
                          `   • Data Points: ${trends.length}\n\n` +
                          `📅 **Recent Data Points:**\n` +
                          trends.slice(-10).map(trend =>
                              `   • ${new Date(trend.timestamp).toLocaleString()}: ${trend.value.toFixed(metric === 'cost' ? 4 : 2)}${metric === 'responseTime' ? 'ms' : metric === 'cost' ? '$' : ''} (${trend.count} requests)`
                          ).join('\n') + '\n\n' +
                          `💡 **Insights:**\n` +
                          `${metric === 'responseTime' && trendDirection.includes('Increasing') ? '   • Response times are increasing - consider optimizing or switching models\n' : ''}` +
                          `${metric === 'cost' && trendDirection.includes('Increasing') ? '   • Costs are rising - review model selection and usage patterns\n' : ''}` +
                          `${metric === 'successRate' && trendDirection.includes('Decreasing') ? '   • Success rates are declining - investigate model performance issues\n' : ''}` +
                          `${metric === 'qualityScore' && trendDirection.includes('Increasing') ? '   • Quality is improving - current strategy is working well\n' : ''}` +
                          `   • Use this data to optimize model selection and usage patterns`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get performance trends: ${error.message}`
                }]
            };
        }
    }

    async handleListWorkflowTemplates(args) {
        const { category, framework, complexity, search } = args;

        try {
            let templates;

            if (search || category || framework || complexity) {
                // Use search/filter functionality
                const filters = {};
                if (category) filters.category = category;
                if (framework) filters.framework = framework;
                if (complexity) filters.complexity = complexity;

                templates = this.workflowTemplates.searchTemplates(search || '', filters);
            } else {
                // Get all templates
                templates = this.workflowTemplates.getAllTemplates();
            }

            if (templates.length === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: `📋 **Workflow Templates**\n\n` +
                              `No templates found matching your criteria.\n\n` +
                              `**Available filters:**\n` +
                              `• Category: web_development, api_development, data_analysis, machine_learning, devops\n` +
                              `• Framework: react, nodejs, python, docker, etc.\n` +
                              `• Complexity: low, medium, high\n` +
                              `• Search: keywords in name, description, or tags\n\n` +
                              `Try adjusting your filters or use \`get_template_recommendations\` for suggestions.`
                    }]
                };
            }

            const templateList = templates.slice(0, 10).map((template, index) => {
                const usageCount = template.usageCount || 0;
                const typeIcon = template.type === 'built-in' ? '🏗️' : '👤';
                const complexityIcon = template.complexity === 'high' ? '🔴' :
                                     template.complexity === 'medium' ? '🟡' : '🟢';

                return `${index + 1}. ${typeIcon} **${template.name}**\n` +
                       `   📋 ${template.description}\n` +
                       `   🏷️ Category: ${template.category}\n` +
                       `   ⚙️ Framework: ${template.framework || 'N/A'}\n` +
                       `   ${complexityIcon} Complexity: ${template.complexity || 'N/A'}\n` +
                       `   ⏱️ Est. Time: ${template.estimatedTime || 'N/A'}\n` +
                       `   📊 Used: ${usageCount} times\n` +
                       `   🏷️ Tags: ${template.tags ? template.tags.join(', ') : 'None'}\n` +
                       `   🆔 ID: ${template.id}`;
            }).join('\n\n');

            const totalCount = templates.length;
            const showing = Math.min(10, totalCount);

            return {
                content: [{
                    type: 'text',
                    text: `📋 **Workflow Templates (${showing}/${totalCount})**\n\n` +
                          `${search ? `🔍 Search: "${search}"\n` : ''}` +
                          `${category ? `📂 Category: ${category}\n` : ''}` +
                          `${framework ? `⚙️ Framework: ${framework}\n` : ''}` +
                          `${complexity ? `🎯 Complexity: ${complexity}\n` : ''}` +
                          `\n${templateList}\n\n` +
                          `💡 **Tips:**\n` +
                          `• Use \`get_workflow_template\` with an ID for detailed steps\n` +
                          `• Use \`get_template_recommendations\` for personalized suggestions\n` +
                          `• Create custom templates with \`create_custom_template\``
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to list workflow templates: ${error.message}`
                }]
            };
        }
    }

    async handleGetWorkflowTemplate(args) {
        const { templateId } = args;

        if (!templateId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing templateId parameter. Please provide a template ID.'
                }]
            };
        }

        try {
            const template = this.workflowTemplates.getTemplate(templateId);

            if (!template) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ Template "${templateId}" not found.\n\nUse \`list_workflow_templates\` to see available templates.`
                    }]
                };
            }

            // Record template usage
            this.workflowTemplates.recordTemplateUsage(templateId);

            const typeIcon = template.type === 'built-in' ? '🏗️' : '👤';
            const complexityIcon = template.complexity === 'high' ? '🔴' :
                                 template.complexity === 'medium' ? '🟡' : '🟢';

            const stepsText = template.steps.map((step, index) => {
                const tasksText = step.tasks.map(task => `     • ${task}`).join('\n');

                return `**${index + 1}. ${step.title}**\n` +
                       `   📝 ${step.description}\n` +
                       `   ⏱️ Estimated Time: ${step.estimatedTime || 'N/A'}\n` +
                       `   📋 Tasks:\n${tasksText}`;
            }).join('\n\n');

            const dependenciesText = template.dependencies ?
                template.dependencies.map(dep => `   • ${dep}`).join('\n') : '   • None specified';

            const resourcesText = template.resources ?
                template.resources.map(resource => `   • ${resource}`).join('\n') : '   • None provided';

            return {
                content: [{
                    type: 'text',
                    text: `${typeIcon} **${template.name}**\n\n` +
                          `📝 **Description:** ${template.description}\n\n` +
                          `📊 **Template Details:**\n` +
                          `   🏷️ Category: ${template.category}\n` +
                          `   ⚙️ Framework: ${template.framework || 'N/A'}\n` +
                          `   ${complexityIcon} Complexity: ${template.complexity || 'N/A'}\n` +
                          `   ⏱️ Estimated Time: ${template.estimatedTime || 'N/A'}\n` +
                          `   📊 Usage Count: ${template.usageCount || 0}\n` +
                          `   🏷️ Tags: ${template.tags ? template.tags.join(', ') : 'None'}\n\n` +
                          `🛠️ **Dependencies:**\n${dependenciesText}\n\n` +
                          `📋 **Implementation Steps:**\n\n${stepsText}\n\n` +
                          `📚 **Resources:**\n${resourcesText}\n\n` +
                          `💡 **Next Steps:**\n` +
                          `• Follow the steps in order for best results\n` +
                          `• Adjust tasks based on your specific requirements\n` +
                          `• Use the resources for additional guidance\n` +
                          `• Consider creating a project with \`create_project\` to track progress`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get workflow template: ${error.message}`
                }]
            };
        }
    }

    async handleCreateCustomTemplate(args) {
        const { name, category, description, framework, complexity, estimatedTime, tags, steps, dependencies, resources } = args;

        if (!name || !category || !description || !steps) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide: name, category, description, and steps.'
                }]
            };
        }

        try {
            const templateData = {
                name,
                category,
                description,
                framework,
                complexity,
                estimatedTime,
                tags: tags || [],
                steps,
                dependencies: dependencies || [],
                resources: resources || []
            };

            const templateId = await this.workflowTemplates.createCustomTemplate(templateData);

            return {
                content: [{
                    type: 'text',
                    text: `👤 **Custom Template Created Successfully**\n\n` +
                          `🆔 **Template ID:** ${templateId}\n` +
                          `📋 **Name:** ${name}\n` +
                          `🏷️ **Category:** ${category}\n` +
                          `📝 **Description:** ${description}\n` +
                          `📊 **Steps:** ${steps.length}\n\n` +
                          `✅ Your custom template has been saved and is now available for use.\n\n` +
                          `💡 **Next Steps:**\n` +
                          `• Use \`get_workflow_template\` with ID "${templateId}" to view details\n` +
                          `• Share the template ID with team members\n` +
                          `• The template will appear in \`list_workflow_templates\` results`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to create custom template: ${error.message}`
                }]
            };
        }
    }

    async handleGetTemplateRecommendations(args) {
        const { context = {}, limit = 5 } = args;

        try {
            const recommendations = this.workflowTemplates.getRecommendedTemplates(context, limit);

            if (recommendations.length === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: `💡 **Template Recommendations**\n\n` +
                              `No specific recommendations available yet.\n\n` +
                              `**To get better recommendations:**\n` +
                              `• Provide context (category, framework, tags)\n` +
                              `• Use templates to build usage patterns\n` +
                              `• Create custom templates for your workflows\n\n` +
                              `**Example context:**\n` +
                              `\`\`\`json\n` +
                              `{\n` +
                              `  "category": "web_development",\n` +
                              `  "framework": "react",\n` +
                              `  "tags": ["dashboard", "typescript"]\n` +
                              `}\n` +
                              `\`\`\``
                    }]
                };
            }

            const recommendationsList = recommendations.map((template, index) => {
                const typeIcon = template.type === 'built-in' ? '🏗️' : '👤';
                const complexityIcon = template.complexity === 'high' ? '🔴' :
                                     template.complexity === 'medium' ? '🟡' : '🟢';

                return `${index + 1}. ${typeIcon} **${template.name}** (Score: ${template.score.toFixed(1)})\n` +
                       `   📝 ${template.description}\n` +
                       `   🏷️ ${template.category} | ⚙️ ${template.framework || 'N/A'} | ${complexityIcon} ${template.complexity || 'N/A'}\n` +
                       `   📊 Used ${template.usageCount || 0} times | 🆔 ${template.id}`;
            }).join('\n\n');

            return {
                content: [{
                    type: 'text',
                    text: `💡 **Template Recommendations**\n\n` +
                          `🎯 **Context:** ${JSON.stringify(context)}\n\n` +
                          `📋 **Recommended Templates:**\n\n${recommendationsList}\n\n` +
                          `💡 **Tips:**\n` +
                          `• Higher scores indicate better matches for your context\n` +
                          `• Use \`get_workflow_template\` with an ID for detailed steps\n` +
                          `• Recommendations improve as you use more templates`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get template recommendations: ${error.message}`
                }]
            };
        }
    }

    async handleGetTemplateAnalytics(args) {
        try {
            const analytics = this.workflowTemplates.getTemplateAnalytics();

            const categoryList = Object.entries(analytics.categoryStats)
                .sort((a, b) => b[1] - a[1])
                .map(([category, count]) => `   • ${category}: ${count}`)
                .join('\n');

            const frameworkList = Object.entries(analytics.frameworkStats)
                .sort((a, b) => b[1] - a[1])
                .slice(0, 5)
                .map(([framework, count]) => `   • ${framework}: ${count}`)
                .join('\n');

            const complexityList = Object.entries(analytics.complexityStats)
                .map(([complexity, count]) => `   • ${complexity}: ${count}`)
                .join('\n');

            const mostUsedList = analytics.mostUsedTemplates.length > 0 ?
                analytics.mostUsedTemplates.map((template, index) =>
                    `   ${index + 1}. ${template.name}: ${template.usageCount} uses`
                ).join('\n') : '   • No usage data yet';

            return {
                content: [{
                    type: 'text',
                    text: `📊 **Workflow Template Analytics**\n\n` +
                          `📈 **Overview:**\n` +
                          `   • Total Templates: ${analytics.totalTemplates}\n` +
                          `   • Built-in Templates: ${analytics.builtInCount}\n` +
                          `   • Custom Templates: ${analytics.customCount}\n` +
                          `   • Categories: ${analytics.categories}\n` +
                          `   • Total Usage: ${analytics.totalUsage}\n\n` +
                          `🏷️ **Templates by Category:**\n${categoryList}\n\n` +
                          `⚙️ **Popular Frameworks:**\n${frameworkList}\n\n` +
                          `🎯 **Complexity Distribution:**\n${complexityList}\n\n` +
                          `🔥 **Most Used Templates:**\n${mostUsedList}\n\n` +
                          `💡 **Insights:**\n` +
                          `• ${analytics.customCount > 0 ? `You have ${analytics.customCount} custom templates` : 'Consider creating custom templates for your workflows'}\n` +
                          `• ${analytics.totalUsage > 10 ? 'Great template usage! Keep building with proven workflows' : 'Start using templates to boost productivity'}\n` +
                          `• Most popular category: ${Object.entries(analytics.categoryStats).sort((a, b) => b[1] - a[1])[0]?.[0] || 'N/A'}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get template analytics: ${error.message}`
                }]
            };
        }
    }

    async handleCreateWorkspace(args) {
        const { name, creatorId, options = {} } = args;

        if (!name || !creatorId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide workspace name and creatorId.'
                }]
            };
        }

        try {
            const workspaceId = await this.collaboration.createWorkspace(name, creatorId, options);

            return {
                content: [{
                    type: 'text',
                    text: `👥 **Collaborative Workspace Created**\n\n` +
                          `🆔 **Workspace ID:** ${workspaceId}\n` +
                          `📋 **Name:** ${name}\n` +
                          `👤 **Creator:** ${creatorId}\n` +
                          `⚙️ **Settings:**\n` +
                          `   • Public: ${options.isPublic ? 'Yes' : 'No'}\n` +
                          `   • Allow Guests: ${options.allowGuests ? 'Yes' : 'No'}\n` +
                          `   • Max Users: ${options.maxUsers || 20}\n` +
                          `   • Conflict Resolution: ${options.conflictResolution || 'last-write-wins'}\n\n` +
                          `✅ Workspace is ready for collaboration!\n\n` +
                          `💡 **Next Steps:**\n` +
                          `• Share workspace ID with team members\n` +
                          `• Use \`join_workspace\` to add collaborators\n` +
                          `• Start collaborative AI sessions and projects\n` +
                          `• Use \`get_workspace_status\` to monitor activity`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to create workspace: ${error.message}`
                }]
            };
        }
    }

    async handleJoinWorkspace(args) {
        const { workspaceId, userId, userInfo = {} } = args;

        if (!workspaceId || !userId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide workspaceId and userId.'
                }]
            };
        }

        try {
            const result = await this.collaboration.joinWorkspace(workspaceId, userId, userInfo);
            const workspace = result.workspace;

            return {
                content: [{
                    type: 'text',
                    text: `🎉 **Successfully Joined Workspace**\n\n` +
                          `📋 **Workspace:** ${workspace.name}\n` +
                          `🆔 **Session ID:** ${result.sessionId}\n` +
                          `👥 **Online Users:** ${workspace.users.filter(u => u.status === 'online').length}/${workspace.users.length}\n` +
                          `🔧 **Your Role:** ${workspace.userRole}\n\n` +
                          `📊 **Workspace Activity:**\n` +
                          `   • Projects: ${workspace.sharedState.projects.length}\n` +
                          `   • Active AI Sessions: ${workspace.sharedState.activeAISessions.length}\n` +
                          `   • Shared Documents: ${workspace.sharedState.sharedDocuments.length}\n` +
                          `   • Chat Messages: ${workspace.sharedState.chatHistory.length}\n\n` +
                          `💡 **Available Actions:**\n` +
                          `• Use \`execute_collaborative_operation\` to perform actions\n` +
                          `• Monitor workspace with \`get_workspace_status\`\n` +
                          `• Collaborate in real-time with team members\n\n` +
                          `🔗 **Session ID:** ${result.sessionId}\n` +
                          `Keep this session ID for all collaborative operations.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to join workspace: ${error.message}\n\n` +
                          `**Common issues:**\n` +
                          `• Workspace not found or invalid ID\n` +
                          `• Workspace is full (max users reached)\n` +
                          `• Access denied (private workspace)\n` +
                          `• User already in workspace`
                }]
            };
        }
    }

    async handleExecuteCollaborativeOperation(args) {
        const { sessionId, operation } = args;

        if (!sessionId || !operation) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide sessionId and operation.'
                }]
            };
        }

        if (!operation.type || !operation.data) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Invalid operation format. Operation must have type and data properties.'
                }]
            };
        }

        try {
            const result = await this.collaboration.executeOperation(sessionId, operation);

            const operationDescriptions = {
                'project:create': 'Project created',
                'project:update': 'Project updated',
                'document:create': 'Document created',
                'document:edit': 'Document edited',
                'ai_session:start': 'AI session started',
                'ai_session:update': 'AI session updated',
                'chat:message': 'Chat message sent',
                'annotation:add': 'Annotation added',
                'cursor:update': 'Cursor position updated'
            };

            const description = operationDescriptions[operation.type] || 'Operation executed';

            return {
                content: [{
                    type: 'text',
                    text: `⚡ **Collaborative Operation Successful**\n\n` +
                          `🔧 **Operation:** ${description}\n` +
                          `🆔 **Operation ID:** ${result.operationId}\n` +
                          `⏰ **Timestamp:** ${new Date(result.timestamp).toLocaleString()}\n\n` +
                          `📊 **Result:**\n` +
                          `${JSON.stringify(result.result, null, 2)}\n\n` +
                          `✅ Operation has been synchronized with all workspace members.\n\n` +
                          `💡 **Real-time Updates:**\n` +
                          `• All online users have been notified\n` +
                          `• Changes are immediately visible to collaborators\n` +
                          `• Operation logged for conflict resolution`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to execute collaborative operation: ${error.message}\n\n` +
                          `**Common issues:**\n` +
                          `• Invalid or expired session ID\n` +
                          `• Workspace not found\n` +
                          `• Insufficient permissions\n` +
                          `• Conflict with concurrent operations\n\n` +
                          `**Supported Operations:**\n` +
                          `• project:create, project:update\n` +
                          `• document:create, document:edit\n` +
                          `• ai_session:start, ai_session:update\n` +
                          `• chat:message, annotation:add\n` +
                          `• cursor:update`
                }]
            };
        }
    }

    async handleGetWorkspaceStatus(args) {
        const { workspaceId, userId } = args;

        if (!workspaceId || !userId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide workspaceId and userId.'
                }]
            };
        }

        try {
            const workspace = this.collaboration.getWorkspaceForUser(workspaceId, userId);

            if (!workspace) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ Workspace "${workspaceId}" not found or access denied.`
                    }]
                };
            }

            const onlineUsers = workspace.users.filter(u => u.status === 'online');
            const offlineUsers = workspace.users.filter(u => u.status === 'offline');

            const onlineUsersList = onlineUsers.length > 0 ?
                onlineUsers.map(u => `   👤 ${u.id} (${u.role || 'member'})`).join('\n') :
                '   • No users currently online';

            const recentActivity = workspace.sharedState.chatHistory.slice(-3);
            const recentMessages = recentActivity.length > 0 ?
                recentActivity.map(msg =>
                    `   💬 ${msg.userId}: ${msg.message.substring(0, 50)}${msg.message.length > 50 ? '...' : ''}`
                ).join('\n') :
                '   • No recent chat activity';

            return {
                content: [{
                    type: 'text',
                    text: `👥 **Workspace Status: ${workspace.name}**\n\n` +
                          `🆔 **ID:** ${workspace.id}\n` +
                          `👤 **Your Role:** ${workspace.userRole}\n` +
                          `🕒 **Last Activity:** ${new Date(workspace.lastActivity).toLocaleString()}\n\n` +
                          `👥 **Users (${workspace.users.length}):**\n` +
                          `🟢 **Online (${onlineUsers.length}):**\n${onlineUsersList}\n` +
                          `⚫ **Offline:** ${offlineUsers.length}\n\n` +
                          `📊 **Shared Resources:**\n` +
                          `   📁 Projects: ${workspace.sharedState.projects.length}\n` +
                          `   🤖 Active AI Sessions: ${workspace.sharedState.activeAISessions.length}\n` +
                          `   📄 Shared Documents: ${workspace.sharedState.sharedDocuments.length}\n` +
                          `   📝 Annotations: ${workspace.sharedState.annotations.length}\n\n` +
                          `💬 **Recent Chat Activity:**\n${recentMessages}\n\n` +
                          `⚙️ **Workspace Settings:**\n` +
                          `   • Public: ${workspace.settings.isPublic ? 'Yes' : 'No'}\n` +
                          `   • Max Users: ${workspace.settings.maxUsers}\n` +
                          `   • Conflict Resolution: ${workspace.settings.conflictResolution}\n` +
                          `   • Auto-save: ${workspace.settings.autoSave ? 'Enabled' : 'Disabled'}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get workspace status: ${error.message}`
                }]
            };
        }
    }

    async handleGetCollaborationAnalytics(args) {
        try {
            const analytics = this.collaboration.getCollaborationAnalytics();

            const workspacesList = analytics.workspaceDetails.length > 0 ?
                analytics.workspaceDetails.map((ws, index) =>
                    `   ${index + 1}. ${ws.name} (${ws.onlineUsers}/${ws.totalUsers} online)`
                ).join('\n') :
                '   • No active workspaces';

            return {
                content: [{
                    type: 'text',
                    text: `📊 **Real-time Collaboration Analytics**\n\n` +
                          `🏢 **Workspace Overview:**\n` +
                          `   • Total Workspaces: ${analytics.totalWorkspaces}\n` +
                          `   • Active Workspaces: ${analytics.activeWorkspaces}\n` +
                          `   • Active Sessions: ${analytics.activeSessions}\n` +
                          `   • Total Users: ${analytics.totalUsers}\n\n` +
                          `⚡ **Real-time Activity:**\n` +
                          `   • Conflicts Resolved: ${analytics.conflictsResolved}\n` +
                          `   • System Status: ${analytics.activeSessions > 0 ? 'Active' : 'Idle'}\n\n` +
                          `🔥 **Most Active Workspaces:**\n${workspacesList}\n\n` +
                          `💡 **Collaboration Insights:**\n` +
                          `${analytics.activeWorkspaces > 0 ? '• Real-time collaboration is active' : '• No active collaboration sessions'}\n` +
                          `${analytics.totalUsers > 10 ? '• High user engagement across workspaces' : '• Growing user base for collaboration'}\n` +
                          `${analytics.conflictsResolved > 0 ? `• ${analytics.conflictsResolved} conflicts successfully resolved` : '• No conflicts detected - smooth collaboration'}\n\n` +
                          `🎯 **System Performance:** ${analytics.activeSessions < 50 ? 'Optimal' : analytics.activeSessions < 100 ? 'Good' : 'High Load'}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get collaboration analytics: ${error.message}`
                }]
            };
        }
    }

    async handleCreateAISwarm(args) {
        const { taskDescription, requiredAgents = [], options = {} } = args;

        if (!taskDescription) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameter. Please provide a task description for the AI swarm.'
                }]
            };
        }

        try {
            const swarmId = await this.swarmIntelligence.createSwarm(taskDescription, requiredAgents, options);

            // Get swarm details
            const swarmStatus = this.swarmIntelligence.getSwarmStatus(swarmId);

            return {
                content: [{
                    type: 'text',
                    text: `🤖 **AI Swarm Created Successfully**\n\n` +
                          `🆔 **Swarm ID:** ${swarmId}\n` +
                          `📋 **Task:** ${taskDescription}\n` +
                          `👥 **Agents:** ${swarmStatus.agents.join(', ')}\n` +
                          `📊 **Status:** ${swarmStatus.status}\n` +
                          `🔄 **Current Phase:** ${swarmStatus.currentPhase}\n\n` +
                          `🤖 **Agent Specializations:**\n` +
                          `   • **Frontend:** UI/UX, React/Vue/Angular, responsive design\n` +
                          `   • **Backend:** APIs, databases, server architecture\n` +
                          `   • **DevOps:** CI/CD, infrastructure, monitoring\n` +
                          `   • **Testing:** Quality assurance, automation, performance\n` +
                          `   • **Security:** Vulnerability assessment, secure coding\n\n` +
                          `⚡ **Swarm Coordination:**\n` +
                          `   • Multi-agent collaboration with specialized expertise\n` +
                          `   • Inter-agent communication and knowledge sharing\n` +
                          `   • Coordinated planning and execution phases\n` +
                          `   • Quality review and consensus building\n\n` +
                          `💡 **Next Steps:**\n` +
                          `• Use \`get_swarm_status\` to monitor progress\n` +
                          `• Agents will collaborate through planning → execution → review phases\n` +
                          `• Final deliverables and recommendations will be generated\n` +
                          `• Use \`get_swarm_analytics\` for performance insights`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to create AI swarm: ${error.message}\n\n` +
                          `**Common issues:**\n` +
                          `• Invalid agent types specified\n` +
                          `• Maximum active swarms limit reached\n` +
                          `• System resource constraints\n\n` +
                          `**Available Agents:**\n` +
                          `• frontend - Frontend development specialist\n` +
                          `• backend - Backend development specialist\n` +
                          `• devops - Infrastructure and operations specialist\n` +
                          `• testing - Quality assurance specialist\n` +
                          `• security - Security engineering specialist`
                }]
            };
        }
    }

    async handleGetSwarmStatus(args) {
        const { swarmId } = args;

        if (!swarmId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameter. Please provide a swarm ID.'
                }]
            };
        }

        try {
            const swarmStatus = this.swarmIntelligence.getSwarmStatus(swarmId);

            if (!swarmStatus) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ AI Swarm "${swarmId}" not found.\n\nUse \`get_swarm_analytics\` to see available swarms.`
                    }]
                };
            }

            const progressBars = Object.entries(swarmStatus.progress).map(([phase, progress]) => {
                const filled = Math.floor(progress / 10);
                const empty = 10 - filled;
                const bar = '█'.repeat(filled) + '░'.repeat(empty);
                return `   ${phase.padEnd(12)}: ${bar} ${progress.toFixed(0)}%`;
            }).join('\n');

            const agentList = swarmStatus.agents.map(agentId => {
                const agentDef = this.swarmIntelligence.agentDefinitions.get(agentId);
                return `   🤖 **${agentDef?.name || agentId}** - ${agentDef?.role || 'Unknown role'}`;
            }).join('\n');

            const deliverablesList = swarmStatus.results?.deliverables?.length > 0 ?
                swarmStatus.results.deliverables.slice(0, 5).map((deliverable, index) =>
                    `   ${index + 1}. ${deliverable.type}: ${deliverable.description}`
                ).join('\n') : '   • No deliverables generated yet';

            const duration = Date.now() - swarmStatus.createdAt;
            const durationMinutes = Math.floor(duration / 60000);
            const durationSeconds = Math.floor((duration % 60000) / 1000);

            return {
                content: [{
                    type: 'text',
                    text: `🤖 **AI Swarm Status: ${swarmId}**\n\n` +
                          `📊 **Overview:**\n` +
                          `   • Status: ${swarmStatus.status}\n` +
                          `   • Current Phase: ${swarmStatus.currentPhase}\n` +
                          `   • Duration: ${durationMinutes}m ${durationSeconds}s\n` +
                          `   • Communications: ${swarmStatus.communicationCount}\n\n` +
                          `👥 **Active Agents (${swarmStatus.agents.length}):**\n${agentList}\n\n` +
                          `📈 **Progress by Phase:**\n${progressBars}\n\n` +
                          `📋 **Task Description:**\n${swarmStatus.taskDescription}\n\n` +
                          `📦 **Deliverables:**\n${deliverablesList}\n\n` +
                          `💡 **Swarm Intelligence Features:**\n` +
                          `• Specialized agent expertise and collaboration\n` +
                          `• Real-time inter-agent communication\n` +
                          `• Coordinated planning and execution\n` +
                          `• Quality review and consensus building\n` +
                          `• Comprehensive deliverable generation`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get swarm status: ${error.message}`
                }]
            };
        }
    }

    async handleGetSwarmAnalytics(args) {
        try {
            const analytics = this.swarmIntelligence.getSwarmAnalytics();

            const agentUsageList = Object.entries(analytics.agentUsage)
                .sort((a, b) => b[1] - a[1])
                .map(([agentId, count]) => {
                    const agentDef = this.swarmIntelligence.agentDefinitions.get(agentId);
                    return `   🤖 ${agentDef?.name || agentId}: ${count} swarms`;
                }).join('\n');

            const qualityRating = analytics.averageQuality >= 0.9 ? '🌟 Excellent' :
                                 analytics.averageQuality >= 0.8 ? '⭐ Good' :
                                 analytics.averageQuality >= 0.7 ? '👍 Satisfactory' :
                                 analytics.averageQuality >= 0.6 ? '⚠️ Needs Improvement' : '❌ Poor';

            const avgDurationMinutes = Math.floor(analytics.averageDuration / 60000);
            const successPercentage = (analytics.successRate * 100).toFixed(1);

            return {
                content: [{
                    type: 'text',
                    text: `🤖 **AI Swarm Intelligence Analytics**\n\n` +
                          `📊 **Overview:**\n` +
                          `   • Total Swarms: ${analytics.totalSwarms}\n` +
                          `   • Active Swarms: ${analytics.activeSwarms}\n` +
                          `   • Completed Swarms: ${analytics.completedSwarms}\n` +
                          `   • Success Rate: ${successPercentage}%\n\n` +
                          `⭐ **Quality Metrics:**\n` +
                          `   • Average Quality: ${qualityRating} (${analytics.averageQuality.toFixed(2)})\n` +
                          `   • Average Duration: ${avgDurationMinutes} minutes\n\n` +
                          `👥 **Agent Usage Statistics:**\n${agentUsageList || '   • No agent usage data yet'}\n\n` +
                          `🎯 **Swarm Intelligence Insights:**\n` +
                          `${analytics.totalSwarms > 0 ? '• AI swarm collaboration is active and productive' : '• No swarms created yet - start with complex development tasks'}\n` +
                          `${analytics.averageQuality > 0.8 ? '• High-quality deliverables from multi-agent collaboration' : '• Consider optimizing agent coordination for better results'}\n` +
                          `${analytics.successRate > 0.9 ? '• Excellent swarm completion rate' : '• Monitor swarm execution for potential improvements'}\n\n` +
                          `💡 **Recommendations:**\n` +
                          `• Use AI swarms for complex, multi-disciplinary tasks\n` +
                          `• Leverage specialized agent expertise for better outcomes\n` +
                          `• Monitor swarm analytics to optimize coordination strategies\n` +
                          `• Consider agent composition based on task requirements`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get swarm analytics: ${error.message}`
                }]
            };
        }
    }

    async handleTerminateSwarm(args) {
        const { swarmId } = args;

        if (!swarmId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameter. Please provide a swarm ID to terminate.'
                }]
            };
        }

        try {
            const success = await this.swarmIntelligence.terminateSwarm(swarmId);

            if (!success) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ AI Swarm "${swarmId}" not found or already terminated.\n\nUse \`get_swarm_analytics\` to see active swarms.`
                    }]
                };
            }

            return {
                content: [{
                    type: 'text',
                    text: `🛑 **AI Swarm Terminated**\n\n` +
                          `🆔 **Swarm ID:** ${swarmId}\n` +
                          `✅ **Status:** Successfully terminated\n\n` +
                          `📊 **Termination Details:**\n` +
                          `• All agent activities stopped\n` +
                          `• Swarm results saved to disk\n` +
                          `• Resources freed for new swarms\n` +
                          `• Performance metrics recorded\n\n` +
                          `💡 **Next Steps:**\n` +
                          `• Review saved swarm results if needed\n` +
                          `• Create new swarms for other tasks\n` +
                          `• Use \`get_swarm_analytics\` for insights`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to terminate swarm: ${error.message}`
                }]
            };
        }
    }

    async handleCreatePredictiveProject(args) {
        const { name, description, team = [], startDate, resources = {} } = args;

        if (!name || !description) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide project name and description.'
                }]
            };
        }

        try {
            const projectId = await this.predictiveTaskManagement.createProject({
                name,
                description,
                team,
                startDate,
                resources
            });

            // Get initial predictions
            const predictions = this.predictiveTaskManagement.getProjectPredictions(projectId);

            return {
                content: [{
                    type: 'text',
                    text: `🔮 **Predictive Project Created Successfully**\n\n` +
                          `🆔 **Project ID:** ${projectId}\n` +
                          `📋 **Name:** ${name}\n` +
                          `📝 **Description:** ${description}\n` +
                          `👥 **Team Size:** ${team.length}\n` +
                          `📅 **Start Date:** ${startDate ? new Date(startDate).toLocaleDateString() : 'Not set'}\n\n` +
                          `🤖 **AI Predictions Initialized:**\n` +
                          `   • Failure Risk Assessment: Active\n` +
                          `   • Timeline Estimation: Active\n` +
                          `   • Resource Optimization: Active\n` +
                          `   • Risk Monitoring: Every 60 seconds\n\n` +
                          `📊 **Initial Health Score:** ${(predictions?.predictions?.overallHealth * 100 || 80).toFixed(1)}%\n\n` +
                          `🔮 **Predictive Features:**\n` +
                          `• AI-powered failure prediction for tasks\n` +
                          `• Intelligent timeline estimation and optimization\n` +
                          `• Resource allocation recommendations\n` +
                          `• Real-time risk assessment and alerts\n` +
                          `• Historical data learning and model improvement\n\n` +
                          `💡 **Next Steps:**\n` +
                          `• Use \`add_predictive_task\` to add tasks with AI predictions\n` +
                          `• Monitor predictions with \`get_project_predictions\`\n` +
                          `• Optimize timeline with \`optimize_project_timeline\`\n` +
                          `• View analytics with \`get_predictive_analytics\``
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to create predictive project: ${error.message}`
                }]
            };
        }
    }

    async handleAddPredictiveTask(args) {
        const { projectId, title, description, type = 'development', complexity = 'medium',
                priority = 'medium', assignee, estimatedHours, dependencies = [] } = args;

        if (!projectId || !title || !description) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide projectId, title, and description.'
                }]
            };
        }

        try {
            const taskId = await this.predictiveTaskManagement.addTaskToProject(projectId, {
                title,
                description,
                type,
                complexity,
                priority,
                assignee,
                estimatedHours,
                dependencies
            });

            // Get task predictions
            const taskPredictions = this.predictiveTaskManagement.getTaskPredictions(projectId, taskId);
            const projectPredictions = this.predictiveTaskManagement.getProjectPredictions(projectId);

            const failureRisk = (taskPredictions?.predictions?.failureRisk * 100 || 30).toFixed(1);
            const completionProb = (taskPredictions?.predictions?.completionProbability * 100 || 85).toFixed(1);
            const timelineAccuracy = (taskPredictions?.predictions?.timelineAccuracy * 100 || 80).toFixed(1);
            const resourceMultiplier = (taskPredictions?.predictions?.resourceRequirement || 1.0).toFixed(2);

            const riskLevel = failureRisk > 70 ? '🔴 High' : failureRisk > 40 ? '🟡 Medium' : '🟢 Low';

            return {
                content: [{
                    type: 'text',
                    text: `🔮 **Predictive Task Added Successfully**\n\n` +
                          `🆔 **Task ID:** ${taskId}\n` +
                          `📋 **Title:** ${title}\n` +
                          `🏷️ **Type:** ${type} | **Complexity:** ${complexity} | **Priority:** ${priority}\n` +
                          `👤 **Assignee:** ${assignee || 'Unassigned'}\n` +
                          `⏱️ **Estimated Hours:** ${estimatedHours || 'Not specified'}\n` +
                          `🔗 **Dependencies:** ${dependencies.length}\n\n` +
                          `🤖 **AI Predictions:**\n` +
                          `   ${riskLevel} **Failure Risk:** ${failureRisk}%\n` +
                          `   📈 **Completion Probability:** ${completionProb}%\n` +
                          `   🎯 **Timeline Accuracy:** ${timelineAccuracy}%\n` +
                          `   📊 **Resource Multiplier:** ${resourceMultiplier}x\n` +
                          `   🔍 **Confidence:** ${(taskPredictions?.confidence * 100 || 80).toFixed(1)}%\n\n` +
                          `📊 **Project Impact:**\n` +
                          `   • Project Health: ${(projectPredictions?.predictions?.overallHealth * 100 || 80).toFixed(1)}%\n` +
                          `   • Project Risk: ${(projectPredictions?.predictions?.failureRisk * 100 || 30).toFixed(1)}%\n\n` +
                          `💡 **Recommendations:**\n` +
                          `${failureRisk > 70 ? '• ⚠️ High risk task - consider additional resources or scope reduction\n' : ''}` +
                          `${resourceMultiplier > 1.5 ? '• 📈 Task may require more resources than estimated\n' : ''}` +
                          `${dependencies.length > 3 ? '• 🔗 Many dependencies - monitor for bottlenecks\n' : ''}` +
                          `• Use \`get_task_predictions\` for detailed analysis\n` +
                          `• Monitor progress and update predictions as work progresses`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to add predictive task: ${error.message}\n\n` +
                          `**Common issues:**\n` +
                          `• Project not found\n` +
                          `• Invalid task parameters\n` +
                          `• Dependency task IDs not found`
                }]
            };
        }
    }

    async handleGetProjectPredictions(args) {
        const { projectId } = args;

        if (!projectId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameter. Please provide a project ID.'
                }]
            };
        }

        try {
            const predictions = this.predictiveTaskManagement.getProjectPredictions(projectId);

            if (!predictions) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ Project "${projectId}" not found.\n\nUse \`get_predictive_analytics\` to see available projects.`
                    }]
                };
            }

            const overallHealth = (predictions.predictions.overallHealth * 100).toFixed(1);
            const failureRisk = (predictions.predictions.failureRisk * 100).toFixed(1);
            const timelineAccuracy = (predictions.predictions.timelineAccuracy * 100).toFixed(1);
            const resourceOptimization = (predictions.predictions.resourceOptimization * 100).toFixed(1);

            const healthIcon = overallHealth > 80 ? '🟢' : overallHealth > 60 ? '🟡' : '🔴';
            const riskIcon = failureRisk > 70 ? '🔴' : failureRisk > 40 ? '🟡' : '🟢';

            const risksList = predictions.risks?.length > 0 ?
                predictions.risks.map((risk, index) =>
                    `   ${index + 1}. ${risk.severity.toUpperCase()}: ${risk.description}`
                ).join('\n') : '   • No significant risks identified';

            const timelineInfo = predictions.timeline ?
                `   • Estimated Completion: ${new Date(predictions.timeline.estimatedCompletionDate).toLocaleDateString()}\n` +
                `   • Total Estimated Hours: ${predictions.timeline.totalEstimatedHours}\n` +
                `   • Weeks to Complete: ${predictions.timeline.weeksToComplete.toFixed(1)}\n` +
                `   • Confidence: ${(predictions.timeline.confidence * 100).toFixed(1)}%` :
                '   • Timeline estimation in progress...';

            return {
                content: [{
                    type: 'text',
                    text: `🔮 **Project Predictions: ${projectId}**\n\n` +
                          `📊 **Overall Health:** ${healthIcon} ${overallHealth}%\n\n` +
                          `🤖 **AI Predictions:**\n` +
                          `   ${riskIcon} **Failure Risk:** ${failureRisk}%\n` +
                          `   🎯 **Timeline Accuracy:** ${timelineAccuracy}%\n` +
                          `   ⚙️ **Resource Optimization:** ${resourceOptimization}%\n\n` +
                          `📅 **Timeline Estimates:**\n${timelineInfo}\n\n` +
                          `⚠️ **Risk Assessment:**\n${risksList}\n\n` +
                          `🔄 **Last Updated:** ${new Date(predictions.lastUpdated).toLocaleString()}\n\n` +
                          `💡 **Insights:**\n` +
                          `${overallHealth > 80 ? '• ✅ Project is in excellent health' : overallHealth > 60 ? '• ⚠️ Project needs attention in some areas' : '• 🚨 Project requires immediate intervention'}\n` +
                          `${failureRisk > 70 ? '• 🔴 High failure risk - consider risk mitigation strategies' : failureRisk > 40 ? '• 🟡 Moderate risk - monitor closely' : '• 🟢 Low risk - project on track'}\n` +
                          `${timelineAccuracy > 80 ? '• 📈 High confidence in timeline estimates' : '• 📊 Timeline estimates may need adjustment'}\n\n` +
                          `🛠️ **Recommendations:**\n` +
                          `• Use \`optimize_project_timeline\` for optimization suggestions\n` +
                          `• Monitor individual task predictions with \`get_task_predictions\`\n` +
                          `• Review risk mitigation strategies for high-risk items`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get project predictions: ${error.message}`
                }]
            };
        }
    }

    async handleGetTaskPredictions(args) {
        const { projectId, taskId } = args;

        if (!projectId || !taskId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide both projectId and taskId.'
                }]
            };
        }

        try {
            const predictions = this.predictiveTaskManagement.getTaskPredictions(projectId, taskId);

            if (!predictions) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ Task "${taskId}" not found in project "${projectId}".`
                    }]
                };
            }

            const failureRisk = (predictions.predictions.failureRisk * 100).toFixed(1);
            const completionProb = (predictions.predictions.completionProbability * 100).toFixed(1);
            const timelineAccuracy = (predictions.predictions.timelineAccuracy * 100).toFixed(1);
            const resourceRequirement = predictions.predictions.resourceRequirement.toFixed(2);
            const confidence = (predictions.confidence * 100).toFixed(1);

            const riskLevel = failureRisk > 70 ? '🔴 Critical' :
                             failureRisk > 50 ? '🟠 High' :
                             failureRisk > 30 ? '🟡 Medium' : '🟢 Low';

            return {
                content: [{
                    type: 'text',
                    text: `🔮 **Task Predictions**\n\n` +
                          `🆔 **Task ID:** ${taskId}\n` +
                          `📋 **Project ID:** ${projectId}\n\n` +
                          `🤖 **AI Predictions:**\n` +
                          `   🎯 **Failure Risk:** ${riskLevel} (${failureRisk}%)\n` +
                          `   ✅ **Completion Probability:** ${completionProb}%\n` +
                          `   📅 **Timeline Accuracy:** ${timelineAccuracy}%\n` +
                          `   📊 **Resource Requirement:** ${resourceRequirement}x estimated\n` +
                          `   🔍 **Prediction Confidence:** ${confidence}%\n\n` +
                          `📈 **Risk Analysis:**\n` +
                          `${failureRisk > 70 ? '   🚨 **Critical Risk Factors:**\n     • Task complexity may be underestimated\n     • Consider breaking down into smaller tasks\n     • Assign experienced team members\n     • Increase testing and review cycles\n' : ''}` +
                          `${failureRisk > 50 && failureRisk <= 70 ? '   ⚠️ **High Risk Factors:**\n     • Monitor progress closely\n     • Consider additional resources\n     • Review dependencies and blockers\n' : ''}` +
                          `${failureRisk > 30 && failureRisk <= 50 ? '   📊 **Medium Risk Factors:**\n     • Standard monitoring recommended\n     • Ensure clear requirements\n     • Regular check-ins with assignee\n' : ''}` +
                          `${failureRisk <= 30 ? '   ✅ **Low Risk:**\n     • Task is well-defined and manageable\n     • Standard project management practices apply\n' : ''}` +
                          `\n💡 **Optimization Suggestions:**\n` +
                          `${resourceRequirement > 1.5 ? '   • 📈 Task may require ' + Math.round((resourceRequirement - 1) * 100) + '% more resources than estimated\n' : ''}` +
                          `${timelineAccuracy < 70 ? '   • 📅 Timeline estimates have low confidence - consider re-estimation\n' : ''}` +
                          `${completionProb < 80 ? '   • 🎯 Consider risk mitigation strategies to improve completion probability\n' : ''}` +
                          `   • Monitor actual progress vs predictions to improve future estimates\n` +
                          `   • Update predictions as task progresses and new information becomes available\n\n` +
                          `🔄 **Last Updated:** ${new Date(predictions.lastUpdated).toLocaleString()}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get task predictions: ${error.message}`
                }]
            };
        }
    }

    async handleGetPredictiveAnalytics(args) {
        try {
            const analytics = this.predictiveTaskManagement.getPredictiveAnalytics();

            const healthRating = analytics.avgProjectHealth >= 0.8 ? '🟢 Excellent' :
                                analytics.avgProjectHealth >= 0.6 ? '🟡 Good' :
                                analytics.avgProjectHealth >= 0.4 ? '🟠 Fair' : '🔴 Poor';

            const modelAccuracyRating = analytics.modelAccuracy >= 0.85 ? '🌟 Excellent' :
                                      analytics.modelAccuracy >= 0.75 ? '⭐ Good' :
                                      analytics.modelAccuracy >= 0.65 ? '👍 Fair' : '⚠️ Needs Improvement';

            const modelsList = analytics.predictionModels.map((model, index) =>
                `   ${index + 1}. **${model.name}**\n` +
                `      • Accuracy: ${(model.accuracy * 100).toFixed(1)}%\n` +
                `      • Predictions Made: ${model.predictions}\n` +
                `      • Last Trained: ${new Date(model.lastTrained).toLocaleDateString()}`
            ).join('\n\n');

            return {
                content: [{
                    type: 'text',
                    text: `🔮 **Predictive Task Management Analytics**\n\n` +
                          `📊 **Overview:**\n` +
                          `   • Active Projects: ${analytics.totalProjects}\n` +
                          `   • Total Tasks: ${analytics.totalTasks}\n` +
                          `   • High-Risk Projects: ${analytics.highRiskProjects}\n` +
                          `   • High-Risk Tasks: ${analytics.highRiskTasks}\n\n` +
                          `🏥 **Project Health:**\n` +
                          `   • Average Health: ${healthRating} (${(analytics.avgProjectHealth * 100).toFixed(1)}%)\n` +
                          `   • Risk Assessment: ${analytics.riskAssessmentActive ? '✅ Active' : '❌ Inactive'}\n\n` +
                          `🤖 **AI Model Performance:**\n` +
                          `   • Overall Accuracy: ${modelAccuracyRating} (${(analytics.modelAccuracy * 100).toFixed(1)}%)\n` +
                          `   • Historical Data Points: ${analytics.historicalDataPoints}\n` +
                          `   • Active Models: ${analytics.predictionModels.length}\n\n` +
                          `📈 **Prediction Models:**\n${modelsList}\n\n` +
                          `💡 **Insights:**\n` +
                          `${analytics.totalProjects > 0 ? '• Predictive analytics actively monitoring projects' : '• No active projects - create projects to start predictions'}\n` +
                          `${analytics.highRiskProjects > 0 ? `• ${analytics.highRiskProjects} projects need immediate attention` : '• All projects are in good health'}\n` +
                          `${analytics.modelAccuracy > 0.8 ? '• AI models are performing well and providing reliable predictions' : '• Consider training models with more historical data'}\n` +
                          `${analytics.historicalDataPoints > 100 ? '• Rich historical data enables accurate predictions' : '• More historical data will improve prediction accuracy'}\n\n` +
                          `🎯 **Recommendations:**\n` +
                          `• Use predictive insights to optimize resource allocation\n` +
                          `• Monitor high-risk projects and tasks closely\n` +
                          `• Leverage timeline optimization for better planning\n` +
                          `• Continue building historical data for model improvement`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get predictive analytics: ${error.message}`
                }]
            };
        }
    }

    async handleOptimizeProjectTimeline(args) {
        const { projectId } = args;

        if (!projectId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameter. Please provide a project ID.'
                }]
            };
        }

        try {
            const optimization = await this.predictiveTaskManagement.optimizeProjectTimeline(projectId);

            const timeSavingPercentage = (optimization.potentialTimeSaving * 100).toFixed(1);

            const criticalPathList = optimization.criticalPath.length > 0 ?
                optimization.criticalPath.map((taskId, index) =>
                    `   ${index + 1}. Task ${taskId}`
                ).join('\n') : '   • No critical path identified';

            const optimizationsList = optimization.optimizations.length > 0 ?
                optimization.optimizations.map((opt, index) => {
                    let details = '';
                    switch (opt.type) {
                        case 'parallel_execution':
                            details = `\n      Tasks: ${opt.opportunities?.[0]?.tasks?.join(', ') || 'Multiple tasks'}`;
                            break;
                        case 'workload_rebalancing':
                            details = `\n      Overloaded: ${opt.overloaded?.join(', ') || 'N/A'}\n      Underutilized: ${opt.underutilized?.join(', ') || 'N/A'}`;
                            break;
                        case 'skill_optimization':
                            details = `\n      Affected Tasks: ${opt.tasks?.length || 0}`;
                            break;
                        case 'scope_adjustment':
                            details = `\n      High-Risk Tasks: ${opt.tasks?.length || 0}`;
                            break;
                    }

                    return `   ${index + 1}. **${opt.type.replace(/_/g, ' ').toUpperCase()}** (${opt.impact} impact)\n` +
                           `      ${opt.description}${details}`;
                }).join('\n\n') : '   • No optimization opportunities identified';

            return {
                content: [{
                    type: 'text',
                    text: `🔮 **Project Timeline Optimization**\n\n` +
                          `🆔 **Project ID:** ${projectId}\n` +
                          `⏱️ **Potential Time Saving:** ${timeSavingPercentage}%\n` +
                          `🎯 **Optimization Confidence:** ${(optimization.confidence * 100).toFixed(1)}%\n\n` +
                          `🛤️ **Critical Path:**\n${criticalPathList}\n\n` +
                          `⚡ **Optimization Opportunities:**\n${optimizationsList}\n\n` +
                          `💡 **Implementation Recommendations:**\n` +
                          `${optimization.optimizations.some(o => o.type === 'parallel_execution') ? '• 🔄 Execute independent tasks in parallel to reduce overall timeline\n' : ''}` +
                          `${optimization.optimizations.some(o => o.type === 'workload_rebalancing') ? '• ⚖️ Redistribute workload to balance team capacity\n' : ''}` +
                          `${optimization.optimizations.some(o => o.type === 'skill_optimization') ? '• 🎯 Reassign tasks to better match team member expertise\n' : ''}` +
                          `${optimization.optimizations.some(o => o.type === 'scope_adjustment') ? '• 📋 Consider scope adjustments for high-risk tasks\n' : ''}` +
                          `• 📊 Monitor progress against optimized timeline\n` +
                          `• 🔄 Re-run optimization as project evolves\n` +
                          `• 📈 Use predictive insights to validate optimization decisions\n\n` +
                          `⚠️ **Important Notes:**\n` +
                          `• Timeline optimization is based on current project state\n` +
                          `• Actual results may vary based on execution quality\n` +
                          `• Consider team capacity and external dependencies\n` +
                          `• Regular monitoring and adjustment recommended`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to optimize project timeline: ${error.message}\n\n` +
                          `**Common issues:**\n` +
                          `• Project not found\n` +
                          `• Insufficient project data for optimization\n` +
                          `• No tasks available for optimization`
                }]
            };
        }
    }

    async handleStartCodeSession(args) {
        const { userId, projectId, language, fileName, initialCode = '' } = args;

        if (!userId || !language || !fileName) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide userId, language, and fileName.'
                }]
            };
        }

        try {
            const sessionId = await this.realTimeCodeGeneration.startCodeSession({
                userId,
                projectId,
                language,
                fileName,
                initialCode
            });

            const sessionStatus = this.realTimeCodeGeneration.getSessionStatus(sessionId);

            return {
                content: [{
                    type: 'text',
                    text: `💻 **Real-time Code Session Started**\n\n` +
                          `🆔 **Session ID:** ${sessionId}\n` +
                          `👤 **User ID:** ${userId}\n` +
                          `📁 **Project ID:** ${projectId || 'None'}\n` +
                          `🔤 **Language:** ${language}\n` +
                          `📄 **File:** ${fileName}\n` +
                          `📝 **Initial Code Length:** ${initialCode.length} characters\n\n` +
                          `🤖 **Real-time Features Active:**\n` +
                          `   • Live syntax analysis and error detection\n` +
                          `   • Intelligent code completion suggestions\n` +
                          `   • Performance optimization recommendations\n` +
                          `   • Security vulnerability scanning\n` +
                          `   • Bug detection and prevention\n` +
                          `   • Code quality metrics tracking\n\n` +
                          `⚡ **Analysis Interval:** 1 second\n` +
                          `📊 **Max Suggestions:** 5 per analysis\n\n` +
                          `💡 **Next Steps:**\n` +
                          `• Use \`update_code\` to send code changes and get real-time feedback\n` +
                          `• Use \`generate_code\` to generate code from natural language\n` +
                          `• Use \`get_code_suggestions\` to get current analysis results\n` +
                          `• Use \`end_code_session\` when finished coding\n\n` +
                          `🔄 **Session Status:** Active and monitoring for code changes`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to start code session: ${error.message}`
                }]
            };
        }
    }

    async handleUpdateCode(args) {
        const { sessionId, code, cursor = { line: 0, column: 0 }, changeType = 'edit' } = args;

        if (!sessionId || code === undefined) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide sessionId and code.'
                }]
            };
        }

        try {
            const result = await this.realTimeCodeGeneration.updateCode(sessionId, {
                code,
                cursor,
                changeType
            });

            const suggestions = result.suggestions.slice(0, 3); // Show top 3 suggestions
            const issues = result.issues.filter(issue => issue.severity === 'critical' || issue.severity === 'high').slice(0, 3);

            const suggestionsList = suggestions.length > 0 ?
                suggestions.map((suggestion, index) =>
                    `   ${index + 1}. **${suggestion.type.toUpperCase()}** (${suggestion.priority})\n` +
                    `      ${suggestion.message || suggestion.description}\n` +
                    `      ${suggestion.suggestion || suggestion.insertText || ''}`
                ).join('\n\n') : '   • No suggestions at this time';

            const issuesList = issues.length > 0 ?
                issues.map((issue, index) =>
                    `   ${index + 1}. **${issue.severity.toUpperCase()}**: ${issue.message}\n` +
                    `      ${issue.suggestion ? 'Fix: ' + issue.suggestion : ''}\n` +
                    `      ${issue.line ? 'Line: ' + issue.line : ''}`
                ).join('\n\n') : '   • No critical issues detected';

            const metricsText = `   • Lines of Code: ${result.metrics.linesOfCode}\n` +
                               `   • Complexity Score: ${result.metrics.complexity}\n` +
                               `   • Maintainability Index: ${result.metrics.maintainabilityIndex}\n` +
                               `   • Security Score: ${result.metrics.securityScore}/100\n` +
                               `   • Total Issues: ${result.metrics.issueCount}\n` +
                               `   • Suggestions Available: ${result.metrics.suggestionCount}`;

            return {
                content: [{
                    type: 'text',
                    text: `💻 **Code Updated - Real-time Analysis**\n\n` +
                          `🆔 **Session ID:** ${sessionId}\n` +
                          `📝 **Code Length:** ${code.length} characters\n` +
                          `📍 **Cursor Position:** Line ${cursor.line + 1}, Column ${cursor.column + 1}\n` +
                          `🔄 **Change Type:** ${changeType}\n\n` +
                          `🤖 **AI Suggestions:**\n${suggestionsList}\n\n` +
                          `⚠️ **Issues Detected:**\n${issuesList}\n\n` +
                          `📊 **Code Metrics:**\n${metricsText}\n\n` +
                          `💡 **Tips:**\n` +
                          `• Continue typing to get real-time feedback\n` +
                          `• Use \`get_code_suggestions\` for complete analysis\n` +
                          `• Address high-severity issues for better code quality`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to update code: ${error.message}\n\n` +
                          `**Common issues:**\n` +
                          `• Session not found or expired\n` +
                          `• Invalid code format\n` +
                          `• Analysis engine error`
                }]
            };
        }
    }

    async handleGenerateCode(args) {
        const { prompt, language, context = {} } = args;

        if (!prompt || !language) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide prompt and language.'
                }]
            };
        }

        try {
            const result = await this.realTimeCodeGeneration.generateCode(prompt, language, context);

            const suggestionsList = result.suggestions.length > 0 ?
                result.suggestions.map((suggestion, index) =>
                    `   ${index + 1}. **${suggestion.type.toUpperCase()}**: ${suggestion.message}\n` +
                    `      ${suggestion.suggestion}`
                ).join('\n\n') : '   • No additional suggestions';

            return {
                content: [{
                    type: 'text',
                    text: `🤖 **Code Generated Successfully**\n\n` +
                          `🆔 **Generation ID:** ${result.id}\n` +
                          `💬 **Prompt:** "${prompt}"\n` +
                          `🔤 **Language:** ${language}\n` +
                          `🎯 **Intent:** ${result.intent.type} (${result.intent.complexity} complexity)\n` +
                          `📝 **Entities:** ${result.intent.entities.join(', ') || 'None detected'}\n\n` +
                          `📄 **Generated Code:**\n\`\`\`${language}\n${result.code}\n\`\`\`\n\n` +
                          `💡 **AI Suggestions:**\n${suggestionsList}\n\n` +
                          `🔧 **Next Steps:**\n` +
                          `• Review and modify the generated code as needed\n` +
                          `• Start a code session to get real-time analysis\n` +
                          `• Add error handling and documentation\n` +
                          `• Write tests for the generated code\n\n` +
                          `⏰ **Generated:** ${new Date(result.timestamp).toLocaleString()}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to generate code: ${error.message}\n\n` +
                          `**Tips for better code generation:**\n` +
                          `• Be specific about what you want the code to do\n` +
                          `• Mention the programming paradigm (OOP, functional, etc.)\n` +
                          `• Include any specific requirements or constraints\n` +
                          `• Specify input/output expectations`
                }]
            };
        }
    }

    async handleGetCodeSuggestions(args) {
        const { sessionId } = args;

        if (!sessionId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameter. Please provide a session ID.'
                }]
            };
        }

        try {
            const sessionStatus = this.realTimeCodeGeneration.getSessionStatus(sessionId);

            if (!sessionStatus) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ Code session "${sessionId}" not found.\n\nUse \`get_codegen_analytics\` to see active sessions.`
                    }]
                };
            }

            // Get the session to access suggestions and issues
            const session = this.realTimeCodeGeneration.activeSessions.get(sessionId);

            const suggestionsList = session.suggestions.length > 0 ?
                session.suggestions.map((suggestion, index) =>
                    `   ${index + 1}. **${suggestion.type.toUpperCase()}** (${suggestion.priority || 'medium'} priority)\n` +
                    `      📝 ${suggestion.message || suggestion.description}\n` +
                    `      ${suggestion.suggestion || suggestion.insertText ? '💡 ' + (suggestion.suggestion || suggestion.insertText) : ''}\n` +
                    `      ${suggestion.line ? '📍 Line: ' + suggestion.line : ''}`
                ).join('\n\n') : '   • No suggestions available';

            const issuesList = session.issues.length > 0 ?
                session.issues.map((issue, index) => {
                    const severityIcon = issue.severity === 'critical' ? '🔴' :
                                       issue.severity === 'high' ? '🟠' :
                                       issue.severity === 'medium' ? '🟡' : '🔵';
                    return `   ${index + 1}. ${severityIcon} **${issue.severity.toUpperCase()}**: ${issue.message}\n` +
                           `      ${issue.suggestion ? '🔧 Fix: ' + issue.suggestion : ''}\n` +
                           `      ${issue.line ? '📍 Line: ' + issue.line : ''}\n` +
                           `      ${issue.cwe ? '🛡️ CWE: ' + issue.cwe : ''}`;
                }).join('\n\n') : '   • No issues detected';

            const metricsText = `   • 📏 Lines of Code: ${sessionStatus.metrics.linesOfCode}\n` +
                               `   • 🔄 Complexity Score: ${sessionStatus.metrics.complexity}\n` +
                               `   • 🔧 Maintainability Index: ${sessionStatus.metrics.maintainabilityIndex}\n` +
                               `   • 🛡️ Security Score: ${sessionStatus.metrics.securityScore}/100\n` +
                               `   • ⚠️ Total Issues: ${sessionStatus.issueCount}\n` +
                               `   • 💡 Available Suggestions: ${sessionStatus.suggestionCount}`;

            const healthScore = sessionStatus.metrics.securityScore > 80 &&
                               sessionStatus.metrics.maintainabilityIndex > 70 &&
                               sessionStatus.issueCount < 3 ? '🟢 Excellent' :
                               sessionStatus.metrics.securityScore > 60 &&
                               sessionStatus.issueCount < 5 ? '🟡 Good' : '🔴 Needs Attention';

            return {
                content: [{
                    type: 'text',
                    text: `💻 **Real-time Code Analysis**\n\n` +
                          `🆔 **Session ID:** ${sessionId}\n` +
                          `📄 **File:** ${sessionStatus.fileName}\n` +
                          `🔤 **Language:** ${sessionStatus.language}\n` +
                          `🏥 **Code Health:** ${healthScore}\n` +
                          `⏰ **Last Activity:** ${new Date(sessionStatus.lastActivity).toLocaleString()}\n\n` +
                          `📊 **Code Metrics:**\n${metricsText}\n\n` +
                          `🤖 **AI Suggestions:**\n${suggestionsList}\n\n` +
                          `⚠️ **Issues Detected:**\n${issuesList}\n\n` +
                          `💡 **Recommendations:**\n` +
                          `${sessionStatus.issueCount > 5 ? '• 🚨 High issue count - consider refactoring\n' : ''}` +
                          `${sessionStatus.metrics.securityScore < 70 ? '• 🛡️ Security concerns detected - review and fix\n' : ''}` +
                          `${sessionStatus.metrics.maintainabilityIndex < 50 ? '• 🔧 Low maintainability - consider simplifying code\n' : ''}` +
                          `${sessionStatus.metrics.complexity > 10 ? '• 🔄 High complexity - break down into smaller functions\n' : ''}` +
                          `• Continue coding to get real-time feedback\n` +
                          `• Address critical and high-severity issues first\n` +
                          `• Use suggestions to improve code quality`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get code suggestions: ${error.message}`
                }]
            };
        }
    }

    async handleGetCodegenAnalytics(args) {
        try {
            const analytics = this.realTimeCodeGeneration.getCodeGenAnalytics();

            const languageStatsText = Object.entries(analytics.languageStats).length > 0 ?
                Object.entries(analytics.languageStats).map(([lang, count]) =>
                    `   • ${lang}: ${count} sessions`
                ).join('\n') : '   • No language statistics available';

            const utilizationRate = analytics.totalSessions > 0 ?
                ((analytics.activeSessions / Math.max(analytics.totalSessions, 1)) * 100).toFixed(1) : '0.0';

            return {
                content: [{
                    type: 'text',
                    text: `💻 **Real-time Code Generation Analytics**\n\n` +
                          `📊 **Session Overview:**\n` +
                          `   • Active Sessions: ${analytics.activeSessions}\n` +
                          `   • Total Sessions: ${analytics.totalSessions}\n` +
                          `   • Total Code Generations: ${analytics.totalGenerations}\n` +
                          `   • Session Utilization: ${utilizationRate}%\n\n` +
                          `🔤 **Language Support:**\n` +
                          `   • Supported Languages: ${analytics.supportedLanguages}\n` +
                          `   • Language Usage:\n${languageStatsText}\n\n` +
                          `📈 **Average Metrics:**\n` +
                          `   • Lines of Code per Session: ${analytics.avgMetrics.linesOfCode}\n` +
                          `   • Issues Found per Session: ${analytics.avgMetrics.issuesFound}\n` +
                          `   • Suggestions per Session: ${analytics.avgMetrics.suggestionsProvided}\n\n` +
                          `🔧 **System Components:**\n` +
                          `   • Code Analyzers: ${analytics.analyzers}\n` +
                          `   • Suggestion Engines: ${analytics.suggestionEngines}\n` +
                          `   • Security Scanners: ${analytics.securityScanners}\n\n` +
                          `💡 **Insights:**\n` +
                          `${analytics.activeSessions > 0 ? '• Real-time code analysis is actively helping developers' : '• No active sessions - ready for new coding sessions'}\n` +
                          `${analytics.totalSessions > 10 ? '• Good adoption of real-time coding assistance' : '• Consider promoting real-time coding features'}\n` +
                          `${analytics.avgMetrics.issuesFound > 3 ? '• High issue detection rate - system is effectively catching problems' : '• Low issue rate indicates good code quality'}\n` +
                          `${analytics.avgMetrics.suggestionsProvided > 5 ? '• High suggestion rate - providing valuable coding assistance' : '• Moderate suggestion rate - system is selective with recommendations'}\n\n` +
                          `🎯 **Recommendations:**\n` +
                          `• Use real-time sessions for complex coding tasks\n` +
                          `• Leverage AI suggestions to improve code quality\n` +
                          `• Address security issues immediately\n` +
                          `• Monitor code metrics for maintainability`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get code generation analytics: ${error.message}`
                }]
            };
        }
    }

    async handleEndCodeSession(args) {
        const { sessionId } = args;

        if (!sessionId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameter. Please provide a session ID.'
                }]
            };
        }

        try {
            const result = await this.realTimeCodeGeneration.endCodeSession(sessionId);

            const durationMinutes = Math.round(result.summary.duration / (1000 * 60));
            const durationText = durationMinutes > 60 ?
                `${Math.floor(durationMinutes / 60)}h ${durationMinutes % 60}m` :
                `${durationMinutes}m`;

            const productivityScore = result.summary.linesOfCode > 50 && result.summary.issuesFound < 5 ? '🟢 High' :
                                    result.summary.linesOfCode > 20 ? '🟡 Medium' : '🔴 Low';

            return {
                content: [{
                    type: 'text',
                    text: `💻 **Code Session Ended**\n\n` +
                          `🆔 **Session ID:** ${sessionId}\n` +
                          `⏱️ **Duration:** ${durationText}\n` +
                          `📊 **Productivity Score:** ${productivityScore}\n\n` +
                          `📈 **Session Summary:**\n` +
                          `   • Lines of Code Written: ${result.summary.linesOfCode}\n` +
                          `   • Issues Detected: ${result.summary.issuesFound}\n` +
                          `   • Suggestions Provided: ${result.summary.suggestionsProvided}\n` +
                          `   • Average Issues per 100 LOC: ${result.summary.linesOfCode > 0 ? Math.round((result.summary.issuesFound / result.summary.linesOfCode) * 100) : 0}\n\n` +
                          `🎯 **Session Insights:**\n` +
                          `${result.summary.linesOfCode > 100 ? '• 🚀 Productive session with significant code output\n' : ''}` +
                          `${result.summary.issuesFound === 0 ? '• ✅ Clean code - no issues detected\n' : ''}` +
                          `${result.summary.issuesFound > 10 ? '• ⚠️ High issue count - consider code review\n' : ''}` +
                          `${result.summary.suggestionsProvided > 10 ? '• 💡 AI provided extensive coding assistance\n' : ''}` +
                          `${durationMinutes > 120 ? '• ⏰ Long coding session - consider taking breaks\n' : ''}` +
                          `• Session data saved for analytics and learning\n\n` +
                          `💡 **Next Steps:**\n` +
                          `• Review any remaining issues in your code\n` +
                          `• Consider running tests on the written code\n` +
                          `• Start a new session for continued development\n` +
                          `• Use \`get_codegen_analytics\` to see overall progress`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to end code session: ${error.message}\n\n` +
                          `**Common issues:**\n` +
                          `• Session not found or already ended\n` +
                          `• Session cleanup error`
                }]
            };
        }
    }

    async handleRecordUserFeedback(args) {
        const { userId, type, category, content, rating, context = {} } = args;

        if (!userId || !type || !category || !content || rating === undefined) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide userId, type, category, content, and rating.'
                }]
            };
        }

        try {
            // Sanitize and validate inputs
            const sanitizedUserId = this.inputSanitizer.sanitizeUserId(userId);
            const sanitizedFeedbackData = this.inputSanitizer.validateFeedbackData({
                type,
                category,
                content,
                rating,
                context
            });

            const feedbackId = await this.adaptiveLearningSystem.recordUserFeedback(
                sanitizedUserId,
                sanitizedFeedbackData
            );

            const userProfile = this.adaptiveLearningSystem.getUserLearningProfile(userId);

            const feedbackIcon = type === 'positive' ? '👍' : type === 'negative' ? '👎' : '👌';
            const ratingStars = '⭐'.repeat(rating) + '☆'.repeat(5 - rating);

            return {
                content: [{
                    type: 'text',
                    text: `🧠 **User Feedback Recorded Successfully**\n\n` +
                          `🆔 **Feedback ID:** ${feedbackId}\n` +
                          `👤 **User ID:** ${userId}\n` +
                          `${feedbackIcon} **Type:** ${type}\n` +
                          `📂 **Category:** ${category}\n` +
                          `${ratingStars} **Rating:** ${rating}/5\n` +
                          `💬 **Content:** "${content}"\n\n` +
                          `🤖 **Adaptive Learning Impact:**\n` +
                          `   • User profile updated with new feedback patterns\n` +
                          `   • Learning models adjusted based on feedback type\n` +
                          `   • Personalization algorithms refined\n` +
                          `   • Adaptation rules generated for future interactions\n\n` +
                          `📊 **User Learning Stats:**\n` +
                          `   • Total Feedback: ${userProfile?.profile?.feedbackStats?.total || 1}\n` +
                          `   • Average Rating: ${(userProfile?.profile?.feedbackStats?.averageRating || rating).toFixed(1)}/5\n` +
                          `   • Personality Type: ${userProfile?.profile?.learningPreferences?.personalityType || 'analyzing...'}\n` +
                          `   • Adaptation Level: ${userProfile?.profile?.learningPreferences?.adaptationLevel || 'medium'}\n\n` +
                          `💡 **Learning Benefits:**\n` +
                          `• System learns your preferences and adapts recommendations\n` +
                          `• Coding style analysis becomes more accurate\n` +
                          `• Project outcome predictions improve\n` +
                          `• Personalized suggestions get better over time\n\n` +
                          `🔄 **Next Steps:**\n` +
                          `• Continue providing feedback to improve personalization\n` +
                          `• Use \`get_personalized_recommendations\` for tailored suggestions\n` +
                          `• Check \`get_user_learning_profile\` to see your learning profile`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to record user feedback: ${error.message}`
                }]
            };
        }
    }

    async handleAnalyzeCodingStyle(args) {
        const { userId, language, code } = args;

        if (!userId || !language || !code) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide userId, language, and code.'
                }]
            };
        }

        try {
            const styleAnalysis = await this.adaptiveLearningSystem.analyzeCodingStyle(userId, {
                language,
                code
            });

            const userProfile = this.adaptiveLearningSystem.getUserLearningProfile(userId);
            const codingStyleProfile = userProfile?.codingStyle;

            const indentationText = styleAnalysis.style.indentation.type === 'tabs' ?
                `Tabs` : `${styleAnalysis.style.indentation.size} spaces`;

            const consistencyScore = codingStyleProfile ?
                (codingStyleProfile.consistency * 100).toFixed(1) : 'Calculating...';

            return {
                content: [{
                    type: 'text',
                    text: `🎨 **Coding Style Analysis Complete**\n\n` +
                          `🆔 **Analysis ID:** ${styleAnalysis.codeId}\n` +
                          `👤 **User ID:** ${userId}\n` +
                          `🔤 **Language:** ${language}\n` +
                          `📝 **Code Length:** ${code.length} characters\n\n` +
                          `🎯 **Detected Style:**\n` +
                          `   • 📐 **Indentation:** ${indentationText}\n` +
                          `   • 🏷️ **Naming Convention:** ${styleAnalysis.style.namingConvention}\n` +
                          `   • 💬 **Comment Style:** ${styleAnalysis.style.commentStyle}\n` +
                          `   • 🏗️ **Code Structure:** ${styleAnalysis.style.codeStructure}\n` +
                          `   • 🔄 **Complexity Score:** ${styleAnalysis.style.complexity}/10\n\n` +
                          `📊 **User Style Profile:**\n` +
                          `   • 🎯 **Consistency Score:** ${consistencyScore}%\n` +
                          `   • 📈 **Total Analyses:** ${codingStyleProfile?.analyses?.length || 1}\n` +
                          `   • 🏷️ **Preferred Naming:** ${userProfile?.profile?.codingStyle?.namingConvention || 'Learning...'}\n` +
                          `   • 📐 **Preferred Indentation:** ${userProfile?.profile?.codingStyle?.indentation || 'Learning...'}\n\n` +
                          `🤖 **Adaptive Learning Impact:**\n` +
                          `   • Coding style profile updated with new patterns\n` +
                          `   • Personalized recommendations will match your style\n` +
                          `   • Code generation will adapt to your preferences\n` +
                          `   • Style consistency tracking improved\n\n` +
                          `💡 **Style Insights:**\n` +
                          `${styleAnalysis.style.complexity > 7 ? '• 🔄 High complexity detected - consider breaking down functions\n' : ''}` +
                          `${styleAnalysis.style.commentStyle === 'minimal' ? '• 💬 Consider adding more descriptive comments\n' : ''}` +
                          `${styleAnalysis.style.codeStructure === 'monolithic' ? '• 🏗️ Consider modular code structure for better maintainability\n' : ''}` +
                          `• Continue coding to improve style analysis accuracy\n` +
                          `• Use consistent patterns to increase consistency score\n\n` +
                          `🔄 **Next Steps:**\n` +
                          `• Continue coding to build comprehensive style profile\n` +
                          `• Use \`get_personalized_recommendations\` for style-aware suggestions\n` +
                          `• Check \`get_user_learning_profile\` to see style evolution`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to analyze coding style: ${error.message}`
                }]
            };
        }
    }

    async handleRecordProjectOutcome(args) {
        const { userId, projectId, projectName, outcome, duration, complexity, teamSize,
                technologies = [], challenges = [], successFactors = [], lessons = [] } = args;

        if (!userId || !projectId || !projectName || !outcome) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide userId, projectId, projectName, and outcome.'
                }]
            };
        }

        if (!['success', 'failure', 'partial'].includes(outcome)) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Outcome must be one of: success, failure, partial'
                }]
            };
        }

        try {
            const outcomeId = await this.adaptiveLearningSystem.recordProjectOutcome(userId, {
                projectId,
                projectName,
                outcome,
                duration,
                complexity,
                teamSize,
                technologies,
                challenges,
                successFactors,
                lessons
            });

            const userProfile = this.adaptiveLearningSystem.getUserLearningProfile(userId);
            const userOutcomes = userProfile?.outcomes || [];

            const outcomeIcon = outcome === 'success' ? '✅' : outcome === 'failure' ? '❌' : '⚠️';
            const successRate = userOutcomes.length > 0 ?
                ((userOutcomes.filter(o => o.outcome === 'success').length / userOutcomes.length) * 100).toFixed(1) : 'N/A';

            return {
                content: [{
                    type: 'text',
                    text: `📊 **Project Outcome Recorded Successfully**\n\n` +
                          `🆔 **Outcome ID:** ${outcomeId}\n` +
                          `👤 **User ID:** ${userId}\n` +
                          `📋 **Project:** ${projectName} (${projectId})\n` +
                          `${outcomeIcon} **Outcome:** ${outcome.toUpperCase()}\n` +
                          `⏱️ **Duration:** ${duration ? duration + ' days' : 'Not specified'}\n` +
                          `🔄 **Complexity:** ${complexity || 'Not specified'}\n` +
                          `👥 **Team Size:** ${teamSize || 'Not specified'}\n\n` +
                          `🛠️ **Technologies Used:**\n${technologies.length > 0 ? technologies.map(t => `   • ${t}`).join('\n') : '   • None specified'}\n\n` +
                          `⚠️ **Challenges Faced:**\n${challenges.length > 0 ? challenges.map(c => `   • ${c}`).join('\n') : '   • None specified'}\n\n` +
                          `🎯 **Success Factors:**\n${successFactors.length > 0 ? successFactors.map(f => `   • ${f}`).join('\n') : '   • None specified'}\n\n` +
                          `📚 **Lessons Learned:**\n${lessons.length > 0 ? lessons.map(l => `   • ${l}`).join('\n') : '   • None specified'}\n\n` +
                          `📈 **User Project History:**\n` +
                          `   • Total Projects: ${userOutcomes.length}\n` +
                          `   • Success Rate: ${successRate}%\n` +
                          `   • Recent Trend: ${this.getRecentTrend(userOutcomes)}\n\n` +
                          `🤖 **Learning Impact:**\n` +
                          `   • Outcome prediction models updated\n` +
                          `   • Success/failure patterns analyzed\n` +
                          `   • Technology preferences learned\n` +
                          `   • Risk factors identified and weighted\n\n` +
                          `💡 **Insights:**\n` +
                          `${outcome === 'success' ? '• ✅ Great job! Success patterns will improve future predictions\n' : ''}` +
                          `${outcome === 'failure' ? '• 📚 Failure analysis will help prevent similar issues\n' : ''}` +
                          `${technologies.length > 3 ? '• 🛠️ Diverse technology stack - good for learning\n' : ''}` +
                          `${challenges.length > 0 ? '• ⚠️ Challenge patterns will improve risk assessment\n' : ''}` +
                          `• Continue recording outcomes to improve predictions\n\n` +
                          `🔄 **Next Steps:**\n` +
                          `• Use \`predict_project_outcome\` for future project planning\n` +
                          `• Get \`get_personalized_recommendations\` based on your history\n` +
                          `• Check \`get_user_learning_profile\` to see learning evolution`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to record project outcome: ${error.message}`
                }]
            };
        }
    }

    getRecentTrend(outcomes) {
        if (outcomes.length < 3) return 'Insufficient data';

        const recent = outcomes.slice(-3);
        const successCount = recent.filter(o => o.outcome === 'success').length;

        if (successCount === 3) return '🚀 Excellent (3/3 successes)';
        if (successCount === 2) return '📈 Good (2/3 successes)';
        if (successCount === 1) return '📊 Mixed (1/3 successes)';
        return '📉 Needs attention (0/3 successes)';
    }

    async handleGetPersonalizedRecommendations(args) {
        const { userId, context = {} } = args;

        if (!userId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameter. Please provide a user ID.'
                }]
            };
        }

        try {
            const recommendations = await this.adaptiveLearningSystem.generatePersonalizedRecommendations(userId, context);

            if (recommendations.recommendations.length === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: `🤖 **No Personalized Recommendations Available**\n\n` +
                              `👤 **User ID:** ${userId}\n\n` +
                              `📝 **Reason:** Insufficient learning data\n\n` +
                              `💡 **To Get Personalized Recommendations:**\n` +
                              `• Provide feedback using \`record_user_feedback\`\n` +
                              `• Analyze your coding style with \`analyze_coding_style\`\n` +
                              `• Record project outcomes with \`record_project_outcome\`\n` +
                              `• Continue using the system to build your profile`
                    }]
                };
            }

            const recommendationsList = recommendations.recommendations.map((rec, index) => {
                const priorityIcon = rec.priority === 'high' ? '🔴' : rec.priority === 'medium' ? '🟡' : '🟢';
                const confidenceBar = '█'.repeat(Math.round(rec.confidence * 10)) + '░'.repeat(10 - Math.round(rec.confidence * 10));

                return `   ${index + 1}. ${priorityIcon} **${rec.type.toUpperCase()}** (${rec.category})\n` +
                       `      📝 ${rec.message}\n` +
                       `      🎯 Priority: ${rec.priority} | Confidence: ${confidenceBar} ${(rec.confidence * 100).toFixed(0)}%`;
            }).join('\n\n');

            const contextText = Object.keys(context).length > 0 ?
                Object.entries(context).map(([key, value]) => `   • ${key}: ${value}`).join('\n') :
                '   • No specific context provided';

            return {
                content: [{
                    type: 'text',
                    text: `🤖 **Personalized Recommendations**\n\n` +
                          `👤 **User ID:** ${userId}\n` +
                          `🎯 **Overall Confidence:** ${(recommendations.confidence * 100).toFixed(1)}%\n` +
                          `📊 **Total Recommendations:** ${recommendations.recommendations.length}\n` +
                          `⏰ **Generated:** ${new Date(recommendations.timestamp).toLocaleString()}\n\n` +
                          `🔍 **Context:**\n${contextText}\n\n` +
                          `💡 **Recommendations:**\n${recommendationsList}\n\n` +
                          `🧠 **Personalization Factors:**\n` +
                          `   • Your feedback patterns and preferences\n` +
                          `   • Coding style analysis and consistency\n` +
                          `   • Project outcome history and success patterns\n` +
                          `   • Behavioral patterns and tool usage\n` +
                          `   • Adaptation rules learned from your interactions\n\n` +
                          `📈 **Recommendation Quality:**\n` +
                          `${recommendations.confidence > 0.8 ? '• 🌟 High-quality recommendations based on rich profile data\n' : ''}` +
                          `${recommendations.confidence > 0.6 && recommendations.confidence <= 0.8 ? '• 👍 Good recommendations with moderate confidence\n' : ''}` +
                          `${recommendations.confidence <= 0.6 ? '• 📊 Basic recommendations - provide more feedback to improve\n' : ''}` +
                          `• Recommendations improve as you use the system more\n\n` +
                          `🔄 **Next Steps:**\n` +
                          `• Follow high-priority recommendations first\n` +
                          `• Provide feedback on recommendation quality\n` +
                          `• Continue using the system to refine recommendations`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get personalized recommendations: ${error.message}`
                }]
            };
        }
    }

    async handlePredictProjectOutcome(args) {
        const { userId, projectId, complexity, teamSize, timeline, technologies = [], challenges = [] } = args;

        if (!userId || !projectId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide userId and projectId.'
                }]
            };
        }

        try {
            const prediction = await this.adaptiveLearningSystem.predictProjectOutcome(userId, {
                projectId,
                complexity,
                teamSize,
                timeline,
                technologies,
                challenges
            });

            const outcomeIcon = prediction.outcome === 'success' ? '✅' :
                               prediction.outcome === 'failure' ? '❌' : '⚠️';
            const confidenceBar = '█'.repeat(Math.round(prediction.confidence * 10)) +
                                 '░'.repeat(10 - Math.round(prediction.confidence * 10));
            const successBar = '█'.repeat(Math.round(prediction.successProbability * 10)) +
                              '░'.repeat(10 - Math.round(prediction.successProbability * 10));

            const recommendationsList = prediction.recommendations.length > 0 ?
                prediction.recommendations.map((rec, index) =>
                    `   ${index + 1}. **${rec.type.toUpperCase()}** (${rec.priority})\n` +
                    `      ${rec.message}`
                ).join('\n\n') : '   • No specific recommendations at this time';

            return {
                content: [{
                    type: 'text',
                    text: `🔮 **Project Outcome Prediction**\n\n` +
                          `👤 **User ID:** ${userId}\n` +
                          `📋 **Project ID:** ${projectId}\n` +
                          `${outcomeIcon} **Predicted Outcome:** ${prediction.outcome.toUpperCase()}\n` +
                          `🎯 **Confidence:** ${confidenceBar} ${(prediction.confidence * 100).toFixed(1)}%\n` +
                          `📈 **Success Probability:** ${successBar} ${(prediction.successProbability * 100).toFixed(1)}%\n\n` +
                          `📊 **Project Factors:**\n` +
                          `   • Complexity: ${complexity || 'Not specified'}\n` +
                          `   • Team Size: ${teamSize || 'Not specified'}\n` +
                          `   • Timeline: ${timeline ? timeline + ' days' : 'Not specified'}\n` +
                          `   • Technologies: ${technologies.length > 0 ? technologies.join(', ') : 'None specified'}\n` +
                          `   • Expected Challenges: ${challenges.length > 0 ? challenges.join(', ') : 'None specified'}\n\n` +
                          `🧠 **Prediction Basis:**\n` +
                          `   • User History: ${prediction.factors.userHistory}\n` +
                          `   • Project Complexity: ${prediction.factors.projectComplexity}\n` +
                          `   • Team Size Impact: ${prediction.factors.teamSize}\n` +
                          `   • Timeline Pressure: ${prediction.factors.timeline}\n\n` +
                          `💡 **Recommendations:**\n${recommendationsList}\n\n` +
                          `📈 **Success Factors:**\n` +
                          `${prediction.successProbability > 0.7 ? '• 🚀 High success probability - project looks promising\n' : ''}` +
                          `${prediction.successProbability >= 0.4 && prediction.successProbability <= 0.7 ? '• ⚖️ Moderate success probability - monitor key factors\n' : ''}` +
                          `${prediction.successProbability < 0.4 ? '• ⚠️ Low success probability - consider risk mitigation\n' : ''}` +
                          `${teamSize && teamSize > 8 ? '• 👥 Large team requires strong coordination\n' : ''}` +
                          `${complexity === 'high' ? '• 🔄 High complexity requires careful planning\n' : ''}` +
                          `${timeline && timeline < 30 ? '• ⏰ Tight timeline increases risk\n' : ''}` +
                          `• Prediction accuracy improves with more project data\n\n` +
                          `🔄 **Next Steps:**\n` +
                          `• Consider the recommendations to improve success probability\n` +
                          `• Record actual outcome when project completes\n` +
                          `• Use insights for better project planning\n` +
                          `• Monitor progress against predictions`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to predict project outcome: ${error.message}`
                }]
            };
        }
    }

    async handleGetLearningAnalytics(args) {
        try {
            const analytics = this.adaptiveLearningSystem.getLearningAnalytics();

            const personalityChart = Object.entries(analytics.personalityDistribution).length > 0 ?
                Object.entries(analytics.personalityDistribution).map(([personality, count]) =>
                    `   • ${personality}: ${count} users (${((count / analytics.totalUsers) * 100).toFixed(1)}%)`
                ).join('\n') : '   • No personality data available';

            const systemHealth = analytics.modelAccuracy > 0.8 ? '🟢 Excellent' :
                                analytics.modelAccuracy > 0.6 ? '🟡 Good' : '🔴 Needs Improvement';

            return {
                content: [{
                    type: 'text',
                    text: `🧠 **Adaptive Learning System Analytics**\n\n` +
                          `📊 **Overview:**\n` +
                          `   • Total Users: ${analytics.totalUsers}\n` +
                          `   • Total Feedback Entries: ${analytics.totalFeedback}\n` +
                          `   • Project Outcomes Recorded: ${analytics.totalOutcomes}\n` +
                          `   • Coding Style Analyses: ${analytics.totalStyleAnalyses}\n\n` +
                          `📈 **Learning Quality:**\n` +
                          `   • Average Feedback Rating: ${analytics.avgFeedbackRating.toFixed(1)}/5\n` +
                          `   • Model Accuracy: ${(analytics.modelAccuracy * 100).toFixed(1)}%\n` +
                          `   • System Health: ${systemHealth}\n` +
                          `   • Active Adaptation Rules: ${analytics.adaptationRules}\n\n` +
                          `🧠 **User Personality Distribution:**\n${personalityChart}\n\n` +
                          `🤖 **Learning Components:**\n` +
                          `   • Learning Models: ${analytics.learningModels}\n` +
                          `   • Learning Categories: ${analytics.learningCategories}\n` +
                          `   • Adaptation Rules Generated: ${analytics.adaptationRules}\n\n` +
                          `💡 **System Insights:**\n` +
                          `${analytics.totalUsers > 10 ? '• 👥 Good user adoption - diverse learning data\n' : '• 📈 Growing user base - more data will improve accuracy\n'}` +
                          `${analytics.avgFeedbackRating > 4.0 ? '• 😊 High user satisfaction with recommendations\n' : analytics.avgFeedbackRating > 3.0 ? '• 👍 Moderate satisfaction - room for improvement\n' : '• 📊 Low satisfaction - system needs refinement\n'}` +
                          `${analytics.modelAccuracy > 0.8 ? '• 🎯 High model accuracy - reliable predictions\n' : '• 📚 Models learning - accuracy will improve with more data\n'}` +
                          `${analytics.totalOutcomes > 50 ? '• 📊 Rich outcome data enables accurate predictions\n' : '• 🔮 More outcome data needed for better predictions\n'}` +
                          `• System continuously learns and adapts to user patterns\n\n` +
                          `🎯 **Recommendations:**\n` +
                          `• Encourage users to provide more feedback\n` +
                          `• Record project outcomes for better predictions\n` +
                          `• Analyze coding styles to improve personalization\n` +
                          `• Monitor model performance and retrain as needed`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get learning analytics: ${error.message}`
                }]
            };
        }
    }

    async handleGetUserLearningProfile(args) {
        const { userId } = args;

        if (!userId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameter. Please provide a user ID.'
                }]
            };
        }

        try {
            const learningProfile = this.adaptiveLearningSystem.getUserLearningProfile(userId);

            if (!learningProfile) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ No learning profile found for user "${userId}".\n\n` +
                              `💡 **To Create a Learning Profile:**\n` +
                              `• Provide feedback using \`record_user_feedback\`\n` +
                              `• Analyze coding style with \`analyze_coding_style\`\n` +
                              `• Record project outcomes with \`record_project_outcome\`\n` +
                              `• Use the system regularly to build your profile`
                    }]
                };
            }

            const profile = learningProfile.profile;
            const codingStyle = learningProfile.codingStyle;

            const feedbackChart = `${profile.feedbackStats.positive}👍 ${profile.feedbackStats.negative}👎 ${profile.feedbackStats.neutral}👌`;
            const personalityIcon = profile.learningPreferences.personalityType === 'analytical' ? '🔬' :
                                   profile.learningPreferences.personalityType === 'creative' ? '🎨' :
                                   profile.learningPreferences.personalityType === 'practical' ? '🔧' : '⚖️';

            const recentFeedback = learningProfile.feedback.slice(-3).map((f, index) =>
                `   ${index + 1}. ${f.type === 'positive' ? '👍' : f.type === 'negative' ? '👎' : '👌'} ${f.category} (${f.rating}/5): "${f.content.substring(0, 50)}..."`
            ).join('\n') || '   • No recent feedback';

            const adaptationRulesList = learningProfile.adaptationRules.slice(0, 3).map((rule, index) =>
                `   ${index + 1}. ${rule.condition} → ${rule.action} (applied ${rule.appliedCount} times)`
            ).join('\n') || '   • No adaptation rules generated yet';

            const outcomesSummary = learningProfile.outcomes.length > 0 ?
                `${learningProfile.outcomes.filter(o => o.outcome === 'success').length}✅ ` +
                `${learningProfile.outcomes.filter(o => o.outcome === 'failure').length}❌ ` +
                `${learningProfile.outcomes.filter(o => o.outcome === 'partial').length}⚠️` :
                'No outcomes recorded';

            return {
                content: [{
                    type: 'text',
                    text: `🧠 **User Learning Profile**\n\n` +
                          `👤 **User ID:** ${userId}\n` +
                          `📅 **Profile Created:** ${new Date(profile.createdAt).toLocaleDateString()}\n` +
                          `⏰ **Last Activity:** ${new Date(profile.lastActivity).toLocaleString()}\n` +
                          `🔄 **Total Interactions:** ${profile.totalInteractions}\n\n` +
                          `📊 **Feedback Statistics:**\n` +
                          `   • Total Feedback: ${profile.feedbackStats.total}\n` +
                          `   • Distribution: ${feedbackChart}\n` +
                          `   • Average Rating: ${profile.feedbackStats.averageRating.toFixed(1)}/5\n\n` +
                          `🧠 **Learning Preferences:**\n` +
                          `   ${personalityIcon} **Personality Type:** ${profile.learningPreferences.personalityType}\n` +
                          `   • Adaptation Level: ${profile.learningPreferences.adaptationLevel}\n` +
                          `   • Collaboration Style: ${profile.behaviorPatterns.collaborationStyle}\n\n` +
                          `🎨 **Coding Style:**\n` +
                          `   • Indentation: ${profile.codingStyle.indentation} (${profile.codingStyle.indentSize})\n` +
                          `   • Naming Convention: ${profile.codingStyle.namingConvention}\n` +
                          `   • Comment Style: ${profile.codingStyle.commentStyle}\n` +
                          `   • Code Structure: ${profile.codingStyle.codeStructure}\n` +
                          `   • Style Consistency: ${codingStyle ? (codingStyle.consistency * 100).toFixed(1) + '%' : 'Calculating...'}\n\n` +
                          `📈 **Project Outcomes:**\n` +
                          `   • Total Projects: ${learningProfile.outcomes.length}\n` +
                          `   • Outcome Distribution: ${outcomesSummary}\n` +
                          `   • Success Rate: ${learningProfile.outcomes.length > 0 ? ((learningProfile.outcomes.filter(o => o.outcome === 'success').length / learningProfile.outcomes.length) * 100).toFixed(1) + '%' : 'N/A'}\n\n` +
                          `💬 **Recent Feedback:**\n${recentFeedback}\n\n` +
                          `🔧 **Adaptation Rules:**\n${adaptationRulesList}\n\n` +
                          `🎯 **Personalization Status:**\n` +
                          `${profile.totalInteractions > 50 ? '• 🌟 Rich profile data - highly personalized recommendations\n' : profile.totalInteractions > 20 ? '• 👍 Good profile data - personalized recommendations available\n' : '• 📊 Building profile - basic personalization active\n'}` +
                          `${profile.feedbackStats.total > 10 ? '• 💬 Sufficient feedback for accurate adaptation\n' : '• 📝 More feedback needed for better adaptation\n'}` +
                          `${codingStyle && codingStyle.consistency > 0.8 ? '• 🎨 Consistent coding style detected\n' : '• 🔍 Learning your coding style preferences\n'}` +
                          `• Profile continuously evolves with your usage\n\n` +
                          `🔄 **Next Steps:**\n` +
                          `• Continue providing feedback to improve personalization\n` +
                          `• Record more project outcomes for better predictions\n` +
                          `• Use \`get_personalized_recommendations\` for tailored suggestions`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get user learning profile: ${error.message}`
                }]
            };
        }
    }

    async handleCreatePrompt(args) {
        const { title, content, description, domain, tags, isPublic, authorId } = args;

        if (!title || !content || !authorId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide title, content, and authorId.'
                }]
            };
        }

        try {
            // Sanitize and validate inputs
            const sanitizedAuthorId = this.inputSanitizer.sanitizeUserId(authorId);
            const sanitizedPromptData = this.inputSanitizer.validatePromptData({
                title,
                content,
                description,
                domain,
                tags,
                isPublic
            });

            const promptId = await this.enhancedPromptLibrary.createPrompt(
                sanitizedPromptData,
                sanitizedAuthorId
            );

            const prompt = this.enhancedPromptLibrary.getPromptById(promptId);
            const variables = prompt.variables.length > 0 ?
                prompt.variables.map(v => `{${v}}`).join(', ') : 'None';

            return {
                content: [{
                    type: 'text',
                    text: `📝 **Prompt Created Successfully**\n\n` +
                          `🆔 **Prompt ID:** ${promptId}\n` +
                          `📋 **Title:** ${prompt.title}\n` +
                          `👤 **Author:** ${prompt.author}\n` +
                          `🏷️ **Domain:** ${prompt.domain}\n` +
                          `🔖 **Tags:** ${prompt.tags.join(', ') || 'None'}\n` +
                          `📊 **Quality Score:** ${(prompt.quality * 100).toFixed(1)}%\n` +
                          `🔧 **Variables:** ${variables}\n` +
                          `🌐 **Public:** ${prompt.isPublic ? 'Yes' : 'No'}\n` +
                          `📝 **Version:** ${prompt.version}\n\n` +
                          `📄 **Content Preview:**\n` +
                          `"${prompt.content.substring(0, 200)}${prompt.content.length > 200 ? '...' : ''}"\n\n` +
                          `💡 **Features:**\n` +
                          `• Automatic quality assessment and scoring\n` +
                          `• Variable extraction for parameterization\n` +
                          `• Version control for future updates\n` +
                          `• Domain categorization for easy discovery\n` +
                          `• Usage tracking and analytics\n\n` +
                          `🔄 **Next Steps:**\n` +
                          `• Use \`search_prompts\` to find similar prompts\n` +
                          `• Rate prompts with \`rate_prompt\` to improve recommendations\n` +
                          `• Create collections with \`create_collection\`\n` +
                          `• Start A/B tests with \`start_ab_test\``
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to create prompt: ${error.message}`
                }]
            };
        }
    }

    async handleSearchPrompts(args) {
        const { query, domain, tags, minQuality, minRating, author, limit = 10 } = args;

        try {
            // Sanitize and validate inputs
            const sanitizedQuery = query ? this.inputSanitizer.sanitizeString(query, 'content') : '';
            const sanitizedDomain = domain ? this.inputSanitizer.sanitizeDomain(domain) : undefined;
            const sanitizedTags = tags ? this.inputSanitizer.sanitizeTags(tags) : undefined;
            const sanitizedAuthor = author ? this.inputSanitizer.sanitizeUserId(author) : undefined;
            const validatedLimit = this.inputSanitizer.validateLimit(limit);

            const filters = {};
            if (sanitizedDomain) filters.domain = sanitizedDomain;
            if (sanitizedTags) filters.tags = sanitizedTags;
            if (minQuality) filters.minQuality = Math.max(0, Math.min(1, minQuality));
            if (minRating) filters.minRating = Math.max(1, Math.min(5, minRating));
            if (sanitizedAuthor) filters.author = sanitizedAuthor;

            const results = this.enhancedPromptLibrary.searchPrompts(sanitizedQuery, filters);
            const limitedResults = results.slice(0, validatedLimit);

            if (limitedResults.length === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: `🔍 **No Prompts Found**\n\n` +
                              `📝 **Search Query:** "${query || 'All prompts'}"\n` +
                              `🏷️ **Filters Applied:** ${Object.keys(filters).length > 0 ? Object.entries(filters).map(([k, v]) => `${k}: ${v}`).join(', ') : 'None'}\n\n` +
                              `💡 **Suggestions:**\n` +
                              `• Try broader search terms\n` +
                              `• Remove some filters\n` +
                              `• Check available domains with \`get_prompt_analytics\`\n` +
                              `• Browse popular prompts with \`get_popular_prompts\``
                    }]
                };
            }

            const promptsList = limitedResults.map((prompt, index) => {
                const variables = prompt.variables.length > 0 ?
                    prompt.variables.map(v => `{${v}}`).join(', ') : 'None';
                const qualityBar = '█'.repeat(Math.round(prompt.quality * 10)) + '░'.repeat(10 - Math.round(prompt.quality * 10));
                const ratingStars = prompt.rating > 0 ? '⭐'.repeat(Math.round(prompt.rating)) + '☆'.repeat(5 - Math.round(prompt.rating)) : 'Not rated';

                return `   ${index + 1}. **${prompt.title}** (${prompt.id})\n` +
                       `      📝 "${prompt.content.substring(0, 100)}${prompt.content.length > 100 ? '...' : ''}"\n` +
                       `      🏷️ Domain: ${prompt.domain} | Tags: ${prompt.tags.join(', ') || 'None'}\n` +
                       `      📊 Quality: ${qualityBar} ${(prompt.quality * 100).toFixed(0)}% | Rating: ${ratingStars}\n` +
                       `      🔧 Variables: ${variables} | Usage: ${prompt.usage} times\n` +
                       `      👤 Author: ${prompt.author} | Version: ${prompt.version}`;
            }).join('\n\n');

            const searchSummary = query ? `"${query}"` : 'All prompts';
            const filterSummary = Object.keys(filters).length > 0 ?
                Object.entries(filters).map(([k, v]) => `${k}: ${Array.isArray(v) ? v.join(', ') : v}`).join(', ') : 'None';

            return {
                content: [{
                    type: 'text',
                    text: `🔍 **Prompt Search Results**\n\n` +
                          `📝 **Search Query:** ${searchSummary}\n` +
                          `🏷️ **Filters:** ${filterSummary}\n` +
                          `📊 **Results:** ${limitedResults.length} of ${results.length} total matches\n\n` +
                          `📋 **Prompts:**\n${promptsList}\n\n` +
                          `💡 **Tips:**\n` +
                          `• Use \`get_prompt_by_id\` to view full prompt details\n` +
                          `• Rate prompts to help improve search relevance\n` +
                          `• Create collections to organize your favorite prompts\n` +
                          `• Submit improvements with \`update_prompt\`\n\n` +
                          `🔄 **Refine Search:**\n` +
                          `• Add domain filter for specific categories\n` +
                          `• Set minimum quality/rating thresholds\n` +
                          `• Use specific tags for better targeting`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to search prompts: ${error.message}`
                }]
            };
        }
    }

    async handleGetPromptById(args) {
        const { promptId } = args;

        if (!promptId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameter. Please provide a prompt ID.'
                }]
            };
        }

        try {
            const prompt = this.enhancedPromptLibrary.getPromptById(promptId);

            if (!prompt) {
                return {
                    content: [{
                        type: 'text',
                        text: `❌ Prompt "${promptId}" not found.\n\n` +
                              `💡 **Try:**\n` +
                              `• Use \`search_prompts\` to find prompts\n` +
                              `• Check \`get_popular_prompts\` for trending prompts\n` +
                              `• Browse by domain with \`get_prompts_by_domain\``
                    }]
                };
            }

            const variables = prompt.variables.length > 0 ?
                prompt.variables.map(v => `   • {${v}}`).join('\n') : '   • No variables';
            const qualityBar = '█'.repeat(Math.round(prompt.quality * 10)) + '░'.repeat(10 - Math.round(prompt.quality * 10));
            const ratingStars = prompt.rating > 0 ? '⭐'.repeat(Math.round(prompt.rating)) + '☆'.repeat(5 - Math.round(prompt.rating)) : 'Not rated yet';

            const recentRatings = prompt.ratings.slice(-3).map((r, index) =>
                `   ${index + 1}. ${r.rating}/5 by ${r.userId}: "${r.feedback || 'No feedback'}"`
            ).join('\n') || '   • No ratings yet';

            return {
                content: [{
                    type: 'text',
                    text: `📝 **Prompt Details**\n\n` +
                          `🆔 **ID:** ${prompt.id}\n` +
                          `📋 **Title:** ${prompt.title}\n` +
                          `📄 **Description:** ${prompt.description || 'No description provided'}\n` +
                          `👤 **Author:** ${prompt.author}\n` +
                          `🏷️ **Domain:** ${prompt.domain}\n` +
                          `🔖 **Tags:** ${prompt.tags.join(', ') || 'None'}\n` +
                          `📊 **Quality:** ${qualityBar} ${(prompt.quality * 100).toFixed(1)}%\n` +
                          `⭐ **Rating:** ${ratingStars} (${prompt.ratings.length} reviews)\n` +
                          `📈 **Usage:** ${prompt.usage} times\n` +
                          `📝 **Version:** ${prompt.version}\n` +
                          `🌐 **Public:** ${prompt.isPublic ? 'Yes' : 'No'}\n` +
                          `📅 **Created:** ${new Date(prompt.createdAt).toLocaleDateString()}\n` +
                          `⏰ **Updated:** ${new Date(prompt.updatedAt).toLocaleDateString()}\n\n` +
                          `📄 **Content:**\n` +
                          `"${prompt.content}"\n\n` +
                          `🔧 **Variables:**\n${variables}\n\n` +
                          `💬 **Recent Reviews:**\n${recentRatings}\n\n` +
                          `💡 **Usage Tips:**\n` +
                          `• Replace variables with your specific values\n` +
                          `• Rate this prompt to help others\n` +
                          `• Create variations with \`update_prompt\`\n` +
                          `• Add to collections for organization\n\n` +
                          `🔄 **Actions:**\n` +
                          `• Rate: \`rate_prompt\`\n` +
                          `• Update: \`update_prompt\`\n` +
                          `• Find similar: \`search_prompts\``
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get prompt: ${error.message}`
                }]
            };
        }
    }

    async handleUpdatePrompt(args) {
        const { promptId, title, content, description, tags, changes, authorId } = args;

        if (!promptId || !authorId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide promptId and authorId.'
                }]
            };
        }

        try {
            const updates = {};
            if (title) updates.title = title;
            if (content) updates.content = content;
            if (description) updates.description = description;
            if (tags) updates.tags = tags;
            if (changes) updates.changes = changes;

            const newVersion = await this.enhancedPromptLibrary.updatePrompt(promptId, updates, authorId);
            const prompt = this.enhancedPromptLibrary.getPromptById(promptId);

            return {
                content: [{
                    type: 'text',
                    text: `📝 **Prompt Updated Successfully**\n\n` +
                          `🆔 **Prompt ID:** ${promptId}\n` +
                          `📋 **Title:** ${prompt.title}\n` +
                          `📝 **New Version:** ${newVersion}\n` +
                          `👤 **Updated by:** ${authorId}\n` +
                          `📊 **Quality Score:** ${(prompt.quality * 100).toFixed(1)}%\n` +
                          `🔧 **Variables:** ${prompt.variables.length > 0 ? prompt.variables.map(v => `{${v}}`).join(', ') : 'None'}\n\n` +
                          `📄 **Updated Content:**\n` +
                          `"${prompt.content.substring(0, 200)}${prompt.content.length > 200 ? '...' : ''}"\n\n` +
                          `🔄 **Changes:** ${changes || 'No change description provided'}\n\n` +
                          `💡 **Version Control:**\n` +
                          `• Previous versions are preserved\n` +
                          `• Quality score recalculated\n` +
                          `• Variables automatically extracted\n` +
                          `• Update history tracked\n\n` +
                          `🔄 **Next Steps:**\n` +
                          `• Test the updated prompt\n` +
                          `• Consider A/B testing with \`start_ab_test\`\n` +
                          `• Share improvements with the community`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to update prompt: ${error.message}`
                }]
            };
        }
    }

    async handleRatePrompt(args) {
        const { promptId, userId, rating, feedback } = args;

        if (!promptId || !userId || rating === undefined) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide promptId, userId, and rating.'
                }]
            };
        }

        try {
            // Sanitize and validate inputs
            const sanitizedUserId = this.inputSanitizer.sanitizeUserId(userId);
            const sanitizedPromptId = this.inputSanitizer.sanitizeString(promptId, 'userId'); // Reuse userId validation for IDs
            const validatedRating = this.inputSanitizer.validateRating(rating);
            const sanitizedFeedback = this.inputSanitizer.sanitizeFeedback(feedback);

            await this.enhancedPromptLibrary.ratePrompt(
                sanitizedPromptId,
                sanitizedUserId,
                validatedRating,
                sanitizedFeedback
            );
            const prompt = this.enhancedPromptLibrary.getPromptById(promptId);

            const ratingStars = '⭐'.repeat(rating) + '☆'.repeat(5 - rating);
            const avgRatingStars = '⭐'.repeat(Math.round(prompt.rating)) + '☆'.repeat(5 - Math.round(prompt.rating));

            return {
                content: [{
                    type: 'text',
                    text: `⭐ **Prompt Rated Successfully**\n\n` +
                          `🆔 **Prompt:** ${prompt.title} (${promptId})\n` +
                          `👤 **Rated by:** ${userId}\n` +
                          `${ratingStars} **Your Rating:** ${rating}/5\n` +
                          `💬 **Feedback:** "${feedback || 'No feedback provided'}"\n\n` +
                          `📊 **Updated Statistics:**\n` +
                          `   ${avgRatingStars} **Average Rating:** ${prompt.rating.toFixed(1)}/5\n` +
                          `   📈 **Total Reviews:** ${prompt.ratings.length}\n` +
                          `   📊 **Usage Count:** ${prompt.usage}\n\n` +
                          `💡 **Impact:**\n` +
                          `• Your rating helps improve search relevance\n` +
                          `• Feedback guides prompt improvements\n` +
                          `• High-rated prompts get better visibility\n` +
                          `• Community benefits from your input\n\n` +
                          `🔄 **Next Steps:**\n` +
                          `• Explore similar prompts with \`search_prompts\`\n` +
                          `• Create your own prompts with \`create_prompt\`\n` +
                          `• Join A/B tests to compare prompt effectiveness`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to rate prompt: ${error.message}`
                }]
            };
        }
    }

    async handleCreateCollection(args) {
        const { name, description, domain, tags, visibility, authorId } = args;

        if (!name || !authorId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide name and authorId.'
                }]
            };
        }

        try {
            const collectionId = await this.enhancedPromptLibrary.createCollection({
                name,
                description,
                domain,
                tags,
                visibility
            }, authorId);

            const collection = this.enhancedPromptLibrary.getCollectionById(collectionId);

            return {
                content: [{
                    type: 'text',
                    text: `📚 **Collection Created Successfully**\n\n` +
                          `🆔 **Collection ID:** ${collectionId}\n` +
                          `📋 **Name:** ${collection.name}\n` +
                          `📄 **Description:** ${collection.description || 'No description provided'}\n` +
                          `👤 **Author:** ${collection.author}\n` +
                          `🏷️ **Domain:** ${collection.domain}\n` +
                          `🔖 **Tags:** ${collection.tags.join(', ') || 'None'}\n` +
                          `👁️ **Visibility:** ${collection.visibility}\n` +
                          `📊 **Prompts:** ${collection.prompts.length}\n` +
                          `👥 **Contributors:** ${collection.contributors.length}\n\n` +
                          `💡 **Collection Features:**\n` +
                          `• Organize related prompts together\n` +
                          `• Share curated prompt sets\n` +
                          `• Collaborative prompt management\n` +
                          `• Domain-specific categorization\n\n` +
                          `🔄 **Next Steps:**\n` +
                          `• Add prompts to your collection\n` +
                          `• Invite collaborators to contribute\n` +
                          `• Share with the community\n` +
                          `• Create themed prompt collections`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to create collection: ${error.message}`
                }]
            };
        }
    }

    async handleStartABTest(args) {
        const { name, description, promptA, promptB, testType, targetMetric, authorId } = args;

        if (!name || !promptA || !promptB || !authorId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide name, promptA, promptB, and authorId.'
                }]
            };
        }

        try {
            const testId = await this.enhancedPromptLibrary.startABTest({
                name,
                description,
                promptA,
                promptB,
                testType,
                targetMetric
            }, authorId);

            const abTest = this.enhancedPromptLibrary.abTests.get(testId);
            const promptADetails = this.enhancedPromptLibrary.getPromptById(promptA);
            const promptBDetails = this.enhancedPromptLibrary.getPromptById(promptB);

            return {
                content: [{
                    type: 'text',
                    text: `🧪 **A/B Test Started Successfully**\n\n` +
                          `🆔 **Test ID:** ${testId}\n` +
                          `📋 **Name:** ${abTest.name}\n` +
                          `📄 **Description:** ${abTest.description || 'No description provided'}\n` +
                          `👤 **Created by:** ${abTest.author}\n` +
                          `🎯 **Test Type:** ${abTest.testType}\n` +
                          `📊 **Target Metric:** ${abTest.targetMetric}\n` +
                          `⏰ **Duration:** 7 days\n` +
                          `📅 **End Date:** ${new Date(abTest.endDate).toLocaleDateString()}\n\n` +
                          `🅰️ **Prompt A:** ${promptADetails?.title || promptA}\n` +
                          `   "${promptADetails?.content.substring(0, 100) || 'Prompt not found'}${promptADetails?.content.length > 100 ? '...' : ''}"\n\n` +
                          `🅱️ **Prompt B:** ${promptBDetails?.title || promptB}\n` +
                          `   "${promptBDetails?.content.substring(0, 100) || 'Prompt not found'}${promptBDetails?.content.length > 100 ? '...' : ''}"\n\n` +
                          `📊 **Test Metrics:**\n` +
                          `• User ratings and feedback\n` +
                          `• Usage frequency and success rates\n` +
                          `• Statistical significance calculation\n` +
                          `• Automatic winner determination\n\n` +
                          `💡 **How It Works:**\n` +
                          `• Users will be randomly assigned prompt variants\n` +
                          `• Usage and ratings are tracked automatically\n` +
                          `• Statistical significance is calculated continuously\n` +
                          `• Test completes after 7 days or significance achieved\n\n` +
                          `🔄 **Next Steps:**\n` +
                          `• Share test prompts with users\n` +
                          `• Monitor test progress\n` +
                          `• Review results when test completes`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to start A/B test: ${error.message}`
                }]
            };
        }
    }

    async handleSubmitCommunityPrompt(args) {
        const { title, content, description, domain, tags, authorId } = args;

        if (!title || !content || !authorId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameters. Please provide title, content, and authorId.'
                }]
            };
        }

        try {
            const contributionId = await this.enhancedPromptLibrary.submitCommunityPrompt({
                title,
                content,
                description,
                domain,
                tags
            }, authorId);

            const contribution = this.enhancedPromptLibrary.communityContributions.get(contributionId);
            const moderationBar = '█'.repeat(Math.round(contribution.moderationScore * 10)) +
                                 '░'.repeat(10 - Math.round(contribution.moderationScore * 10));

            return {
                content: [{
                    type: 'text',
                    text: `🤝 **Community Prompt Submitted Successfully**\n\n` +
                          `🆔 **Contribution ID:** ${contributionId}\n` +
                          `📋 **Title:** ${contribution.title}\n` +
                          `👤 **Author:** ${contribution.author}\n` +
                          `🏷️ **Domain:** ${contribution.domain}\n` +
                          `🔖 **Tags:** ${contribution.tags.join(', ') || 'None'}\n` +
                          `📊 **Status:** ${contribution.status}\n` +
                          `🎯 **Moderation Score:** ${moderationBar} ${(contribution.moderationScore * 100).toFixed(1)}%\n\n` +
                          `📄 **Content Preview:**\n` +
                          `"${contribution.content.substring(0, 200)}${contribution.content.length > 200 ? '...' : ''}"\n\n` +
                          `🔄 **Review Process:**\n` +
                          `${contribution.moderationScore >= 0.8 ? '• ✅ Auto-approved due to high quality score\n' : '• ⏳ Pending community review and moderation\n'}` +
                          `• Community members can vote on your contribution\n` +
                          `• Moderators review for quality and appropriateness\n` +
                          `• Approved prompts become part of the public library\n\n` +
                          `💡 **Community Benefits:**\n` +
                          `• Share knowledge with other developers\n` +
                          `• Get feedback on your prompt designs\n` +
                          `• Build reputation in the community\n` +
                          `• Help improve the prompt ecosystem\n\n` +
                          `🔄 **Next Steps:**\n` +
                          `• Monitor community votes and feedback\n` +
                          `• Respond to reviewer comments\n` +
                          `• Create more high-quality contributions`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to submit community prompt: ${error.message}`
                }]
            };
        }
    }

    async handleGetPopularPrompts(args) {
        const { limit = 10 } = args;

        try {
            const popularPrompts = this.enhancedPromptLibrary.getPopularPrompts(limit);

            if (popularPrompts.length === 0) {
                return {
                    content: [{
                        type: 'text',
                        text: `📊 **No Popular Prompts Available**\n\n` +
                              `💡 **This could mean:**\n` +
                              `• The library is new and needs more usage data\n` +
                              `• No prompts have been rated yet\n` +
                              `• All prompts are private\n\n` +
                              `🔄 **Try:**\n` +
                              `• Create some prompts with \`create_prompt\`\n` +
                              `• Rate existing prompts with \`rate_prompt\`\n` +
                              `• Browse by domain with \`get_prompts_by_domain\``
                    }]
                };
            }

            const promptsList = popularPrompts.map((prompt, index) => {
                const ratingStars = prompt.rating > 0 ? '⭐'.repeat(Math.round(prompt.rating)) + '☆'.repeat(5 - Math.round(prompt.rating)) : 'Not rated';
                const qualityBar = '█'.repeat(Math.round(prompt.quality * 10)) + '░'.repeat(10 - Math.round(prompt.quality * 10));
                const popularityScore = (prompt.usage * 0.4 + prompt.rating * 0.6).toFixed(1);

                return `   ${index + 1}. **${prompt.title}** (${prompt.id})\n` +
                       `      📝 "${prompt.content.substring(0, 100)}${prompt.content.length > 100 ? '...' : ''}"\n` +
                       `      🏷️ Domain: ${prompt.domain} | Tags: ${prompt.tags.join(', ') || 'None'}\n` +
                       `      ${ratingStars} Rating: ${prompt.rating.toFixed(1)}/5 (${prompt.ratings.length} reviews)\n` +
                       `      📊 Quality: ${qualityBar} ${(prompt.quality * 100).toFixed(0)}% | Usage: ${prompt.usage}\n` +
                       `      🔥 Popularity Score: ${popularityScore}/5 | Author: ${prompt.author}`;
            }).join('\n\n');

            return {
                content: [{
                    type: 'text',
                    text: `🔥 **Popular Prompts**\n\n` +
                          `📊 **Showing top ${popularPrompts.length} prompts by popularity**\n` +
                          `🎯 **Ranking based on usage frequency and user ratings**\n\n` +
                          `📋 **Top Prompts:**\n${promptsList}\n\n` +
                          `💡 **Why These Are Popular:**\n` +
                          `• High user ratings and positive feedback\n` +
                          `• Frequent usage by the community\n` +
                          `• Well-structured with clear variables\n` +
                          `• Proven effectiveness in real projects\n\n` +
                          `🔄 **Actions:**\n` +
                          `• View details: \`get_prompt_by_id\`\n` +
                          `• Rate prompts: \`rate_prompt\`\n` +
                          `• Create similar: \`create_prompt\`\n` +
                          `• Start A/B test: \`start_ab_test\``
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get popular prompts: ${error.message}`
                }]
            };
        }
    }

    async handleGetPromptsByDomain(args) {
        const { domain } = args;

        if (!domain) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing required parameter. Please provide a domain.'
                }]
            };
        }

        try {
            const prompts = this.enhancedPromptLibrary.getPromptsByDomain(domain);

            if (prompts.length === 0) {
                const availableDomains = Array.from(this.enhancedPromptLibrary.collections.keys())
                    .filter(key => this.enhancedPromptLibrary.collections.get(key).isDefault);

                return {
                    content: [{
                        type: 'text',
                        text: `📂 **No Prompts Found in Domain "${domain}"**\n\n` +
                              `💡 **Available Domains:**\n` +
                              `${availableDomains.map(d => `   • ${d}`).join('\n')}\n\n` +
                              `🔄 **Try:**\n` +
                              `• Check available domains with \`get_prompt_analytics\`\n` +
                              `• Create prompts for this domain with \`create_prompt\`\n` +
                              `• Search across all domains with \`search_prompts\``
                    }]
                };
            }

            const promptsList = prompts.map((prompt, index) => {
                const ratingStars = prompt.rating > 0 ? '⭐'.repeat(Math.round(prompt.rating)) + '☆'.repeat(5 - Math.round(prompt.rating)) : 'Not rated';
                const variables = prompt.variables.length > 0 ?
                    prompt.variables.map(v => `{${v}}`).join(', ') : 'None';

                return `   ${index + 1}. **${prompt.title}** (${prompt.id})\n` +
                       `      📝 "${prompt.content.substring(0, 100)}${prompt.content.length > 100 ? '...' : ''}"\n` +
                       `      🔖 Tags: ${prompt.tags.join(', ') || 'None'}\n` +
                       `      ${ratingStars} Rating: ${prompt.rating.toFixed(1)}/5 | Usage: ${prompt.usage}\n` +
                       `      🔧 Variables: ${variables} | Author: ${prompt.author}`;
            }).join('\n\n');

            const collection = this.enhancedPromptLibrary.getCollectionById(domain);
            const domainName = collection ? collection.name : domain;

            return {
                content: [{
                    type: 'text',
                    text: `📂 **${domainName} Prompts**\n\n` +
                          `🏷️ **Domain:** ${domain}\n` +
                          `📊 **Total Prompts:** ${prompts.length}\n` +
                          `📈 **Average Rating:** ${prompts.filter(p => p.rating > 0).length > 0 ?
                              (prompts.reduce((sum, p) => sum + p.rating, 0) / prompts.filter(p => p.rating > 0).length).toFixed(1) : 'N/A'}/5\n` +
                          `📊 **Total Usage:** ${prompts.reduce((sum, p) => sum + p.usage, 0)}\n\n` +
                          `📋 **Prompts:**\n${promptsList}\n\n` +
                          `💡 **Domain Insights:**\n` +
                          `• Specialized prompts for ${domainName.toLowerCase()}\n` +
                          `• Curated by domain experts\n` +
                          `• Optimized for specific use cases\n` +
                          `• Regular updates and improvements\n\n` +
                          `🔄 **Actions:**\n` +
                          `• View prompt details: \`get_prompt_by_id\`\n` +
                          `• Create domain-specific prompt: \`create_prompt\`\n` +
                          `• Rate prompts: \`rate_prompt\`\n` +
                          `• Search within domain: \`search_prompts\``
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get prompts by domain: ${error.message}`
                }]
            };
        }
    }

    async handleGetPromptAnalytics(args) {
        try {
            const analytics = this.enhancedPromptLibrary.getPromptAnalytics();

            const domainChart = Object.entries(analytics.domainDistribution)
                .sort(([,a], [,b]) => b - a)
                .map(([domain, count]) =>
                    `   • ${this.enhancedPromptLibrary.formatDomainName(domain)}: ${count} prompts`
                ).join('\n');

            const qualityGrade = analytics.avgQuality > 0.8 ? '🌟 Excellent' :
                               analytics.avgQuality > 0.6 ? '👍 Good' :
                               analytics.avgQuality > 0.4 ? '📊 Fair' : '📈 Needs Improvement';

            const ratingGrade = analytics.avgRating > 4.0 ? '🌟 Excellent' :
                              analytics.avgRating > 3.0 ? '👍 Good' :
                              analytics.avgRating > 2.0 ? '📊 Fair' : '📈 Needs Improvement';

            return {
                content: [{
                    type: 'text',
                    text: `📊 **Enhanced Prompt Library Analytics**\n\n` +
                          `📈 **Overview:**\n` +
                          `   • Total Prompts: ${analytics.totalPrompts}\n` +
                          `   • Total Collections: ${analytics.totalCollections}\n` +
                          `   • Total Usage: ${analytics.totalUsage}\n` +
                          `   • Community Contributions: ${analytics.communityContributions}\n\n` +
                          `📊 **Quality Metrics:**\n` +
                          `   • Average Quality: ${(analytics.avgQuality * 100).toFixed(1)}% (${qualityGrade})\n` +
                          `   • Average Rating: ${analytics.avgRating.toFixed(1)}/5 (${ratingGrade})\n` +
                          `   • Active A/B Tests: ${analytics.activeABTests}\n` +
                          `   • Pending Reviews: ${analytics.pendingContributions}\n\n` +
                          `🏷️ **Domain Distribution:**\n${domainChart}\n\n` +
                          `🧪 **Testing & Community:**\n` +
                          `   • A/B Tests Running: ${analytics.activeABTests}\n` +
                          `   • Community Submissions: ${analytics.pendingContributions} pending\n` +
                          `   • Total Contributions: ${analytics.communityContributions}\n\n` +
                          `💡 **Library Health:**\n` +
                          `${analytics.totalPrompts > 100 ? '• 🌟 Rich prompt library with diverse content\n' : '• 📈 Growing library - more prompts needed\n'}` +
                          `${analytics.avgQuality > 0.7 ? '• 🎯 High-quality prompts with good structure\n' : '• 📊 Quality improvement opportunities available\n'}` +
                          `${analytics.avgRating > 3.5 ? '• 😊 High user satisfaction with prompts\n' : '• 📝 User feedback suggests room for improvement\n'}` +
                          `${analytics.activeABTests > 0 ? '• 🧪 Active optimization through A/B testing\n' : '• 🔬 Consider starting A/B tests for optimization\n'}` +
                          `• 🤝 Community-driven development and improvement\n\n` +
                          `🎯 **Recommendations:**\n` +
                          `• Encourage more community contributions\n` +
                          `• Start A/B tests for popular prompts\n` +
                          `• Focus on underrepresented domains\n` +
                          `• Improve prompt quality through feedback`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get prompt analytics: ${error.message}`
                }]
            };
        }
    }

    setupErrorHandling() {
        this.server.onerror = (error) => {
            console.error('❌ MCP Server error:', error);
            // Don't exit on MCP errors, just log them
        };

        process.on('SIGINT', async () => {
            console.error('🛑 Shutting down ZAI MCP Server...');

            // Stop all active loops
            for (const loop of this.activeLoops.values()) {
                loop.status = 'stopped';
            }
            this.activeLoops.clear();

            try {
                await this.dataCollector.flushData();
                await this.server.close();
            } catch (error) {
                console.error('❌ Error during shutdown:', error);
            }
            process.exit(0);
        });

        process.on('uncaughtException', (error) => {
            console.error('❌ Uncaught exception:', error);
            // Only exit if it's not a buffer-related error
            if (!error.message.includes('subarray') && !error.message.includes('buffer')) {
                process.exit(1);
            }
        });

        process.on('unhandledRejection', (reason, promise) => {
            console.error('❌ Unhandled rejection at:', promise, 'reason:', reason);
            // Only exit if it's not a buffer-related error
            if (reason && !reason.toString().includes('subarray') && !reason.toString().includes('buffer')) {
                process.exit(1);
            }
        });
    }

    async start() {
        try {
            console.error('🚀 Starting ZAI MCP Server...');
            console.error('🔧 Initializing transport layer...');

            // Don't modify stdin before creating transport
            const transport = new StdioServerTransport();
            console.error('📡 Connecting to MCP transport...');

            await this.server.connect(transport);

            console.error('✅ ZAI MCP Server started successfully');
            console.error('📡 Listening for MCP messages...');
            console.error('🎯 Server ready for tool calls');

        } catch (error) {
            console.error('❌ Failed to start MCP server:', error);
            console.error('Error details:', error.stack);
            process.exit(1);
        }
    }
}

// Start the server
async function main() {
    try {
        console.error('🔧 Initializing ZAI MCP Server...');
        const server = new ZAIMCPServer();
        await server.start();
    } catch (error) {
        console.error('❌ Failed to start ZAI MCP Server:', error);
        console.error('Stack trace:', error.stack);
        process.exit(1);
    }
}

// Run if this is the main module
const isMainModule = import.meta.url === `file://${process.argv[1]}` ||
    process.argv[1]?.endsWith('src/index.js') ||
    process.argv[1]?.endsWith('zai-mcp-server') ||
    process.argv[1]?.includes('zai-mcp-server') ||
    process.argv.some(arg => arg.includes('zai-mcp-server'));

if (isMainModule) {
    main().catch((error) => {
        console.error('❌ Fatal error:', error);
        process.exit(1);
    });
}

export { ZAIMCPServer };
